/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.core;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Stack;

import org.apache.log4j.Logger;

/**
 * This class is a timer that can be used to time how long specific filters
 * take. It keeps all the timers on a stack.
 */
public class Timer {
	private String  defaultTaskName = "unknown";
	private Map     taskTimes = new Hashtable();
	private String  taskName  = defaultTaskName;
	private long    taskStart = System.currentTimeMillis();
	private Stack   taskStack = new Stack();
    // private static Logger LOG = Logger.getLogger(Timer.class);

	/**
	 * Reset the timer
	 */
	public void reset() {
		taskTimes.clear();
		taskName  = null;
		taskStart = 0;
	}
	
	public Object clone() 
	{
		Timer newTimer = new Timer();
		newTimer.defaultTaskName = this.defaultTaskName;
		newTimer.taskTimes = new Hashtable(this.taskTimes);
		newTimer.taskName  = this.taskName;
		newTimer.taskStart = this.taskStart;
		newTimer.taskStack = new Stack();
		newTimer.taskStack.addAll(this.taskStack);
		return newTimer;
	}
	
	/**
	 * Start the time for a specific task
	 * 
	 * @param newTaskName The name of the taks that needs timing
	 */
	private void start(String newTaskName) {
		if (this.taskName != null) {
			Long previousTotalTimeLong = (Long)( taskTimes.get(taskName) );
			long previousTotalTime = 0;
			if (previousTotalTimeLong != null)
				previousTotalTime = previousTotalTimeLong.longValue();
			
			long thisTaskStartTime  = taskStart;
			long thisTaskFinishTime = System.currentTimeMillis();
			long thisTaskTimeTaken  = thisTaskFinishTime - thisTaskStartTime;
			
			Long newTotalTimeLong = new Long(
					previousTotalTime + thisTaskTimeTaken);
			taskTimes.put(taskName, newTotalTimeLong);
		}
		
		this.taskName  = newTaskName; 
		this.taskStart = System.currentTimeMillis();
	}
	
	/**
	 * Returns all the timers
	 * 
	 * @return The timers
	 */
	public Map getTimes() {
		return this.taskTimes;
	}
	
	/**
	 * Push a timer on the stack
	 * 
	 * @param newTaskName The name of the task you want to time
	 */
	public void push(String newTaskName) {
		taskStack.push(newTaskName);
		start(newTaskName);
		// LOG.debug("Started "+newTaskName);
	}
	
	/**
	 * Pop a timer of the stack.
	 * 
	 * @return	The popped timer
	 */
	public Timer pop() {
		taskStack.pop();
		String newTaskName = defaultTaskName;
		if (taskStack.size() > 0)
			newTaskName = (String)( taskStack.peek() );
		// LOG.debug("Finished "+taskName+", resuming "+newTaskName);
		start(newTaskName);
		return this;
	}
	
	/**
	 * Writes the taken times to a logger.
	 * 
	 * @param logger The logger you want to write to
	 */
	public void logTimes(Logger logger) {
    	Iterator ti = taskTimes.keySet().iterator();
    	while (ti.hasNext()) {
    		String task = (String)(ti.next());
    		Long time = (Long)( taskTimes.get(task) );
    		logger.debug("Time taken in "+task+": "+time.longValue());
    	}
	}
	
	/**
	 * Returns the current task name
	 * 
	 * @return Taskname
	 */
	public String getCurrentTaskName() {
		return taskName;
	}
}
