/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.core;

import org.aidworld.aidbase.webui.util.WebUtils;
import org.apache.log4j.Logger;

/**
 *
 * This class holds global properties of the application,
 * used on various web pages.
 */
public class Version {
	
	private static Logger log = Logger.getLogger(Version.class);
	
	/**
	 * Returns the name of the application
	 * 
	 * @return application name, e.g "loband"
	 */
	public static String getName() {
		return "loband";
	}
	
	/**
	 * Returns the short name of the application
	 * 
	 * @return short name, e.g ">0-"
	 */
	public static String getShortName() {
		return ">0-";
	}
	
	/**
	 * Returns the short name of the application as html
	 * 
	 * @return short name as html , eg "&amplgt;0-"
	 */
	public static String getShortNameHtml() {
		return "&gt;0-";
	}

	/**
	 * Returns the url of the application
	 * 
	 * @return url
	 */
	public static String getWebsiteUrl() {
		String url = "";
		try {
			url= WebUtils.getApplicationUrl();
		} catch (Exception e){
			log.error("Application URL not configured correctly.");
		}
		return url;
	}
	
	private static final String tag = "$Name:  $"
		.replaceFirst("\\$Name: ", "")
		.replaceFirst(" \\$", "");
	private static final String fakeTag = (tag.equals("") ? "HEAD" : tag);
	
	/**
	 * Returns the current version of the application
	 * 
	 * @return version number/name
	 */
	public static String getVersion() {	
		//This is useful for development purposes
		/*
		TimeZone tz = TimeZone.getTimeZone("GMT+0100");
		Date time = new Date(System.currentTimeMillis());
		DateFormat df = new SimpleDateFormat( "yyyy.MM.dd HH.mm.ss");
		df.setTimeZone(tz);
		String DateTime = df.format(time);
		*/
		
		return fakeTag;
	}
	
	/**
	 * Returns the current versions color
	 * 
	 * @return Hex color
	 */
	public static String getColour() {
		return "#C0C0C0";
	}
}
