/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.error;

/**
 * Exception thrown when the aidbase output is unsatisfactory,
 * and an explanation to the user is needed instead.
 */
public class BadOutputException extends AidbaseException {

    private String m_url = null;
	private String m_displayableMessage = null;

	/**
	 * Main constructor
	 * 
	 * @param message The technical reason for the exception, to log.
	 * @param url     An alternative URL to use, for user display.
	 * @param displayableMessage A user-displayable message - html can be used.
	 */
    public BadOutputException(String message, String url, String displayableMessage) {
        super(message);
		m_url = url;
		m_displayableMessage = displayableMessage;
    }

    /**
     * Return the alternate url
     * 
     * @return alternate url
     */
    public String getAlternateURL(){
    	return m_url;
    }
    
    /**
     * Returns a string representation of the Exception
     * 
     * @return Exception as a string
     */
    public String getDisplayableMessage(){
    	return m_displayableMessage;
    }

}
