/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.error;

import java.net.URL;

/**
 * Exception thrown when the user tries to visit a blocked
 * url.
 */
public class BlockedUrlException extends AidbaseException {
    private URL url = null;

    /**
     * Create a BlockedUrlException 
     * 
     * @param message The error message
     */
    public BlockedUrlException(String message) {
        super(message);
    }

    /**
     * Create a BlockedUrlException with root cause
     * 
     * @param message	The error message
     * @param cause		The root cause of the exception
     */
    public BlockedUrlException(String message, Throwable cause) {
        super(message, cause);
    }
    
    /**
     * Allows an optional url to be set that aidbase will attempt to redirect to.
     * 
     * @param url
     */
    public void setRedirectUrl(URL url) {
        this.url = url;
    }
    
    /**
     * Returns an optional url that aidbase will attempt to redirect to.
     * 
     * @return Url
     */
    public URL getRedirectUrl() {
        return this.url;
    }

}
