/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.factory;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.aidworld.aidbase.error.AidbaseException;
import org.aidworld.aidbase.error.DocumentException;
import org.aidworld.aidbase.error.InternalFaultException;
import org.aidworld.aidbase.error.RecursionDepthException;
import org.aidworld.aidbase.request.Request;
import org.apache.commons.httpclient.Header;
import org.apache.log4j.Logger;


class FileUrlContents extends URLContents
{
	private Logger m_log = Logger.getLogger(FileUrlContents.class);

	protected FileUrlContents(URL url, Request request)
	throws RecursionDepthException, DocumentException, AidbaseException
	{
		this.request = request;
		
		// ByteArrayOutputStream byteBuffer = null;
		URLConnection urlConnection = null;
		boolean retryRequest = true;
		int retryCount = 0;
		Map requestHeaders = request.getHeaders();
		
		while (retryRequest) {
			retryRequest = false;
			retryCount++;
			if (retryCount > 10) {
				throw new RecursionDepthException(
				"infinite loop while trying to fix bad redirection");
			}
			
			try {
				urlConnection = url.openConnection();
			} catch (IOException e2) {
				throw new DocumentException("Error reading document", e2);
			}
			
			urlConnection.setRequestProperty("User-Agent","Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7) Gecko/20040803 Firefox/0.9.3");
			
			for (Iterator headerNames = 
				request.getHeaders().keySet().iterator();
				headerNames.hasNext(); )
			{
				String headerName = (String)( headerNames.next() );
				Object value = requestHeaders.get(headerName);
				if (headerName.equalsIgnoreCase("Cookie")) {
					m_log.debug("Set header "+headerName+": "+value);
					urlConnection.setRequestProperty(headerName, value.toString());
				} else {
					m_log.debug("Not sending header "+headerName+": "+value);
				}
			}
			
			m_log.debug("about to connect");
			
			try {
				this.stream = urlConnection.getInputStream();
				m_log.debug("connected");
			} catch (MalformedURLException e) {
				m_log.warn("Encountered a bad redirect, trying to fix it ("+e+")");
				
				Map headers = urlConnection.getHeaderFields();
				// LOG.debug("oldHeaders = "+oldHeaders);
				Object [] headerNames = headers.keySet().toArray();
				// LOG.debug("received "+headerNames.length+" headers");
				
				for (int i = 0; i < headerNames.length; i++) {
					// LOG.debug("received "+headerNames[i]+" header");
					String name = (String)(headerNames[i]);
					Object value = headers.get(name);
					if (name != null && value != null) {
						// LOG.debug(headerNames[i]+": ("+value.getClass()+")"+value);
						if (name.equalsIgnoreCase("Location")) {
							Collection oldValue = (Collection)value;
							String oldLocation = (String)(oldValue.iterator().next());
							URL currentUrl = urlConnection.getURL();
							
							m_log.debug("Bad redirect location was: "+oldLocation);
							m_log.debug("Probably relative to: "+currentUrl.toString());
							
							try {
								url = new URL(currentUrl, oldLocation);
								retryRequest = true;
							} catch (Exception e2) {
								throw new InternalFaultException(
										"Error fixing Location header: "+e2, e2);
							}
						}
					}
				}
			} catch (IOException e) {
				throw new DocumentException("Error reading document", e);
			}
		}
		
		this.url = urlConnection.getURL();
		Map oldHeaders = urlConnection.getHeaderFields();
		Vector newHeaders = new Vector();
		Iterator oldHeadersNames = oldHeaders.keySet().iterator();
		
		while (oldHeadersNames.hasNext()){
			String name = (String)(oldHeadersNames.next());
			String value = (String) (oldHeaders.get(name));
			Header header = new Header(name,value);
			newHeaders.add(header);
			if ( name.equalsIgnoreCase("set-cookie")){
				this.cookies.add(value);
			}
		}
		this.allHeaders = new Header[newHeaders.size()];
		newHeaders.copyInto(this.allHeaders);
	}
	
}