/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.factory;

import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.aidworld.aidbase.error.AidbaseException;
import org.aidworld.aidbase.error.DocumentException;
import org.aidworld.aidbase.error.DocumentNotFoundException;
import org.aidworld.aidbase.error.FormDataDecodeException;
import org.aidworld.aidbase.error.InternalFaultException;
import org.aidworld.aidbase.error.RecursionDepthException;
import org.aidworld.aidbase.error.ServerProtocolException;
import org.aidworld.aidbase.error.ServerTimeoutException;
import org.aidworld.aidbase.filter.AidCookie;
import org.aidworld.aidbase.request.Request;
import org.aidworld.aidbase.util.Codecs;
import org.aidworld.aidbase.util.NVPair;
import org.aidworld.aidbase.util.PartInfo;
import org.aidworld.aidbase.util.Utils;
import org.apache.commons.httpclient.Cookie;
import org.apache.commons.httpclient.Credentials;
import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpRecoverableException;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.cookie.CookiePolicy;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.log4j.Logger;


class HttpUrlContents extends URLContents {
	private HttpMethod method;
	private static Logger m_log = Logger.getLogger(HttpUrlContents.class);

    protected static void addPostParameters(
    		Request request, PostMethod post)
    throws IOException
	{
    	Map params = request.getParameters();
    	Vector nameValuePairs = new Vector();
    	String [] keys = (String[])( 
    			params.keySet().toArray(new String[params.size()]) );
    	PartInfo[] parts = request.getParts();
    	
    	//System.out.println("Size file: " + request.getFiles().size());
    	if (parts != null) 
    	{
        	NVPair[] hdrs = new NVPair[1];
        	byte[] data = Codecs.mpFormDataEncode(parts,hdrs);
        	// Utils.dumpFile("/tmp/FormData-send", data);
        	String debug = new String(data);
        	m_log.debug("File upload in form:" + debug);
        	post.setRequestBody(new ByteArrayInputStream(data));
        	post.removeRequestHeader("Content-Type");
        	post.addRequestHeader("Content-Type",hdrs[0].getValue());
        }
    	else 
    	{
           	for (int i = 0; i < keys.length; i++) {
        		String paramName  = keys[i];
        		String [] paramValues = (String[])( 
        				params.get(paramName) );
        		for (int j = 0; j < paramValues.length; j++) {
        			nameValuePairs.add(new NameValuePair(paramName, 
        					paramValues[j]));
        		}
        	}
        	
        	NameValuePair [] data = (NameValuePair[])(
        			nameValuePairs.toArray(new NameValuePair[0]) );
        	post.setRequestBody(data);
    	}
    }
    

    /**
     * Returns the contents of the URL specified, as well as the url connection used
     * to retrieve them.
     */
    
    public HttpUrlContents(URL requestedUrl, Request request)
    throws AidbaseException
    {
    	this.url = requestedUrl;
        boolean retryRequest = true;
        int retryCount = 0;
        int statusCode;
        this.request = request;
        
        Map requestHeaders = request.getHeaders();

        HttpClient client = new HttpClient();
        client.setStrictMode(true);

        client.setTimeout                     (Utils.getIoTimeoutMillis());
        client.setConnectionTimeout           (Utils.getIoTimeoutMillis());
        client.setHttpConnectionFactoryTimeout(Utils.getIoTimeoutMillis());
        
        HttpState state = request.getState();
        method = null;
        
        if (state == null) {
        	state = client.getState();
        	request.setState(state);
        	  for (Iterator headerNames = requestHeaders.keySet().iterator();
              headerNames.hasNext(); )
              {
              	String headerName = (String)( headerNames.next() );
              	Object value = requestHeaders.get(headerName);
              	// @todo this "Aidbase Cookies" header hack STINKS
              	if (headerName.equalsIgnoreCase("AidBase Cookies"))
              	{
              		Vector oldCookies = (Vector)(value);
              		for (int i = 0; i < oldCookies.size(); i++) 
              		{
              			AidCookie oldCookie = (AidCookie)(oldCookies.get(i));
              			Cookie newCookie = new Cookie(oldCookie.getDomain(),
              					oldCookie.getName(),oldCookie.getValue());
              			newCookie.setPath(oldCookie.getPath());
              			state.addCookie(newCookie);
              		}
              	} 
              }         
        } else {
        	client.setState(state);
        }
        
        state.setCookiePolicy(CookiePolicy.COMPATIBILITY);
        
        while (retryRequest) {
        	
        	retryRequest = false;
            retryCount++;
            if (retryCount > 10) {
                throw new RecursionDepthException(
                    "too many redirects: possible loop?");
            }

            try {
                statusCode = -1;
                
                if (request.getMethod() == Request.GET_METHOD) {
                	addGetParameters(request);
                	try {
                		method = new GetMethod(url.toString());
                	} catch (Throwable t) {
                		throw new InternalFaultException("Invalid URL: "+
                				url.toString(), t);
                	}
                } else if (request.getMethod() == Request.POST_METHOD) { 
                	addGetParameters(request);
                	method = new PostMethod(url.toString());
                	try {
                		addPostParameters(request, (PostMethod)method);
                	} catch (IOException e) {
                		throw new FormDataDecodeException(
                				"error reading POST data from client", e);
                	}
                } else {
                	throw new InternalFaultException("Unknown method type: "
                			+ request.getMethod());
                }
                
                method.setRequestHeader("User-Agent","Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7) Gecko/20040803 Firefox/0.9.3");
                String auth = "";
                
                for (Iterator headerNames = requestHeaders.keySet().iterator();
                headerNames.hasNext(); )
                {
                	String headerName = (String)( headerNames.next() );
                	Object value = requestHeaders.get(headerName);
                	if (headerName.equalsIgnoreCase("User-Agent")) {
                		method.setRequestHeader("User-Agent", 
                				value + " " + Math.random());
                	} else if (headerName.equalsIgnoreCase("authorization")){
                		auth = (String)( value );
                		auth = auth.substring(auth.indexOf(" ")+1);
                		auth = Codecs.base64Decode(auth);
                		String[] user = auth.split(":");
                		Credentials cred = new 
							UsernamePasswordCredentials(user[0],user[1]);
                		state.setCredentials(null,this.url.getHost(),cred);
                		method.setDoAuthentication(true);
                	}
                }
                
                method.setFollowRedirects(false);
                
                try {
                	statusCode = client.executeMethod(method);
                	this.stream = method.getResponseBodyAsStream();
                } catch (FileNotFoundException e) {
                	throw new DocumentNotFoundException(
                			"Document not found on server", e);
                } catch (HttpRecoverableException e) {
                	throw new ServerTimeoutException("Timeout accessing remote server", e);
                	
                } catch (Exception e) {
                	throw new ServerProtocolException(
                			"Remote server responded badly", e);
                }
                
                Cookie[] stateCookies = request.getState().getCookies();
                
                for (int i = 0; i < stateCookies.length;i++){
                	Cookie ck = stateCookies[i];
                	String cookie = ck.toExternalForm();
                	
                	if (ck.getDomain() != null)
                		cookie += "; domain="+ck.getDomain();
                	if (ck.getPath() != null)
                		cookie += "; path="+ck.getPath();
                	if (ck.getExpiryDate() != null)
                		cookie += "; expires="+ck.getExpiryDate();

                	m_log.debug("Cookie is: "+ cookie);
                	cookies.add(cookie);
                }
                
                if (statusCode >= 300 && statusCode < 400) {
                	Header location = method.getResponseHeader("Location");
                	this.url = new URL(location.getValue());
                	m_log.debug("Redirected to: " + url.toString());
                	statusCode = -1;
                	request.setMethod(Request.GET_METHOD);
                	request.setParameters(new Hashtable());
                }
                
                if (statusCode == 200) {
                	// success, do nothing
                } else if (statusCode >= 400 && statusCode < 600) {
                	// return the error page from the server
                } else if (statusCode == -1) {
                	retryRequest = true;
                } else {
                	throw new DocumentException(
                			"Error " + statusCode + " while retrieving document");
                }
                
            } catch (MalformedURLException e) {
                m_log.warn("Encountered a bad redirect, trying to fix it ("+e+")");
            	
                request.setMethod(Request.GET_METHOD);
            	request.setParameters(new Hashtable());
                
                Header[] headers = method.getResponseHeaders();

                for (int i = 0; i < headers.length; i++) {
                    // LOG.debug("received "+headerNames[i]+" header");
                    String name = headers[i].getName();
                    String value = headers[i].getValue();
                    if (name != null && value != null) {
                        // LOG.debug(headerNames[i]+": ("+value.getClass()+")"+value);
                        if (name.equalsIgnoreCase("Location")) {
                          
                            m_log.debug("Bad redirect location was: "+value);
                            m_log.debug("Probably relative to: "+this.url.toString());

                            try {
                                url = new URL(this.url, value);
                                
                                retryRequest = true;
                            } catch (Exception e2) {
                                throw new InternalFaultException(
                                    "Error fixing Location header: "+e2, e2);
                            }
                        }
                    }
                }
            }
        }
        
        allHeaders = method.getResponseHeaders();
    }
    
    public void cleanup() {
    	Thread releaser = new Thread(){
    		public void run(){
    			method.releaseConnection();
    		}
    	};
    	releaser.start();
        super.cleanup();
    }
}