/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.factory;

import java.util.Iterator;
import java.util.Map;

import org.aidworld.aidbase.document.Document;
import org.aidworld.aidbase.document.HtmlDocument;
import org.aidworld.aidbase.error.AidbaseException;
import org.aidworld.aidbase.request.Request;

/**
 * This factory creates Documents from URLs and Files according to some built-in
 * heuristics. For example, retrieving a document from Reliefweb, or retrieving
 * documents of content type application/x-pdf, might result in a document object
 * with a different class to the usual one (StringDocument) in future.
 */

public class TestDocumentFactory extends DocumentFactoryBase {
    private String testData;
    private Map testHeaders;
    private Map testDocuments;
    
    public TestDocumentFactory (String testData, Map testHeaders) {
        this.testData = testData;
        this.testHeaders = testHeaders;
    }

    public TestDocumentFactory (Map testDocuments, Map testHeaders) {
        this.testDocuments = testDocuments;
        this.testHeaders = testHeaders;
    }
    
    public Document service (Request request)
    throws AidbaseException {
        if (testDocuments != null) {
            String url = request.getURL().toString();
            for (Iterator keys = testDocuments.keySet().iterator(); keys.hasNext(); ) {
                String nextKey = (String)( keys.next() );
                if (nextKey.equals(url)) {
                    return (Document)( testDocuments.get(nextKey) );
                }
            }
        }
        
        // TODO this should return a Document passed in the constructor,
        // not create a new one.
        return new HtmlDocument(testData, testHeaders);
    }
}
