/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.factory;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

import org.aidworld.aidbase.error.AidbaseException;
import org.aidworld.aidbase.error.DocumentException;
import org.aidworld.aidbase.error.InternalFaultException;
import org.aidworld.aidbase.request.Request;
import org.aidworld.aidbase.webui.util.WebUtils;
import org.apache.commons.httpclient.Header;
import org.apache.log4j.Logger;


/**
 * Utility class for collecting together the url contents and the
 * URLConnection object used to retrieve them.
 */
public class URLContents {
	private Logger m_log = Logger.getLogger(URLContents.class);
	protected InputStream stream;
    // byte[] bytes;
    // HttpMethod method;
    public URL url;
    public Vector cookies = new Vector();
    public Header[] allHeaders;
    public Request request;
    /**
     * Add GET parameters from the query string to the request URL,
     * preserving order from the URL sent to Aidbase.
     * They are removed from the request, leaving the parameters
     * which should be POSTed.
     */
    protected void addGetParameters(Request request) 
    throws AidbaseException {
        
        Map requestParams = new Hashtable(request.getParameters());
        String[] keys;
        String urlString = this.url.toString();

        keys = request.getOrderedParamNames();

        for (int k = 0; k < keys.length; k++) 
        {
            String key = keys[k];
            if ( key.startsWith(WebUtils.getAidbaseParamName())){
            	continue;
            }
            // map values are an array of strings
            String[] values = (String[])(requestParams.get(key));
            requestParams.remove(key);

            if (values == null)
                continue;
            
            for (int i = 0; i < values.length; i++) {
                try {
                    if (urlString.indexOf("?") != -1) {
                        urlString += "&";
                    } else {
                        urlString += "?";
                    }
                    
                    urlString += URLEncoder.encode(key, "UTF-8");
                    urlString += "=";
                    urlString += URLEncoder.encode(values[i], "UTF-8");

                } catch (UnsupportedEncodingException e) {
                    throw new InternalFaultException(
                    		"unsupported encoding: "+e, e); 
                }
            }
        }

        try 
		{
            if (m_log.isDebugEnabled())
            	m_log.debug("Old URL = "+request.getURL());

            this.url = new URL(urlString);
            request.setURL(url);

            if (m_log.isDebugEnabled())
            	m_log.debug("New URL = "+request.getURL());
        } 
        catch (MalformedURLException e) 
		{
            throw new InternalFaultException("made an invalid URL: "+e, e);
        }
        
        request.setParameters(requestParams);
    }

    public void cleanup() { }
    
    protected byte[] getBytes() throws DocumentException {
		ByteArrayOutputStream byteBuffer = new ByteArrayOutputStream();
		
		try {
			byte[] buffer = new byte[4096];

			while (true) {
				m_log.debug("reading block");
				int bytesRead = stream.read(buffer);
				if (bytesRead == -1) break;
				m_log.debug("writing block: "+bytesRead+" bytes");
				byteBuffer.write(buffer, 0, bytesRead);
			}
		} catch (IOException e) {
			throw new DocumentException("Error reading document", e);
		}

		cleanup();
		return byteBuffer.toByteArray();
    }
    
    public String getContentTypeHeader() {
    	if (allHeaders == null)
    		return null;
    	
    	for (int i = 0; i < allHeaders.length; i++) {
    		Header h = allHeaders[i];
    		if (h.getName().equalsIgnoreCase("content-type"))
    			return h.getValue();
    	}
    	return null;
    }
}