/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.filter;

import org.aidworld.aidbase.document.Document;
import org.aidworld.aidbase.document.HtmlDocument;
import org.aidworld.aidbase.error.AidbaseException;
import org.aidworld.aidbase.error.BadOutputException;
import org.aidworld.aidbase.request.Request;
import org.apache.log4j.Logger;

/**
 * This filter makes a last ditch attempt to display a sensible message
 * to the user, if the output from Aidbase is detected to be useless,
 * for example if the output is empty due to the page being
 * made entirely of javascript.
 */

public class BadOutputFilter extends BasicFilter {

    private static Logger LOG = Logger.getLogger(BadOutputFilter.class);

    /**
     * This method will check if the document is empty
     * 
     * @param request The request of the user
     * @param server  The server of the requested url
     * @return The html document 
     */
    public Document service(Request request) throws AidbaseException {

		// Do all other things first
		Document document = super.service(request);

        if (LOG.isDebugEnabled()) LOG.debug("BadOutputFilter starting");
        
		// If the output is empty, throw a BadOutputException
		// with an explarationaly note. 
        if (document instanceof HtmlDocument){
			HtmlDocument htmlDocument = (HtmlDocument)document;
			if (htmlDocument.getHtmlBodyContents().trim().equals("")){
				// I18N 3015
				String displayMessage = "This page has a high reliance " +
						"on JavaScript, making it difficult to extract " +
						"the meaningful contents.";
				String alturl = request.getURL().toString();
				throw new BadOutputException("Empty filtered output", 
						alturl, displayMessage);
			}
        }
        
		// if (LOG.isDebugEnabled()) LOG.debug("Sample output = '"+(document.toString().length()<20? document.toString():document.toString().substring(15))+"'");
        
		// All is fine - return the filtered document
        if (LOG.isDebugEnabled()) LOG.debug("BadOutputFilter finished");
        return document;
    }
}
