/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter;

import java.util.LinkedList;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.opensymphony.module.sitemesh.html.HTMLProcessorContext;
import com.opensymphony.module.sitemesh.html.State;
import com.opensymphony.module.sitemesh.html.Tag;
import com.opensymphony.module.sitemesh.html.TagRule;
import com.opensymphony.module.sitemesh.html.Text;
import com.opensymphony.module.sitemesh.html.tokenizer.TagTokenizer;
import com.opensymphony.module.sitemesh.html.tokenizer.TokenHandler;
import com.opensymphony.module.sitemesh.html.util.CharArray;

public class HTMLProcessor 
{
    private final char[] input;
    private final CharArray body;
    private Properties params;
    private final State defaultState = new State();
    private static final Logger LOG = Logger.getLogger(HTMLProcessor.class);

    private State currentState = defaultState;

    public HTMLProcessor(char[] input, CharArray body, Properties params) {
        this.input = input;
        this.body = body;
        this.params = params;
    }

    public State defaultState() {
        return defaultState;
    }

    public void process() {
        TagTokenizer tokenizer = new TagTokenizer(input);
        final HTMLProcessorContext context = new HTMLProcessorContext() {
            public State currentState() {
                return currentState;
            }

            public void changeState(State newState) {
                currentState = newState;
            }

            private LinkedList bufferStack = new LinkedList();

            public void pushBuffer(CharArray buffer) {
                bufferStack.add(buffer);
            }

            public CharArray currentBuffer() {
                return (CharArray) bufferStack.getLast();
            }

            public CharArray popBuffer() {
                return (CharArray) bufferStack.removeLast();
            }

            public void mergeBuffer() {
                CharArray top = (CharArray) bufferStack.getLast();
                CharArray nextDown = (CharArray) bufferStack.get(bufferStack.size() - 2);
                nextDown.append(top);
            }
        };
        context.pushBuffer(body);
        tokenizer.start(new TokenHandler() {

            public boolean shouldProcessTag(String name) {
                return currentState.shouldProcessTag(name);
            }

            public void tag(Tag tag) {
                TagRule tagRule = currentState.getRule(tag.getName());
                tagRule.setContext(context);
                tagRule.process(tag);
            }

            public void text(Text text) {
            	if(!params.get("script.tag").equals("0")){
            		text.writeTo(context.currentBuffer());
            	}
            	else{
//            		if (params.get("bodyProcessed").equals("false"))
//                		params.put("bodyProcessed", "error");
            		String strippedComment = text.getContents();
            		strippedComment = strippedComment.replaceAll("(<!--.*)(-->)","");
            		//strippedComment = strippedComment.replaceAll("(<!\\[.*)(\\]>)","");
            		strippedComment = strippedComment.replaceAll("(<\\?.*)(\\?>)","");
            		context.currentBuffer().append(strippedComment);

            	}
            }

            public void warning(String message, int line, int column) 
            {
            	LOG.warn(message+" at input line "+line+", column "+column);
            }
        });
        // LOG.debug(context == null);
//        defaultState.endOfState();
        defaultState.endOfState(context);
    }
}
