/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter;

import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.aidworld.aidbase.webui.util.WebUtils;
import org.apache.log4j.Logger;

/**
 * This class will try to rewrite javascript so that all 
 * redirections done by javascript still go through Aidbase. 
 * Also does some specific changes for special sites. 
 */
public class JavaScriptRewriter 
{
    private static Logger LOG = Logger.getLogger(JavaScriptRewriter.class); 
	/**
	 * This method will rewrite the javascript for specials sites and
	 * handles forms.
	 * 
	 * @param value		The JavaScript
	 * @param params	General parameters	
	 * @return	The rewritten Javascript
	 */
	public static String rewrite(String value, Properties params){
		boolean write = true;
		
		String nofilter = (String) (params.get("aidbase.filter.disable"));
		boolean filtering=true;
		if(nofilter!=null){filtering=false;}
		String url = (String) (params.get("original.url.full"));
			
		String special = (String) (params.get("special"));
		if (special != null && special.equals("hotmail")){
			value = value.replaceAll("document.doattach\\[0\\].name=\\'Attach.x\\';",
			"document.doattach[1].name='Attach.x';");
		}
		
    	// download.com and part of hotmail need to keep their document.write
		if(special != null && ((special.equals("hotmail") &&
				url.indexOf(".hotmail.msn.com/cgi-bin/AddressPicker")>-1)
				|| special == "download.com")){
			
			write = false;
		}
			
		if(!filtering)
		{
			write = false;
		}
		
		if(special != null && special.equals("openknowledge.net") &&	
				// trying to make openknowledge.net work
				url.indexOf("menu_com.js") > -1){
			write = false;
		}
		
		Pattern ourpattern = Pattern.compile("forms\\[[0-9]+\\]");
		Matcher m = ourpattern.matcher(value);
		StringBuffer sb = new StringBuffer();
		while (m.find())
		{
			int i =Integer.parseInt(m.group().substring(6,m.group().length()-1)) +1;
			m.appendReplacement(sb, "forms["+i+"]");	
		}
		sb = m.appendTail(sb);
		value = sb.toString();
		
		Pattern pattern = Pattern.compile("\\.elements\\[[0-9]+\\]");
		sb.setLength(0);
		m = pattern.matcher(value);
		while (m.find())
		{
			int i =Integer.parseInt(m.group().substring(10,m.group().length()-1)) +1;
			m.appendReplacement(sb, ".elements["+i+"]");	
		}
		
		sb = m.appendTail(sb);
		value = sb.toString();
		
		if (write) {
			value = emptyParenthesis(value, params);
		}
		
	    value = rewriteLink(value, params);
	    if(params.get("aidbase.filter.disable")==null)
		{		
	    	value = removeComments(value, params);
	    	value = removeVisibilityHidden(value, params);
		}
		
	    if(special != null) {
	    	// a bug accurs when doing a search in google and then clicking the link Images
	    	// this is because google uses some javascript which loband messes up
	    	// the next few lines solve this problem
	    	String filtstr="_ab_nofilter&";
	    	if(filtering){filtstr="";} 
	    	if(special.equals("google")){
	    		String functionQS = "function qs(el) {if (window.RegExp && window.encodeURIComponent) {var qe=encodeURIComponent(document.gs.q.value);if (el.href.indexOf(\"q=\")!=-1) {el.href=el.href.replace(new RegExp(\"q=[^&$]*\"),\"q=\"+qe);} else {el.href+=\"&q=\"+qe;}}return 1;}";
	    		int findString = value.indexOf(functionQS);
	    		if(findString != -1)
	    		{
	    			value = value.substring(0, findString)
						+ "function qs(el) {if (window.RegExp && window.encodeURIComponent) {var qe=encodeURIComponent(document.gs.q.value);if (el.href.indexOf(\"q%3D\")!=-1) {el.href=el.href.replace(new RegExp(\"q=[^&$]*\"),\"q=\"+qe);} else {el.href+=\"&q=\"+qe;}}return 1;}" 
						+ value.substring(findString + functionQS.length());
	    		}
	    	} else if (special.equals("yahoo_mail")){
	    		value = value.replaceAll("form name=sf1.*id=sf1>", 
	    				"form name=sf1 action='/loband/page'><input type='hidden'" +
						" name='_ab_url' value='http://uk.search.yahoo.com/search' id='sf1'>");
	    	} else if(special != null && special.equals("reliefweb")){
	    		try{
	    			value = value.replaceAll("var url=unescape\\(window.location.href\\);",
	    					"var url=" +
							"unescape\\(document.forms\\[0\\]._ab_url.value\\)+'http://new.reliefweb.int';");

                    value = value.replaceAll("window\\.location= url_base\\.substring\\(",
	    					"window.location='"+ WebUtils.getContextPath() + "/page?" + filtstr + 
							"_ab_url='+url_base.substring(");
				
	    			value = value.replaceAll("window\\.location=baseLink",
	    					"window.location='"+ WebUtils.getContextPath() + "/page?"+filtstr+"_ab_url='+baseLink");

                    value = value.replaceAll("window\\.location=valueLink",
	    					"window.location='"+ WebUtils.getContextPath() + "/page?"+filtstr+"_ab_url='+valueLink");
				
	    			//Makes log in and out work.
	    			value = value.replaceAll("href=\\\\\"MyRW",
	    					"href=\\\\\""+WebUtils.getContextPath() + "/page?"+filtstr+"_ab_url=http://" +
							"www.reliefweb.int/rw/MyRW");

                    // Other reliefweb problems are taken care of in
                    // AttributeFilterRule.processAttribute(Tag tag, String name, String value)

	    		}catch(Exception e){}
	    	} else if(special.equals("bbc")){
                // bbc.co.uk rewrites some of its links, filtering the page through forces all links to
                // be rewritten and break them. The next few lines fixes this.
                if(value.indexOf("LinkTrack.prototype.updateHrefs") != -1)
                    value = value.replaceAll("hostname\\.indexOf\\('\\.bbc\\.co\\.uk'\\)",
                            "currlink.href.indexOf('bbc.co.uk') == -1 && " +
                            "currlink.href != '" + WebUtils.getContextPath() + "' && " +
                            "currlink.href.indexOf('/uk/co/bbc')");
            }
	    	
		}
		return value;
	}
	
	/**
	 * This method removes all comments written with // and /**
	 * @param value
	 * @param params
	 * @return
	 * TODO: finish writing this method
	 */
	public static String removeComments(String value, Properties params)
	{
		// remove comments indicated by '/** */'
		
		value = value.replaceAll("(?s)\\s*/\\*.*?\\*/",""); 
		
		// the remaining of the function removes comments indicated by '//'
		// while making sure that we do not remove anything inside quotes 
		// (double slash can appear between quotes such as in 
		// "http://www.loband.org" but that is not a comment)
		StringBuffer newValue = new StringBuffer();
		String testString = "";
		int indexBackSlashN = 0, indexBackSlashR = 0, indexNewLine = 0;

		int index = 0;
		boolean insideAQuote = false;
		boolean evenNumberOfBackslash = true;
		boolean insideComment = false;
		char typeOfQuote = '"';
		char currentChar;
		
		
		while(index < value.length()){
			currentChar = value.charAt(index++);
			if(insideComment){
				if(currentChar == '\n' || currentChar == '\r'){
					insideComment = false;
					newValue.append("\n");
				}
				continue;
			}
			if(insideAQuote){
				if(currentChar == typeOfQuote && evenNumberOfBackslash){
					insideAQuote = false;
				}
				if(currentChar == '\\'){
					evenNumberOfBackslash = !evenNumberOfBackslash;
				}
			} else {
				if(currentChar == '"' || currentChar == '\''){
					typeOfQuote = currentChar;
					insideAQuote = true;
				} else {
					if(currentChar == '/' && index < value.length() && value.charAt(index) == '/')
					{
						indexBackSlashN = value.indexOf("\n",index);
						indexBackSlashR = value.indexOf("\r", index);
						indexNewLine = -1;
						if(indexBackSlashN != -1)
						{
							indexNewLine = indexBackSlashN;
						}
						if(indexBackSlashR != -1 && indexBackSlashR < indexBackSlashN)
						{
							indexNewLine = indexBackSlashR;
						}
						if(indexNewLine == -1)
						{
							testString = value.substring(index);
						} else {
							testString = value.substring(index, indexNewLine);
						}
						if(testString.indexOf("-->") == -1)
						{
							insideComment = true;
							continue;
						}
					}
				}
			}
			newValue.append(currentChar);
		}
		
		return newValue.toString();
	}
	
	/**
	 * Removes all javascript command concerning visibility of
	 * objects.
	 * @param value
	 * @param params
	 * @return
	 */
	public static String removeVisibilityHidden(String value, Properties params)
	{
		return value.replaceAll("[A-Za-z]+\\.[vV]isibility\\s*=\\s*['\"][\\w]*['\"]"
				
				, "");
	}
	
	/**
	 * Removes everything that is between two parenthesis after a .write
	 * @param value
	 * @param params
	 * @return
	 */
	public static String emptyParenthesis(String value, Properties params)
	{
		String newValue = "";
		Pattern pat = Pattern.compile("\\.write[^\\(]*\\(");
		Matcher mat = pat.matcher(value);
		
		char aChar;
		int aCounter = 0;
		boolean aQuoteIsOpened = false;
		char typeOfQuote = '\'';
		int i = 0;
		
		while(mat.find())
		{
			if(mat.start() < i)
				continue;
			
			newValue += value.substring(i, mat.end());
			aQuoteIsOpened = false;
			
			for(i = mat.end(); i < value.length(); i++) {
				aChar = value.charAt(i);
				if(!aQuoteIsOpened && (aChar == '\'' || aChar == '\"'))
				{
					typeOfQuote = aChar;
				}
				if(aChar == typeOfQuote)
				{
					aCounter = 0;
					while(i-aCounter > 0 && value.charAt(i-aCounter - 1) == '\\'){
						aCounter++;
					}
					if(aCounter % 2 == 0){
						aQuoteIsOpened = !aQuoteIsOpened;
					}
				}
				if(aChar == ')' && !aQuoteIsOpened){
					break;
				}
			}
		}
		newValue += value.substring(i, value.length());
		return newValue;
	}
	
	/**
	 * This method will try to rewrite all the javascript redirections
	 * so that they go through Aidbase. It does not rewrite secured link though.
	 * 
	 * @param value		The JavaScript
	 * @param params 	General parameters	
	 * @return	The rewritten Javascript
	 */
	public static String rewriteLink(String value, Properties params){
		// Some javascript has quotation marks that aren't matched, which
		// causes trouble in the code in this function.  So if the code
		// causes an exception anywhere in here we will ignore the javascript 
		// and return an empty string - ""
		try {
			Pattern pat = Pattern.compile("(?s)location\\.href\\s*=\\s*(['\"])\\s*([^/&&[^\\s]])");
			Matcher mat = pat.matcher(value);
			String newValue = "";
			int previousFind = 0;
			
			// first, we replace all relative addresses by absolute ones...
			
			while (mat.find())
			{
				if (value.length() - mat.end() < 3 || 
						!value.substring(mat.end()-1, mat.end() + 3).equals("http")) 
				{
					newValue += value.substring(previousFind,mat.start()) 
						+ "location.href = "
						+ value.substring(mat.start(1),mat.start(1)+1)
						+ params.get("original.host")
						+ params.get("base.url.path") + "/";
					previousFind = mat.end()-1;
				}
			}
			value = newValue + value.substring(previousFind);
			
			value = value.replaceAll("(?s)location\\.href\\s*=\\s*(['\"])\\s*/", 
					"location.href = $1" 
					+ params.get("original.host")
					+ "/");
			
			pat = Pattern.compile("(?s)location\\s*=\\s*(['\"])\\s*([^/&&[^\\s]])");
			mat = pat.matcher(value);
			newValue = "";
			previousFind = 0;
			
			while (mat.find())
			{
				if (value.length() - mat.end() < 3 || 
						!value.substring(mat.end()-1, mat.end() + 3).equals("http")) 
				{
					newValue += value.substring(previousFind,mat.start()) 
						+ "location = "
						+ value.substring(mat.start(1),mat.start(1)+1)
						+ params.get("original.host")
						+ params.get("base.url.path") + "/";
					previousFind = mat.end()-1;
				}
			}
			value = newValue + value.substring(previousFind);
			
			value = value.replaceAll("(?s)location\\s*=\\s*(['\"])\\s*/", 
					"location = $1" 
					+ params.get("original.host")
					+ "/");
				
			value = value.replaceAll("(?s)\\.action\\s*=\\s*(['\"])\\s*([^/&&[^\\s]])",
					".action = $1"
					+ params.get("original.host") 
					+ params.get("base.url.path") + "/$2");
			
			value = value.replaceAll("(?s)\\.action\\s*=\\s*(['\"])\\s*/",
					".action = $1" 
					+ params.get("original.host")
					+ "/");
			
			value = value.replaceAll("(?s)window.location.replace\\((['\"])\\s*/",
					"window.location.replace($1" 
					+ params.get("original.host")	
					+ "/");		
			
	
			value = value.replaceAll("(?s)window.open\\((['\"])\\s*([^/&&[\\s]])",
					"window.open($1" 
					+ params.get("original.host") 
					+ params.get("base.url.path") + "/$2");
		
			value = value.replaceAll("(?s)window.open\\((['\"])\\s*/",
					"window.open($1" 
					+ params.get("original.host") 
					+ "/");	
			
			// ...then we change the absolute addresses by filtering them through loband
			
			pat = Pattern.compile("http:/",Pattern.CASE_INSENSITIVE);
			mat = pat.matcher(value);
			newValue = "";
			previousFind = 0;
			char quote;
			int endQuote;
			boolean hasQuestionMark;
			String url;
			
			
			// this part will bug if there are spaces around the address between the quotes, such
			// as in " http://www.lemonde.fr" or '   http://www.liberation.fr'
			while(mat.find())
			{
				quote = value.charAt(mat.start() - 1);		// here, I assume that there is no
															// annoying space between the quote
															// and the http
				if(quote != '\'' && quote != '"')
				{
					LOG.warn("Error while rewriting link in javascript.");
					continue;
				}
				endQuote = value.indexOf(quote, mat.end());
				if (mat.start() < 0)
					System.out.println("mat.start() is " + mat.start());
				url = value.substring(mat.start() + 7, endQuote);	// the url with 'http:/' removed at the beginning
				url= WebUtils.convertURLToLobandFormat(url);
				url="/"+url;
				hasQuestionMark = (url.indexOf("?") != -1);
				if(params.get("aidbase.filter.disable")!=null)
				{
					url = params.get("aidbase.servlet.goLoband.fullPath") + url;
					if(hasQuestionMark)
						url += "&_ab_nofilter";
					else
						url += "?_ab_nofilter";
				}
				else
				{
					url = params.get("aidbase.servlet.goLoband.fullPath") + url;
				}
				newValue += value.substring(previousFind, mat.start());
				newValue += url + value.charAt(endQuote);
				previousFind = endQuote + 1;
			}
			newValue += value.substring(previousFind);
			value = newValue;
	
	//		if(params.get("aidbase.filter.disable")!=null)
	//		{
	//			value = value.replaceAll("http:", params.get("aidbase.application.url") +
	//				"/page?&_ab_nofilter&titi=titi&_ab_url=http:");
	//		}
	//		else
	//		{
	//			value = value.replaceAll("http:", params.get("aidbase.application.url") +
	//			"/page?titi=tito&_ab_url=http:");
	//		}
		}
		catch (Exception e) {
			LOG.error("Exception while trying to rewrite links in javascript: " + e);
			value = new String("");
		}
		return value;
	}	
}