/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.filter;

import org.aidworld.aidbase.document.Document;
import org.aidworld.aidbase.error.AidbaseException;
import org.aidworld.aidbase.error.IncompatibleDocumentException;
import org.aidworld.aidbase.request.Request;
import org.apache.log4j.Logger;

/**
 * This filter rejects Non-Html document by checking ContentType.
 * Based on RedirectFilter.java 
 */

public class NonHtmlFileFilter extends BasicFilter {

    /* (non-Javadoc)
     * @see org.aidworld.aidbase.core.Filter#transform(org.aidworld.aidbase.core.Document, org.aidworld.aidbase.core.Request)
     */
    /** For details on the Meta Refresh header which this Filter implements,
     * @see http://www.htmlhelp.com/reference/html40/head/meta.html
     * @see http://www.htmlhelp.com/reference/wilbur/head/meta.html
     * .
     */

    private static Logger LOG = Logger.getLogger(NonHtmlFileFilter.class);

    public Document service(Request request)
    throws AidbaseException
    {
        if (LOG.isDebugEnabled()) LOG.debug("NonHtmlFileFilter starting");
        
        // temporary solution
        if (request.getURL() != null){

            // get filename, if any
            String reqURL = request.getURL().toString();
            String filename = reqURL.replaceFirst(".*/","");
            
            // try to find an extension in the filename
            int pos = filename.lastIndexOf('.');
            if (pos != -1) {
                String extension = filename.substring(pos);

                if (extension.equalsIgnoreCase(".pdf")
                    || extension.equalsIgnoreCase(".gif")
                    || extension.equalsIgnoreCase(".jpg")
                    || extension.equalsIgnoreCase(".doc")) 
                {
                    if (LOG.isDebugEnabled()) 
                    	LOG.debug("NonHtmlFileFilter: Extension is " 
                    			+ extension);
                    throw new IncompatibleDocumentException(
                    		// I18N 3016
                    		"This is not an Html file. " +
                    		"Only Html files are supported at this time.");
                }
            }
        }
  /*
		HttpServletRequest req = (HttpServletRequest)request.getRequest();
		// debug: all HttServletRequest return null or getContentType is not set?
		//if (req == null) return document;
		//if (req.getContentType() == null) return document;   <- return NULL!!!

		if(req.getContentType() != null && req.getContentType().equals("text/html")) {
			if (LOG.isDebugEnabled()) LOG.debug("Debug: document is  text/html");
        } else {
            if (LOG.isDebugEnabled()) LOG.debug("NonHtmlFileFilter: ContentType is "+req.getContentType());
             throw new IncompatibleDocumentException("This is not an Html file. Only Html files are supported at this time.");	
        }
*/
        
        Document document = super.service(request);

        if (LOG.isDebugEnabled()) LOG.debug("NonHtmlFileFilter finished");
        return document;
    }

}
