/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.filter;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.aidworld.aidbase.core.Version;
import org.aidworld.aidbase.document.Document;
import org.aidworld.aidbase.document.HtmlDocument;
import org.aidworld.aidbase.error.AidbaseException;
import org.aidworld.aidbase.error.InternalFaultException;
import org.aidworld.aidbase.filter.rulefilter.BaseFormRule;
import org.aidworld.aidbase.filter.rulefilter.BaseRemovingRule;
import org.aidworld.aidbase.filter.rulefilter.BodyRule;
import org.aidworld.aidbase.filter.rulefilter.ChangeSubmitImageToButtonRule;
import org.aidworld.aidbase.filter.rulefilter.FilterList;
import org.aidworld.aidbase.filter.rulefilter.FormActionRewritingRule;
import org.aidworld.aidbase.filter.rulefilter.FrameRule;
import org.aidworld.aidbase.filter.rulefilter.GeneralTagRule;
import org.aidworld.aidbase.filter.rulefilter.HrefRewritingRule;
import org.aidworld.aidbase.filter.rulefilter.IfFormParticipatingRule;
import org.aidworld.aidbase.filter.rulefilter.ImageMapRule;
import org.aidworld.aidbase.filter.rulefilter.LinkFilterRule;
import org.aidworld.aidbase.filter.rulefilter.RemoveTagAndContentsRule;
import org.aidworld.aidbase.filter.rulefilter.ReplaceImageWithAltTextRule;
import org.aidworld.aidbase.filter.rulefilter.ScriptRule;
import org.aidworld.aidbase.filter.rulefilter.SrcRewritingRule;
import org.aidworld.aidbase.filter.rulefilter.SubstituteTagWithText;
import org.aidworld.aidbase.filter.rulefilter.TitleRule;
import org.aidworld.aidbase.request.Request;
import org.aidworld.aidbase.util.Utils;
import org.aidworld.aidbase.webui.simulator.ThrottleFilter;
import org.aidworld.aidbase.webui.util.WebUtils;
import org.apache.log4j.Logger;

import com.opensymphony.module.sitemesh.html.State;
import com.opensymphony.module.sitemesh.html.util.CharArray;

/**
 * This Filter uses the SiteMesh parser to filter HTML documents,
 * removing extraneous formatting, and rewriting links.
 */
public class RuleFilter extends BasicFilter {
	
	private static final Logger LOG = Logger.getLogger(RuleFilter.class);

	public Properties warningParams;
	
	/**
	 * This method will add all the rules to the parser
	 * 
	 * @param request	The user's request
	 * @param server	The server of the requested URL
	 * @return The filtered document
	 */
    public Document service(Request request)
    throws AidbaseException 
	{    
    	String special = "";
    	Document document = super.service(request);
        String html = document.toString();
        //Corrects <!----> and <!-- blah --->
        html = html.replaceAll("(?s)<!--(.*?)-->","<!--$1 -->");
        //Corrects <tag attrib="blah"">
        html = html.replaceAll("(?s)([^=\\s]\\s*)\"\"\\s*>","$1\">");
        html = RuleFilter.addScriptComments(html);
        boolean filtering = true;
        Map imageMap = new HashMap();
        
        CharArray output = new CharArray(1024);
        CharArray dummy = new CharArray(1024);
        Properties params = setUpParams(request);

        if(params.get("aidbase.filter.disable")!=null)
        {
        	filtering=false;
        }

        special = (String) params.get("special");
        HTMLProcessor imageMapProcessor = 
        	new HTMLProcessor(html.toCharArray(), dummy, params);
        State imageMapState = imageMapProcessor.defaultState();
        
        imageMapState.addRule(new ImageMapRule(params,imageMap, special));
        
        imageMapProcessor.process();
        
        HTMLProcessor htmlProcessor = 
        	new HTMLProcessor(html.toCharArray(), output, params);
        
        State defaultState = htmlProcessor.defaultState();
        if(filtering) defaultState.addRule(new TitleRule(params,special));

        BodyRule bodyRule = new BodyRule(params, special);

        defaultState.addRule(bodyRule);       
        defaultState.addListener(bodyRule);
        
        SubstituteTagWithText substituteRule = 
        			new SubstituteTagWithText("embed", "Embedded Object",
        					new String[] {"src"}, params);
        
        if(filtering) {
        	defaultState.addRule(substituteRule);
        	defaultState.addListener(substituteRule);
        }
        
        defaultState.addRule(new FrameRule(params, special));
        
        if(!filtering || special.length()>0)
        {
        	// replaces stylesheets with special loband 
        	// version for certain sites. This will overide the 
        	//removeTagAndContents rule below.
        	defaultState.addRule(new LinkFilterRule(params, special));
        	// for other sites, link tags will be removed by 
        	// removeTagAndContents rule below.
        }
        
        
        String[] tagArray = new String[FilterList.TAGS_REMOVE.length + 
									   FilterList.TAGS_STRIP.length];
	    
        System.arraycopy(FilterList.TAGS_REMOVE, 0, tagArray, 
	    			0, FilterList.TAGS_REMOVE.length);
	    
	    System.arraycopy(FilterList.TAGS_STRIP, 0, 
	    			tagArray, FilterList.TAGS_REMOVE.length, 
					FilterList.TAGS_STRIP.length);

        defaultState.addRule(new ScriptRule(params, special));
        
        if(filtering)
        {
        	defaultState.addRule(
        		new GeneralTagRule(tagArray, params, special));
        	defaultState.addRule(
        		new RemoveTagAndContentsRule(
        				new String[] {"style", "if", "endif","map", "link"},params));
        	defaultState.addRule(
        		new SubstituteTagWithText("applet", "Applet", 
        				new String[] {"alt", "title"}, params));
        	defaultState.addRule(
        		new SubstituteTagWithText("object", "Object", 
        				new String[] {"alt", "type"}, params));
        	defaultState.addRule(
        		new ReplaceImageWithAltTextRule(
        				params, imageMap, special));
        }
        defaultState.addRule(new HrefRewritingRule(params, special));
        defaultState.addRule(new BaseRemovingRule(params, special));
        defaultState.addRule(new SrcRewritingRule(params, special));
        if(filtering)
        {
        	IfFormParticipatingRule[] formRules = new IfFormParticipatingRule[] {
    			new FormActionRewritingRule(params, special) , 
				new ChangeSubmitImageToButtonRule(params, special)
        	};
        	defaultState.addRule(new BaseFormRule(formRules));
        }
        else
        {
        	IfFormParticipatingRule[] formRules = new IfFormParticipatingRule[] {
        			new FormActionRewritingRule(params, special)     			
           	};
        	defaultState.addRule(new BaseFormRule(formRules));
        }
        

        htmlProcessor.process();

        String newHtml = output.toString();
        return new HtmlDocument(newHtml, document.getAttributes());
    }

    /**
     * This method will set up the general properties object
     * which is needed inside the rules
     * 
     * @param request	The user's request
     * @return The general properties object
     */
	public static Properties setUpParams(Request request) throws AidbaseException {
        Properties params = new Properties();
        String url = (String) ( params.get("original.host") );
        params.put("pageMightDisplayBadly","false");
        params.put("special","");
        params.put("javascript","");
        params.put("linkcounter","0");
        params.put("framecounter","0");
        params.put("original.a.href","");
        params.put("link.to.image", "");
        params.put("image.in.anchor", "false");
        params.put("image.in.anchor.description", "");
        params.put("aidbase.paramname.simulatorbps", 
        		ThrottleFilter.BANDWIDTH_PARAM);
        params.put("aidbase.paramname.simulatorkey", 
        		ThrottleFilter.THROTTLER_KEY_URL_PARAM);
        params.put("aidbase.simulatorOnOff",
        		String.valueOf(request.getSimulatorOnOff()));
        params.put("aidbase.simulatorbps", 
        		String.valueOf(request.getSimulatorBps()));       
        params.put("aidbase.simulatorkey", 
        		String.valueOf(request.getSimulatorKey()));
        if(request.getFiltering() == false)
        {
        	params.put("aidbase.filter.disable", "true");
        }
        params.put("aidbase.version",   Version.getVersion());
        params.put("aidbase.colour",    Version.getColour());

        try {
	        // http://localhost:8060/loband
	        params.put("aidbase.application.url", WebUtils.getApplicationUrl());
	        params.put("aidbase.application.url.ssl", 
	        		WebUtils.getApplicationUrlSSL());
        } catch (IOException e) {
        	throw new InternalFaultException("Failed to load properties file", e);
        }
        // _ab_url
        params.put("aidbase.param.url", WebUtils.getAidbaseUrlParam());
        
        // _ab_type
        params.put("aidbase.param.type", WebUtils.getAidbasePipelineParam());
        
        // _ab_noheader
        params.put("aidbase.param.noheader", WebUtils.getAidbaseParamName() +
        		"noheader");
        
        // _ab_type=binary
        params.put("aidbase.param.type.binary", WebUtils.getAidbasePipelineParam()
                + "=" + WebUtils.getAidbaseBinaryType());
        
        // _ab_type=script
        params.put("aidbase.param.type.script", WebUtils.getAidbasePipelineParam()
                + "=" + WebUtils.getAidbaseScriptType());
        
        String displayLinkToImage = "true";
        try {
        	Properties props = Utils.getAppProps();
        	displayLinkToImage = props.getProperty(
        			"application.showimages", "true");
        } catch (Exception e) {
        	LOG.warn("Couldn't read from properties file: " + e);
        }
        
        params.put("display.link.to.image", displayLinkToImage);
        
    	params.put("bodyProcessed", "false");
    	params.put("htmlOpened","false");
    	params.put("headOpened","false");
    	params.put("headClosed","false");
    	params.put("titleProcessed","false");
    	   	
    	params.put("aidbase.header.enable",
    			(request.getParameter("_ab_noheader") != null) ? "0" : "1");
    	
    	// /loband/page
    	params.put("aidbase.servlet.filter.path",   
        		WebUtils.getContextPath() + WebUtils.getRealFilterServletPath());
    	
    	// /loband/goLoband
    	params.put("aidbase.servlet.goLoband.path",
    			WebUtils.getContextPath() + WebUtils.getRealFilterServletPath());
    	
    	// /loband/goLoband
    	try
		{
    		params.put("aidbase.servlet.goLoband.fullPath", WebUtils.getApplicationUrl()
    				+ WebUtils.getRealFilterServletPath());
		} catch (IOException e) {}
    	
    	
        params.put("aidbase.context.path", request.getContextPath());
        
        // https://localhost:8443
        params.put("aidbase.server.root.https", request.getHttpsHost());
        
        // http://localhost:8080
        params.put("aidbase.server.root.http",	request.getHttpHost());
        params.put("aidbase.loc","_ab_locinfo");
    	URL originalUrl = request.getURL();		// this includes the parameters
    											// at the end of the url

    	String[] locinfoArray = (String[])(
    			request.getParameter("_ab_locinfo") );
    	String locinfo = null;
    	if (locinfoArray != null) {
    		locinfo = locinfoArray[0];
    	}
    	if (locinfo != null) {
    		LOG.debug("locinfo is: "+locinfo);
    		try {
    			originalUrl = new URL(locinfo);
    		} catch (MalformedURLException e) {
    			throw new InternalFaultException(
    					"Invalid URL: "+locinfo, e);
    		}
    	}
    	
//    	http
    	params.put("base.url.proto", originalUrl.getProtocol());
    	String originalHost = originalUrl.getProtocol() + "://" + 
    		originalUrl.getHost();
    	
    	if (originalUrl.getPort() != -1)
    		originalHost += ":" + originalUrl.getPort();
//    	http://www.trustfunding.org
    	params.put("original.host", originalHost);
    	url=originalHost;
    	String originalUrlFull = originalUrl.toString();
    	String originalPath = originalHost;
		if(originalUrl.getPath().startsWith("/")){
			originalPath += originalUrl.getPath();
		}else{
			originalPath += "/" + originalUrl.getPath();
		}

//		http://www.trustfunding.org/trustfunding/Login.asp
    	params.put("original.path", originalPath);
//    	http://www.trustfunding.org/trustfunding/Login.asp  + parameters, if there are any
		params.put("original.url.full", originalUrlFull);
//		http://www.trustfunding.org/trustfunding/Login.asp
		params.put("base.url.full", originalUrlFull);
//		http://www.trustfunding.org
		params.put("base.url.site", WebUtils.getBaseUrlHost(originalUrl));
		
//		/trustfunding
		params.put("base.url.path", WebUtils.getBaseUrlPath(originalUrl));
		params.put("script.tag", "0");
//		/trustfunding
		params.put("base.url.file", originalUrl.getFile().replaceAll(".*/", ""));
		String originalUrlShort = originalUrlFull;
        if (originalUrlShort.length() > 30) {
        	originalUrlShort = originalUrlShort.substring(0,30) + "...";    
        }
        
        params.put("original.url.short", originalUrlShort);

        // if the page uses ssl, loband will print a message of
        // warning at the bottom of the page

        if(params.get("base.url.proto").equals("https"))
        	params.put("pageMightDisplayBadly", "true");
        
        if(url.indexOf("yahoo.com") > -1)
        {
        	params.put("special","yahoo_mail");
        }
        else if(url.indexOf("hotmail.msn.com")    > -1 ||
        		url.indexOf("login.passport.com") > -1 ||
				url.indexOf("login.passport.net") > -1 ||
				url.indexOf("calendar.msn.com")   > -1)
        {
        	params.put("special","hotmail");
        }
        else if(url.indexOf("google.") > -1)
        {
        	params.put("special", "google");
        }
        else if(url.indexOf("openknowledge.net") > -1)
        {		// trying to make openknowledge.net work
        	params.put("special","openknowledge.net");
        }
        else if(url.indexOf("reliefweb.int") > -1)
        {
        	params.put("special","reliefweb");
        }
        else if(url.indexOf("schoolnetafrica.net") > -1)
        {
        	params.put("special","schoolnetafrica");
        }
        else if(url.indexOf("bbc.co.uk") != -1)
        {
            params.put("special", "bbc");
        }
        else if(url.indexOf("download.com") != -1)
        {
        	params.put("special", "download.com");
        }
        
		return params;
	}
	
	/**
	 * This is a small parser that will comment out all the javascript
	 * between script tags else the parser won't recognize it
	 * 
	 * @param html
	 * @return
	 */
	public static String addScriptComments(String html)
	{
		CharArray[] buffers = new CharArray[7];
		for (int i = 0; i < buffers.length; i++) {
			buffers[i] = new CharArray(1024);
		}	
		char[] input = html.toCharArray();
		
		int countOpenChars = 0, countCloseChars = 0, comment= 0, state = 0;
		final int ST_INITIAL = 0, ST_OPEN = 1, ST_CLOSE = 2, ST_ELEMENT = 3,
			ST_SCRIPT = 4, ST_COMMENT = 5, ST_ADDCOMMENT = 6;
		
		for(int i=0;i<input.length;i++){
			char c = input[i];
			
			if (state == ST_INITIAL)
			{
				if (c == '<'){
					state = ST_ELEMENT;
					countOpenChars=0;
				}
				buffers[state].append(c);
			} 
			else if (state == ST_ELEMENT) 
			{
				buffers[state].append(c);
				if ((c == 's' || c == 'S') && countOpenChars == 0) {
					state = ST_OPEN;
					buffers[ST_OPEN].append(buffers[ST_ELEMENT]);
					buffers[ST_ELEMENT].clear();
					countOpenChars = 1;
				} else if(c == '>') {
					state = ST_INITIAL;
					buffers[ST_INITIAL].append(buffers[ST_ELEMENT]);
					buffers[ST_ELEMENT].clear();
				} else {
					countOpenChars++;
				}
			} 
			else if (state == ST_OPEN) 
			{
				buffers[state].append(c);
				if ((c == 'c' || c == 'C') && countOpenChars == 1) 
				{
					countOpenChars = 2;
				}
				else if ((c == 'r' || c == 'R') && countOpenChars == 2) 
				{
					countOpenChars = 3;
				}
				else if ((c == 'i' || c == 'I') && countOpenChars == 3)
				{
					countOpenChars = 4;
				}
				else if ((c == 'p' || c == 'P') && countOpenChars == 4)
				{
					countOpenChars = 5;
				}
				else if ((c == 't' || c == 'T') && countOpenChars == 5)
				{
					countOpenChars = 6;
				}
				else if (c == '>' && countOpenChars==6)
				{
					buffers[ST_INITIAL].append(buffers[state]);
					buffers[state].clear();
					state = ST_SCRIPT;
					comment = 0;
				}
				else if (countOpenChars == 6)
				{
					// inside <script ...> tag, not seen end of tag yet,
					// ignore the rest of the tag.
				}
				else 
				{
					state = ST_ELEMENT;
					buffers[ST_ELEMENT].append(buffers[ST_OPEN]);
					buffers[ST_OPEN].clear();
				}
			}
			else if (state == ST_SCRIPT)
			{
				// inside script tag: look for an open comment tag
				buffers[state].append(c);
				
				if (c == '<' && comment == 0)
				{
					comment = 1;
				}
				else if (c == '!' && comment == 1) 
				{
					comment = 2;
				}
				else if (c == '/' && comment == 1)
				{
					buffers[ST_CLOSE].append(buffers[state]);
					buffers[state].clear();
					state = ST_CLOSE;
					countCloseChars = 2;
				}
				else if (c == '-' && comment == 2)
				{
					comment=3;
				}
				else if (c == '-' && comment == 3)
				{
					// we found it: go into ST_COMMENT state
					state = ST_COMMENT;
					buffers[ST_INITIAL].append(buffers[ST_SCRIPT]);
					buffers[ST_SCRIPT].clear();
					comment = 0;
				}
				else if (c == ' ' || c == '\t' || c == '\n' || c == '\r')
				{
					comment=0;
				}
				else 
				{
					// found something else: write an open comment tag
					// before it, to ensure that RuleFilter ignores it,
					// and go into ST_ADDCOMMENT STATE, which tells us
					// that we added an open comment tag, so we must
					// add a close comment tag before </script>.
					buffers[state].setLength(buffers[state].length()-1);
					state = ST_ADDCOMMENT;
					countCloseChars = 0;
					buffers[state].append("<!--\n");
					buffers[state].append(c);
					buffers[ST_INITIAL].append(buffers[ST_SCRIPT]);
					buffers[ST_SCRIPT].clear();
				}
			} 
			else if (state == ST_COMMENT) 
			{
				// inside a script comment (<script><!-- ...)
				// look for closing comment
				buffers[state].append(c);
				if (c == '-' && comment == 0){
					comment = 1;
				}
				else if (c == '-' && comment == 1)
				{
					comment = 2;
				}
				else if (c == '>' && comment == 2)
				{
					// found it! go back to ST_SCRIPT state
					// (just inside <script>, not commented
					comment = 0;
					state = ST_SCRIPT;
					buffers[ST_INITIAL].append(buffers[ST_COMMENT]);
					buffers[ST_COMMENT].clear();
				}
				else 
				{
					comment = 0;
				}
			} 
			else if (state == ST_ADDCOMMENT) 
			{
				// look for </script>, so we can add a 
				// close comment before it, when we find it.
				buffers[state].append(c);
				if (c == '<' && countCloseChars == 0)
				{
					countCloseChars = 1;
				}
				else if (c=='/' && countCloseChars == 1)
				{
					countCloseChars = 2;
				}
				else if ((c == 's' || c == 'S') && countCloseChars == 2)
				{
					countCloseChars = 3;
				}
				else if ((c == 'c' || c == 'C') && countCloseChars == 3)
				{
					countCloseChars = 4;
				}
				else if ((c == 'r' || c == 'R') && countCloseChars == 4)
				{
					countCloseChars = 5;
				}
				else if ((c == 'i' || c == 'I') && countCloseChars == 5)
				{
					countCloseChars = 6;
				}
				else if ((c == 'p' || c == 'P') && countCloseChars == 6)
				{
					countCloseChars = 7;
				}
				else if ((c == 't' || c == 'T') && countCloseChars == 7)
				{
					countCloseChars = 8;
				}
				else if (c == '>' && countCloseChars >= 8)
				{
					// found it! write the close comment,
					// then close script.
					countCloseChars++;
					buffers[state].setLength(
							buffers[state].length()-countCloseChars);
					buffers[state].append("\n//--></script>");
					buffers[ST_INITIAL].append(buffers[state]);
					buffers[state].clear();
					state = ST_INITIAL;
					countOpenChars = 0;
				} 
				else if (countCloseChars >= 8)
				{
					countCloseChars++;
				}
				else 
				{
					countCloseChars=0;
				}
			}
			else if (state == ST_CLOSE)
			{
				// this is some kind of closing tag (</....>)
				// is it a close script tag?
				buffers[state].append(c);
				if ((c == 's' || c == 'S') && countCloseChars == 2)
				{
					countCloseChars = 3;
				}
				else if ((c == 'c' || c == 'C') && countCloseChars == 3)
				{
					countCloseChars =4;
				}
				else if ((c == 'r' || c == 'R') && countCloseChars == 4)
				{
					countCloseChars = 5;
				}
				else if ((c == 'i' || c == 'I') && countCloseChars == 5)
				{
					countCloseChars =6;
				}
				else if ((c == 'p' || c == 'P') && countCloseChars == 6)
				{
					countCloseChars =7;
				}
				else if ((c == 't' || c == 'T') && countCloseChars==7){
					countCloseChars = 8;
				}
				else if (c == '>' && countCloseChars == 8)
				{
					buffers[ST_INITIAL].append(buffers[state]);
					buffers[state].clear();
					state = ST_INITIAL;
					countOpenChars = 0;
				}
				else if (countCloseChars == 8)
				{
					
				}
				else 
				{
					buffers[ST_SCRIPT].append(buffers[state]);
					state = ST_SCRIPT;
					comment = 0;
				}
			}
		}
		
		return buffers[ST_INITIAL].toString();
	}

}