/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 *
 * Created on 08-Dec-2003
 */
package org.aidworld.aidbase.filter;

import java.net.MalformedURLException;
import java.net.URL;

import org.aidworld.aidbase.document.Document;
import org.aidworld.aidbase.error.AidbaseException;
import org.aidworld.aidbase.error.BlockedUrlException;
import org.aidworld.aidbase.request.Request;
import org.apache.log4j.Logger;

/**
 * This Filter checks for various sites which Aidbase should
 * not filter, but instead report a sensible error to the user.
 * This is better than producing an empty or unreadable page,
 * but not as good as handling the site correctly.
 */

public class UrlBlockingFilter extends BasicFilter {

    private static Logger LOG = Logger.getLogger(UrlBlockingFilter.class);

    String[][] blockedUrls = new String[][] {
        {"http%3A%2F%2Fwww.aidbase.org", "http://www.aidbase.org"},
        {"http://www.aidbase.org", "http://www.aidbase.org"},
		{"http://www.loband.org", "http://www.loband.org"},
    };

    /**
     * @see org.aidworld.aidbase.core.Filter#filter(org.aidworld.aidbase.core.Request, org.aidworld.aidbase.core.Server)
     */
    public Document service(Request request) throws AidbaseException
	{
            if (LOG.isDebugEnabled()) LOG.debug("UrlBlockingFilter starting");
        
            // temporary solution
            URL requestUrl = request.getURL();
            if (requestUrl != null){

                for(int i=0; i<blockedUrls.length; i++){
                    if(requestUrl.toString().equals(blockedUrls[i][0])) {
                        BlockedUrlException bue = new BlockedUrlException(
                        	// I18N 3017 
                            "Aidbase does not currently work with this page.");
                        
                        String destinationUrl = blockedUrls[i][1];
                        if (destinationUrl != null && !destinationUrl.equals("")) 
                        {
                            try {
                                bue.setRedirectUrl(new URL(destinationUrl));
                            } catch (MalformedURLException e) {
                                if(LOG.isDebugEnabled()) 
                                	LOG.debug("Malformed URL for redirection");
                            }
                        }
                        
                        throw bue;
                    }
                }
            }

            Document document = super.service(request);

            if (LOG.isDebugEnabled()) LOG.debug("UrlBlockingFilter finished");
            return document;

    }

}
