/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import com.opensymphony.module.sitemesh.html.BasicRule;
import com.opensymphony.module.sitemesh.html.Tag;
import com.opensymphony.module.sitemesh.html.TagRule;
import com.opensymphony.module.sitemesh.html.util.CharArray;

//TODO : Javadoc for this class
public class BaseFormRule extends BasicRule {
	protected boolean submitFound = false;
	protected IfFormParticipatingRule[] rules;

	
	public BaseFormRule(IfFormParticipatingRule[] formRules) {
		super();
		rules = formRules;
		if (rules == null) {
			rules = new IfFormParticipatingRule[0];
		}
		
		//register ourselves with each of the subrules so they
		//can call us back
		for (int i = 0; i < rules.length; i++) {
			rules[i].registerBaseFormRule(this);
		}
	}

	/**
	 * Before we write out a closing form tag we check to see if
	 * we need to write out a submit button.
	 */
	protected void writeCloseFormTag(CharArray buffer) {
		//if (!submitFound) {
			//buffer.append("<input type=\"submit\" />");
		//}
		buffer.append("</form>");
	}

	
	/**
	 * Delegate the choice of processing to our member rules
	 */
	public boolean shouldProcess(String tagName) {
		//loop through our configured rules to see if any
		//of them are interested in the tag
		for (int i = 0; i < rules.length; i++) {
			TagRule rule = (TagRule)rules[i];
			if (rule.shouldProcess(tagName)) return true;
		}
		return false;
	}
	
	public void process(Tag tag) {
		for (int i = 0; i < rules.length; i++) {
			TagRule rule = (TagRule)rules[i];
			if (rule.shouldProcess(tag.getName())) {
				rule.setContext(context);
				rule.process(tag);
				return;
			}
		}
	}

	public void newForm() {
		submitFound = false;
	}
	
	public void foundSubmit() {
		submitFound = true;
	}
	
}

