/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

import org.aidworld.aidbase.webui.util.WebUtils;
import org.apache.log4j.Logger;

import com.opensymphony.module.sitemesh.html.Tag;

/**
 * This rule will detect base tags and change 
 * the properties so that every link will include
 * the base href attribute.
 */
public class BaseRemovingRule extends AttributeFilterRule {

	Logger LOG = Logger.getLogger (BaseRemovingRule.class);
	
	/**
	 * Main constructor
	 * 
	 * @param	params	The general properties object 
	 * 					which is made in RuleFilter
	 * @param	special	Name of the special site if we 
	 * 					are working on one, e.g. "hotmail".
	 */
	public BaseRemovingRule(Properties params,String special) {
		super(new String[]{"base"}, params, special);
	}

	/**
	 * Checks the base tag for a valid href attribute.
	 * 
	 * @param	tag	The base tag itself.
	 */
	public void process(Tag tag) {
		String href = tag.getAttributeValue("href");
		if(href == null){
			super.process(tag);
			return;
		}
		try {
			URL u = new URL(href);
			setBaseParams(u);
		} catch (MalformedURLException e) {
			//throw new Error(e);
			try {
				String href2 = "http://" + href;
				URL u = new URL(href2);
				setBaseParams(u);
			} catch (MalformedURLException e2) {
				LOG.warn("Malformed base url: " + href);
			}
		}
	}
	
	/**
	 * Changes all the properties with an url in the params
	 * to contain the base tag. 
	 * 
	 * @param baseUrl	The href attribute of the base tag
	 */
	public void setBaseParams(URL baseUrl) {
		String baseUrlFull = baseUrl.toString();
		params.put("base.url.full",  baseUrlFull);
        params.put("base.url.proto", baseUrl.getProtocol());
        params.put("base.url.site",  WebUtils.getBaseUrlHost(baseUrl));
        params.put("base.url.path",  WebUtils.getBaseUrlPath(baseUrl));
        params.put("base.url.file",  baseUrl.getFile().replaceAll(".*/", ""));
	}
}
