/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.util.Properties;

import org.aidworld.aidbase.core.Version;
import org.aidworld.aidbase.webui.util.WebUtils;

import com.opensymphony.module.sitemesh.html.HTMLProcessorContext;
import com.opensymphony.module.sitemesh.html.StateChangeListener;
import com.opensymphony.module.sitemesh.html.Tag;
import com.opensymphony.module.sitemesh.html.util.CharArray;

// TODO : Javadoc for this class
/**
 * Rule called for BODY and HTML tags and at the end of the document.
 * It handles inserting a default header and footer and also sorts out
 * problems with unclosed body/html tags and with multiple body tags
 */
public class BodyRule extends AttributeFilterRule implements StateChangeListener {
	private Properties params = null;
    public BodyRule(Properties params, String special) {
        super(new String[] {"body", "html", "head"}, params, special);
        this.params = params;
    }
    
    public void process(Tag tag) {
    	
    	if(tag.getName().equalsIgnoreCase("html") && tag.getType() == Tag.OPEN){
    		params.put("htmlOpened","true");
    		context.currentBuffer().append("<html>");
    	}//Closing of the html tag is being done in stateFinished()
    	
    	else if (tag.getName().equalsIgnoreCase("head")){
    		if (tag.getType() == Tag.OPEN ){
    			params.put("headOpened","true");
    			writeAidbaseHead(context.currentBuffer(), params);
    		}else if (tag.getType() == Tag.CLOSE ){
    			params.put("headClosed","true");
    	    	if (params.get("titleProcessed").equals("false")){
    	    		writeAidbaseTitle(context.currentBuffer());
    	    		params.put("titleProcessed", "true");
    	    	}
    			context.currentBuffer().append("</head>");
    		}
    		
    	}else if (tag.getName().equalsIgnoreCase("body")){
    		if (tag.getType() == Tag.OPEN ){
    			
    			if (params.get("headOpened").equals("true")
    					&& params.get("headClosed").equals("false")){
	    				if (params.get("titleProcessed").equals("false"))
	        	    		writeAidbaseTitle(context.currentBuffer());
    					context.currentBuffer().append("</head>");
    			}
    			else if (params.get("headOpened").equals("false")){
    				writeAidbaseHead(context.currentBuffer(), params);
    				params.put("headOpened","true");
    				params.put("headClosed","true");
    				if (params.get("titleProcessed").equals("false")){
        	    		writeAidbaseTitle(context.currentBuffer());
        	    		params.put("titleProcessed", "true");
    				}
    				context.currentBuffer().append("</head>");
    			}
    			
	    		if (params.get("bodyProcessed").equals("false")){
	    			params.put("bodyProcessed","true");
	    	    	if (tag != null)
	    	    		super.process(tag);
	    			else
	    				context.currentBuffer().append("<body>");
	    	    	writeAidbaseHeader(tag, context.currentBuffer(), params);
	    		}else{
	    			context.currentBuffer().append("</body>");
	    			super.process(tag);
	    		}
    		}//Closing of the body is being done in stateFinished()
    		//but this allows some mis-nesting of tags eg <noframes><body></body></noframes> 
    		//becomes <noframes><body></noframes></body>, but this is better than missing off
    		//the footer or placing it too early or multiple times in the page.
    	}
    	
    	
    }
    
    private static void writeAidbaseHead(CharArray buffer, Properties params)
    {
    	if (params.get("htmlOpened").equals("false"))
    	{
    		params.put("htmlOpened", "true");
    		buffer.append("<html>");
    	}
    	
    	buffer.append("<head>");
    	if (params.get("aidbase.filter.disable")!=null)
    	{
    		buffer.append(WebUtils.getPageHeadHtmlNoFilter());
    	}
    	else
    	{
    		buffer.append(WebUtils.getPageHeadHtml());		
    	}
    }
    private static void writeAidbaseTitle(CharArray buffer){
    	buffer.append("<title>" +
    					Version.getShortName().replaceAll(">",
    							"&gt;") 
						+ " </title>");
    }
    
    private static void writeAidbaseHeader(Tag tag, CharArray buffer, Properties params){
    	if (!params.get("aidbase.header.enable").equals("0"))
    	{
    		String x= params.get("aidbase.simulatorOnOff").toString();					
			
    		if(params.get("aidbase.filter.disable")!=null ||
    			 x.equals("true")) {
    			buffer.append(WebUtils.getPageHeaderSimulator(
    					(String) (params.get("original.url.full")), params));
    		}
    		else {
    			buffer.append(WebUtils.getPageHeader(
    				(String) (params.get("original.url.full"))));
    		}
    	}
    }

    
    
	public void stateFinished(HTMLProcessorContext context) {
		boolean aidbaseHeaderEnable = !params.get("aidbase.header.enable").equals("0");
		boolean bodyProcessed = params.get("bodyProcessed").equals("true");
		boolean addWarning = params.get("pageMightDisplayBadly").equals("true");
		if (params.get("htmlOpened").equals("true")){
			
			if (aidbaseHeaderEnable && bodyProcessed)
			{
				if(params.get("aidbase.filter.disable")==null)
				{
					context.currentBuffer().append(WebUtils.getPageFooter(addWarning));
				}
				else
				{
					context.currentBuffer().append("</body></html>");
				}
			}
			
			if (bodyProcessed)
				context.currentBuffer().append("</body>");
			
			if(params.get("headOpened").equals("false")){
				writeAidbaseHead(context.currentBuffer(), params);
				if (params.get("titleProcessed").equals("false")){
    	    		writeAidbaseTitle(context.currentBuffer());
    	    		params.put("titleProcessed", "true");
				}
				context.currentBuffer().append("</head>");
			}
			
			context.currentBuffer().append("</html>");

		}
		
	}
	
	
	public static void processBody(Tag tag, Properties params, CharArray buffer){
		String[] tagsInsideHead = FilterList.TAGS_INSIDE_HEAD;
		boolean processBody = true;
		
		
		for (int i = 0; i<tagsInsideHead.length; i++)
			if (tag.getName().equalsIgnoreCase(tagsInsideHead[i]))
				processBody = false;
		
		if (processBody){
			if(params.get("headOpened").equals("false")){
				writeAidbaseHead(buffer, params);
				params.put("headOpened","true");
				params.put("headClosed","true");
				if (params.get("titleProcessed").equals("false")){
    	    		writeAidbaseTitle(buffer);
    	    		params.put("titleProcessed", "true");
				}
				buffer.append("</head>");
			}
			params.put("bodyProcessed","true");
			buffer.append("<body>");
			writeAidbaseHeader(tag, buffer, params);
		}
		
	}
	
}
