/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.util.Properties;

import com.opensymphony.module.sitemesh.html.Tag;

/**
 * This rule changes any form image submission buttons into standard submit
 * buttons. If the original image element has a name and value pair associated with it
 * then these are carried over to the new element, otherwise all other non-Javascript
 * attributes are stripped out.
 */
public class ChangeSubmitImageToButtonRule extends AttributeFilterRule
implements IfFormParticipatingRule {
	protected BaseFormRule baseFormRule; 
	private Properties params = null;
	
	/**
	 * Main constructor
	 * 
	 * @param	params	The general properties object which is made in RuleFilter
	 * @param	special	Name of the special site if we are working on one 
	 */
	public ChangeSubmitImageToButtonRule(Properties params,String special) {
		super(new String[]{"input"},params,special);
		this.params = params;
	}

	/**
	 * This method will change image submit button to normal submit 
	 * buttons.
	 * 
	 * @param tag	The input tag	
	 */
	public void process(Tag tag) {
		processBody(tag);
		if("image".equalsIgnoreCase(tag.getAttributeValue("type"))) {
			context.currentBuffer().append("<input type=\"submit\"");
			if(tag.hasAttribute("name") && tag.hasAttribute("value")) {
				context.currentBuffer().append(" name=\"").append(tag.getAttributeValue("name")).append("\"");
				context.currentBuffer().append(" value=\"").append(tag.getAttributeValue("value")).append("\"");
			}else if (tag.hasAttribute("value")) 
				context.currentBuffer().append(" value=\"").append(tag.getAttributeValue("value")).append("\"");
			else
				context.currentBuffer().append(" value=\"Submit\"");
			
			for(int i=0;i<tag.getAttributeCount();i++){
				if(tag.getAttributeName(i).startsWith("on")){
					context.currentBuffer().append(" " + 
							tag.getAttributeName(i) + "=\"" + 
							tag.getAttributeValue(i) + "\"");
				}
			}
			
			context.currentBuffer().append("/>");
			
			baseFormRule.foundSubmit();
		} else if("submit".equals(tag.getAttributeValue("type"))) {
			super.process(tag);
			baseFormRule.foundSubmit();
		} else {
			super.process(tag);
		}
	}

	/**
	 * TODO: Javadoc
	 */
	public void registerBaseFormRule(BaseFormRule baseRule) {
		this.baseFormRule = baseRule;
	}
	
	/**
	 * This method will check if there is a body tag before
	 * this rule's input tag or else add
	 * one. This to ensure that we get a proper Aidbar.
	 * 
	 * @param tag The input tag of the rule
	 */
    private void processBody(Tag tag){
    	if (this.params.get("bodyProcessed").equals("false"))
    		BodyRule.processBody(tag, this.params, context.currentBuffer());
    }
}
