/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.util.HashMap;
import java.util.Map;

/**
 * This class contains some static lists for 
 * filtering purposes.  
 */
public class FilterList {
	
	/**
	 * A list of attributes we want to keep for special
	 * sites.
	 */
	public static final String[] SPECIAL_ATTRIBUTES = new String[]{
			"class"
	};
	
	/**
	 * A list of attributes we want to keep in tags.
	 * Every attribute that is not on this list gets 
	 * removed from the tags.
	 */
	public static final String[] ATTRIBUTES = new String[]{
			"accept-charset", 
			"action",
			"alt",
			"checked",
			"cols",
			"colspan",
			"content",
			"coords",
			"disabled",
			"enctype",
			"href",
			"http-equiv",
			"id",
			"label",
			"language",
			"maxlength",
			"method",
			"multiple",
			"name",
			"rel",
			"rows",
			"rowspan",
			"selected",
			"shape",
			"size",
			"src",
			"tabindex",
			"target",
			"title",
			"type",
			"usemap",
			"value",
			"onblur",
			"onchange",
			"onclick",
			"ondblclick",
			"onfocus",
			"onkeydown",
			"onkeypress",
			"onkeyup",
			"onload",
			"onmousedown",
			"onmousemove",
			"onmouseout",
			"onmouseover",
			"onmouseup",
			"onreset",
			"onselect",
			"onsubmit",
			"onunload"
	};
	
	/**
	 * A list of tags of which the attributes are stripped. 
	 * The attributes declared in ATTRIBUTES are not removed. 
	 */
	public static final String[] TAGS_STRIP = new String[] {
			"blockquote",
			"button",
			"b",
			"code",
			"em",
			"fieldset",
			"isindex",
			"i",
			"h1",
			"h2",
			"h3",
			"h4",
			"h5",
			"h6",
			"hr",
			"label",
			"legend",
			"li",
			"map",
			"nextid",
			"range",
			"ol",
			"optgroup",
			"option",
			"p",
			"pre",
			"q",
			"s",
			"samp",
			"select",
			"span",
			"strike",
			"strong",
			"sub",
			"sup",
			"tbody",
			"table",
			"br",
			"div",
			"tfoot",
			"tr",
			"td",
			"textarea",
			"th",
			"thead",
			"u",
			"ul"
	};
	
	/**
	 * A list of tags that get completly removed including
	 * contents.
	 */
	public static final String[] TAGS_REMOVE = new String[] {
			"abbr",
			"acronym",
			"address",
			"basefont",
			"bdo",
			"big",
			"caption",
			"center",
			"cite",
			"col",
			"colgroup",
			"del",
			"dfn",
			"dir",
			"dl",
			"dd",
			"dt",
			"font",
			"hr",
			"iframe",
			"ins",
			"kbd",
			"layer",
			"link",
			"menu",
			"meta",
			"nobr",
			"param",
			"small",
			"tt",
			"var"	
	};
	
	/**
	 * A list of image names which will be comletly removed.
	 * They also don't get an image link.
	 */
	public static final String[] IMAGES_REMOVE = new String[] {
			"spacer.",
			"pixel.",
			"bullet.",
			"arrow",
			"cleardot.",
			"poweredby",
			"sponsor",
			"rollover",
			"pix.",
			"nav.",
			"space.",
			"blank."
	};
	
	/**
	 * A list of tags which should be located inside the HEAD 
	 * tags.
	 */
	public static final String[] TAGS_INSIDE_HEAD = new String[]{
			"isindex",
			"link",
			"meta",
			"nextid",
			"frameset", 
			"frame",
			"style",
			"if", 
			"endif"
//			base, title and script have their own rules where from 
//			processBody() is never called	
	};

	/**
	 * A hashmap for replacing special images with text. 
	 * This list is for YAHOO_MAIL only which is a special
	 * site.
	 */
	public static final Map YAHOO_MAIL = new HashMap();
	static {
		YAHOO_MAIL.put("bt_dd_","&#160;&#160;&#160;&#160;\\/");
		YAHOO_MAIL.put("shift1.gif","shift");
		YAHOO_MAIL.put("clip_1.gif","[ATTACH]");
		YAHOO_MAIL.put("flag.gif","[FLAG]");
	}
	
	/**
	 * A hashmap for replacing special images with text. 
	 * This list is for GOOGLE only which is a special
	 * site.
	 */
	public static final Map GOOGLE = new HashMap();
	static {
		GOOGLE.put("nav_previous.gif","G");
		GOOGLE.put("nav_first.gif","G");
		GOOGLE.put("nav_current.gif","O");
		GOOGLE.put("nav_page.gif", "o");
		GOOGLE.put("nav_next", "gle");
	}
	
	/**
	 * A hashmap for replacing special images with text. 
	 * This list is for HOTMAIL only which is a special
	 * site.
	 */
	public static final Map HOTMAIL = new HashMap();
	static {
		HOTMAIL.put("i.p.downarrow.gif","+");
		HOTMAIL.put("i.p.sort.desc.gif","v");
		HOTMAIL.put("i.p.sort.asc.gif","^");
		HOTMAIL.put("i.p.writenew.gif","");
		HOTMAIL.put("i.p.delete.gif","");
		HOTMAIL.put("i.p.block.gif","");
		HOTMAIL.put("i.p.junkmail.gif","");
		HOTMAIL.put("i.p.search.gif","");
		HOTMAIL.put("i.p.putinfolder.gif","");
		HOTMAIL.put("i.p.new.gif","");
		HOTMAIL.put("i.p.folder.inbox.gif","");
		HOTMAIL.put("i.p.trash.gif","");
		HOTMAIL.put("i.p.folder.sent.gif","");
		HOTMAIL.put("i.p.folder.drafts.gif","");
		HOTMAIL.put("i.p.folder.gif","");
		HOTMAIL.put("tab.separator.off.gif","");
		HOTMAIL.put("tab.separator.end.gif","");
		HOTMAIL.put("i.p.cal.gif","");
		HOTMAIL.put("tab.separator.on.r.gi","");
		HOTMAIL.put("tab.separator.on.l.gif","");
		HOTMAIL.put("i.p.cont.individual.gif","");
		HOTMAIL.put("tab.slide.hm.li.gif","");
		HOTMAIL.put("i.p.arr.left.gif","");
		HOTMAIL.put("i.p.send.gif","");
		HOTMAIL.put("i.p.attach.gif","");
		HOTMAIL.put("i.p.importance.h.gif","!");
		HOTMAIL.put("i.p.importance.l.gif","v");
		HOTMAIL.put("i.p.tools.gif","");
		HOTMAIL.put("i.p.cancel.gif","");
		HOTMAIL.put("i.p.white.b.gif","");
		HOTMAIL.put("i.p.black.b.gif","");
		HOTMAIL.put("lgo_msn_118x35.gif","");
		HOTMAIL.put("i.p.greetcard.gif","");
		HOTMAIL.put("i.p.cal.meet.req.gif","");
		HOTMAIL.put("i.p.cal.task.gif","");
		HOTMAIL.put("i.p.cal.note.gif","");
		HOTMAIL.put("i.p.cont.group.gif","");
		HOTMAIL.put("i.p.attachfile.gif","");
		HOTMAIL.put("i.p.delete.d.gif","");
		HOTMAIL.put("i.p.reply.gif","");
		HOTMAIL.put("i.p.replyall.gif","");
		HOTMAIL.put("i.p.fwd.gif","");
		HOTMAIL.put("i.p.printv.gif","");
		HOTMAIL.put("i.p.replyall.d.gif","");
		HOTMAIL.put("i.p.reply.d.gif","");
	}
	
}
