/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.io.IOException;
import java.net.URL;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.aidworld.aidbase.webui.util.WebUtils;

import com.opensymphony.module.sitemesh.html.Tag;
import com.opensymphony.module.sitemesh.html.util.CharArray;

/**
 * This rule will rewrite a form tag so that the action attribute
 * of the form will be pointing to /loband/page. Also it will
 * add a parameter called _ab_url as an hidden field. 
 * _ab_url will contain the original action attribute of the 
 * form rewritten as a complete URL.
 */
public class FormActionRewritingRule extends AttributeFilterRule 
implements IfFormParticipatingRule {
	protected BaseFormRule baseFormRule;
	private Properties params = null;
	
	/**
	 * Main constructor
	 * 
	 * @param params	The general properties object which is 
	 * 					made in RuleFilter 
	 * @param special	Name of the special site if we are 
	 * 					working on one, e.g. "hotmail".
	 */
	public FormActionRewritingRule(Properties params,String special) {
		super(new String[]{"form"}, params, special);
		this.params = params;
	}

	/**
	 * This method will rewrite the form action attribute and
	 * add the _ab_url paramater.
	 * If the action attribute contains any Javascript it will
	 * not be rewriten. 
	 * 
	 * @param	tag	 The form tag
	 */
	public void process(Tag tag) {
		processBody(tag);
		CharArray buffer = context.currentBuffer();
		
		if (tag.getType() == Tag.CLOSE) {
			baseFormRule.writeCloseFormTag(buffer);
			return;
		}
		
		buffer.append("<");
		buffer.append(tag.getName());
		
		int attributeCount = tag.getAttributeCount();
		for (int i = 0; i < attributeCount; i++) {
			String name  = tag.getAttributeName(i);
			String value = tag.getAttributeValue(i);
			processAttribute(tag, name, value);
		}

		
		
		//Find out what the original action was
			
		String oldAction = tag.getAttributeValue("action");
		if (oldAction == null) 
			oldAction = params.getProperty("base.url.full"); 
		/*
		 * change uppercase HTTP/Https to lowercase
		 */
		Pattern pat=Pattern.compile("http://",Pattern.CASE_INSENSITIVE);
		Matcher mat=pat.matcher(oldAction);
		if (mat.find()&&mat.start()==0)
			oldAction="http://"+oldAction.substring(7);
		
		Pattern pat2=Pattern.compile("https://",Pattern.CASE_INSENSITIVE);
		Matcher mat2=pat2.matcher(oldAction);
		if (mat2.find()&&mat2.start()==0)
			oldAction="https://"+oldAction.substring(8);
		
		
		
		oldAction = oldAction.replaceAll("\"", "\\\"");	//clean it up

		if (tag.getType() == Tag.CLOSE) {
			baseFormRule.writeCloseFormTag(buffer);
		} else if (tag.getType() == Tag.EMPTY) {
			baseFormRule.newForm();

			String newAction = "";
			//with an empty tag we have just one param so it's fine to send it in the querystring
			if(oldAction.startsWith("javascript:")){
				newAction = oldAction;
			}else {
				newAction = rewriteUrl(tag, oldAction,
				        "");
			}
			super.processAttribute(tag, "action", newAction);
			buffer.append(">");
			
			baseFormRule.writeCloseFormTag(buffer);
		} else {
			baseFormRule.newForm();

			if(oldAction.startsWith("https://")){
				super.processAttribute(tag, "action", oldAction); 
				buffer.append(">");	
				
			}
			
			else if(oldAction.startsWith("javascript:")){
				super.processAttribute(tag, "action", oldAction); 
				buffer.append(">");
			}
			else{
				String newAction;
				URL currentUrl, submitUrl;			
				try {
					currentUrl = new URL((String)( params.get("base.url.full") ));
					submitUrl  = new URL(currentUrl, oldAction);
				} catch (Exception e) {
					throw new Error("Invalid URL:", e);
				}
	
				
				String servletPath = 
					params.getProperty("aidbase.servlet.filter.path");
				
				if (submitUrl.getProtocol().equals("https") && 
						! currentUrl.getProtocol().equals("https")) {
					newAction = params.get("aidbase.server.root.https") +
						servletPath;
				} else if (submitUrl.getProtocol().equals("http") && 
						! currentUrl.getProtocol().equals("http")) {
					newAction = params.get("aidbase.server.root.http") +
						servletPath;
				} else {
					try
					{
						newAction = WebUtils.getLobandCallUrl(submitUrl.toString(),"");
					} catch (IOException e) {newAction = "";}		// TODO: newAction should be set to
												// at least an error page if an exception is caught
				}
				
				if(params.getProperty("special").equals("reliefweb") && tag.hasAttribute("url"))
				{
					buffer.append(" url=\"" + tag.getAttributeValue("url") + "\"");
				} else {
					super.processAttribute(tag, "action", newAction);
				}
					
				buffer.append(">");
					
				
					
					String x= params.get("aidbase.simulatorOnOff").toString();					
					if(x.equals("true"))
					{
						String newValue = "<input type=\"hidden\" name=\""
							+ params.get("aidbase.paramname.simulatorkey")
							+ "\" value=\"" + params.get("aidbase.simulatorkey")
							+ "\">";
						buffer.append(newValue);
						newValue = "<input type=\"hidden\" name=\""
							+ params.get("aidbase.paramname.simulatorbps")
							+ "\" value=\"" + params.get("aidbase.simulatorbps")
							+ "\">";
						buffer.append(newValue);
						
					}
					if(params.get("aidbase.filter.disable")!=null)
					{
						String newValue = 
							"<input type=\"hidden\" name=\"_ab_nofilter\">";							
						buffer.append(newValue);
					}
					
				String formTarget = tag.getAttributeValue("target");
					
				if (params.get("aidbase.header.enable").equals("0") &&
						(formTarget == null || ! formTarget.equals("_top"))) {
					buffer.append("<input type=\"hidden\" " +
							"name=\"_ab_noheader\" value=\"1\"/>");
				}
			 }
		}
		
	}

	/**
	 * This method will remove useless attributes from the form 
	 * tag.
	 * 
	 * @param	tag		The form tag itself
	 * @param	name	The name of the attribute
	 * @param 	value	The value of the attribute
	 */
	public void processAttribute(Tag tag, String name, String value) {
		if ("method".equalsIgnoreCase(name) || 
			"target".equalsIgnoreCase(name) ||
			 "name".equalsIgnoreCase(name) || 
			 "enctype".equalsIgnoreCase(name) || 
			 name.startsWith("on") ||
			 name.startsWith("ON")) {
			
				super.processAttribute(tag, name, value); 
		}
	}

	/**
	 * TODO: write javadoc
	 */
	public void registerBaseFormRule(BaseFormRule baseRule) {
		this.baseFormRule = baseRule;
	}
	
	/**
	 * This method will check if there is a body tag before
	 * this rule's input tag or else add
	 * one. This to ensure that we get a proper Aidbar.
	 * 
	 * @param tag The input tag of the rule
	 */
    private void processBody(Tag tag){
    	if (this.params.get("bodyProcessed").equals("false"))
    		BodyRule.processBody(tag, this.params, context.currentBuffer());
    }

}
