/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.util.Properties;

import com.opensymphony.module.sitemesh.html.Tag;
import com.opensymphony.module.sitemesh.html.util.CharArray;

/**
 * If a site uses a frameset this rule will change
 * the frameset so that it will display a frame 
 * at the top of the page containing an aidbar. 
 * All the pages inside the frames won't display an
 * aidbar
 */
public class FrameRule extends AttributeFilterRule {
	private Properties params = null;
	private boolean aidbar = false;
	
	/**
	 * Main constructor
	 * 
	 * @param params	The general properties object which is 
	 * 					made in RuleFilter 
	 * @param special	Name of the special site if we are 
	 * 					working on one, e.g. "hotmail".
	 */
	public FrameRule(Properties params,String special) {
		super(new String[] {"frame","frameset"}, params, special);
		this.params = params;
	}
	  
	
	/**
	 * This method will rewrite the frameset and make sure 
	 * that pages inside the frames don't display an aidbar.
	 * 
	 * @param	tag	The frame(set) tag
	 */
	public void process(Tag tag) {
		processBody(tag);
		CharArray buffer = context.currentBuffer();
		
		
		//If _ab_noheader is true for this frame then pretend we already had one
		if (params.getProperty("aidbase.header.enable").equals("0"))
		{
			
			aidbar =true;
			
		}
		
		if (tag.getName().toLowerCase().equals("frameset")) 
		{
			if (tag.getType() == Tag.OPEN)
			{
				int counter = Integer.parseInt((String)params.get("framecounter"));
				counter++;
			
				if (!aidbar) 
				{
					String dummyServerUrl;
					
					if (params.get("base.url.proto").equals("https"))
						dummyServerUrl = params.getProperty("aidbase.application.url.ssl");
					else
						dummyServerUrl = params.getProperty("aidbase.application.url");
					
			  		String dummyframe = "<frameset border=\"0\" rows=\"45,*,45\">";
			  		dummyframe += "<frame ";
			  		dummyframe += "src=\"" + dummyServerUrl + "/frame_header.jsp?URLFULL="
						+ params.get("original.url.full") + "\" ";
			  		dummyframe += "name=\"_aidbar\" ";
			  		dummyframe += "noresize=\"noresize\" ";
			  		dummyframe += "scrolling=\"no\"/>";
			  		dummyframe += tag.getContents();
			  		buffer.append(dummyframe);
			  		aidbar = true;
				}
				else
				{
					context.currentBuffer().append(tag.getContents());
				}
				params.put("framecounter",Integer.toString(counter));
			}
			else if (tag.getType() == Tag.CLOSE)
			{
				int counter =Integer.parseInt((String)params.get("framecounter"));
				counter--;
				params.put("framecounter",Integer.toString(counter));
				
				context.currentBuffer().append("</frameset>");
				if (counter==0) {
					String dummyServerUrl;
					
					if (params.get("base.url.proto").equals("https"))
						dummyServerUrl = params.getProperty("aidbase.application.url.ssl");
					else
						dummyServerUrl = params.getProperty("aidbase.application.url");
					
					context.currentBuffer().append("<frame "+
							"src=\"" + dummyServerUrl + "/frame_footer.jsp\" />"
							+ "</frameset>");
				}
				
			}
		}
		else if (tag.getName().toLowerCase().equals("frame")) 
		{
			if (tag.hasAttribute("src")){
				String src = rewriteUrl(tag,tag.getAttributeValue("src"), "");
				
				context.currentBuffer().append("<frame");
				
				for (int i = 0; i < tag.getAttributeCount(); i++) 
				{
					String attributenName = tag.getAttributeName(i);
					if (attributenName.equalsIgnoreCase("src")) 
					{
						context.currentBuffer().append(" src=\""+src+"\"");
					} 
					else 
					{
						context.currentBuffer().append(" " + tag.getAttributeName(i)
								+ "=\"" + tag.getAttributeValue(i) +"\"");
					}
				}
				
				context.currentBuffer().append("/>");
			}
		}
	}
	
	/**
	 * This method will check if there is a body tag before
	 * this rule's input tag or else add
	 * one. This to ensure that we get a proper Aidbar.
	 * 
	 * @param tag The input tag of the rule
	 */
    private void processBody(Tag tag){
    	if (this.params.get("bodyProcessed").equals("false"))
    		BodyRule.processBody(tag, this.params, context.currentBuffer());
    }
}
