/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.util.Properties;

import com.opensymphony.module.sitemesh.html.Tag;

/**
 * This rule strips out all useless attributes from the remaining tags
 * once other rules have acted.
 */
public class GeneralTagRule extends AttributeFilterRule { 
	private Properties params = null;
	
	/**
	 * All the tags that are declared in FilterList and don't have
	 * a specific rule end up here.
	 * This rule will filter all the useless attributes or remove the tag
	 * completly.
	 * 
	 * @param tagArray	A list of tags that needs filtering
	 * @param params	params	The general properties object which is made in RuleFilter
	 * @param special	Name of the special site if we are working on one, e.g. "hotmail". 
	 */
    public GeneralTagRule(String[] tagArray, Properties params, 
    		String special) {
    	super(tagArray, params,special);
    	this.params = params;
    }
    
    /**
     * This method will seek for attributes that we want to remove.
     * If it finds any it will pass them on to AttributeFilterRule.
     * After the filtering of all the attributes it will write them
     * out to the main buffer. This method also removes complete tags
     * if we think they are not useful.
     * 
     * @param tag 	The input tag
     */
    public void process(Tag tag) {
    	processBody(tag);
    	
    	boolean removeTag = false;
    	for ( int i = 0; i< FilterList.TAGS_REMOVE.length; i++){
    		if(tag.getName().equalsIgnoreCase(FilterList.TAGS_REMOVE[i])){
    			removeTag = true;
    		}
    	}
    	
    	if(!removeTag){
	    	if (tag.getType() == Tag.OPEN) {
	        	context.currentBuffer().append("<").append(tag.getName());
				boolean stripTag = false;
	        	for (int i = 0; i< FilterList.TAGS_STRIP.length; i++){
	        		if(tag.getName().equalsIgnoreCase(FilterList.TAGS_STRIP[i])){
	        			stripTag = true;
	        			break;
	        		}
				}
	        	if(stripTag){
		        	for (int i = 0; i < tag.getAttributeCount(); i++ ){	        		
		        		processAttribute(tag, tag.getAttributeName(i), tag.getAttributeValue(i));
		        	}
	        	}
				context.currentBuffer().append(">");
	        } else 
	        if (tag.getType() == Tag.CLOSE) {
	        	tag.writeTo(context.currentBuffer());
	        } else 
	        if (tag.getType() == Tag.EMPTY) {
	        	context.currentBuffer().append("<").append(tag.getName());
	        	
				boolean stripTag = false;
	        	for (int i = 0; i< FilterList.TAGS_STRIP.length; i++){
	        		if(tag.getName().equalsIgnoreCase(FilterList.TAGS_STRIP[i])){
	        			stripTag = true;
	        		}
				}
	        	if(stripTag){
		        	for (int i = 0; i < tag.getAttributeCount(); i++ ){	        		
		        		processAttribute(tag, tag.getAttributeName(i), tag.getAttributeValue(i));
		        	}
	        	}
	        	
	        	context.currentBuffer().append(" />");
	        }
	        
    	}else{
    		processBody(tag);
    	}
    }
    
	/**
	 * This method will check if there is a body tag before
	 * this rule's input tag or else add
	 * one. This is to ensure that we get a proper Aidbar.
	 * 
	 * @param tag The input tag of the rule
	 */
    private void processBody(Tag tag){
    	if (this.params.get("bodyProcessed").equals("false"))
    		BodyRule.processBody(tag, this.params, context.currentBuffer());
    }
}

