/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.util.Properties;

import com.opensymphony.module.sitemesh.html.Tag;

/**
 * This rule will rewrite all the href attributes inside
 * 'a' tags so that the user will stay inside Loband when 
 * clicking on a link.
 */
public class HrefRewritingRule extends AttributeFilterRule {
	
	/**
	 * Main constructor
	 * 
	 * @param params	The general properties object which is made in RuleFilter
	 * @param special	Name of the special site if we are working on one, e.g. "hotmail". 
	 */
	public HrefRewritingRule(Properties params,String special) 
	{
		super(new String[]{"a"}, params, special);
	}

	
	/**
	 * This method will rewrite the tag and also place links to
	 * any images inside the 'a' tags. 
	 * 
	 * @param	tag The 'a' tag
	 */
	public void process(Tag tag){
		processBody(tag);
				
		if (tag.getType() == Tag.OPEN){
			
			int counter = 0;
			if (tag.getName().equalsIgnoreCase("a")){
				counter = 1;
			}
			
			params.put("linkcounter",Integer.toString(counter));
			params.put("original.a.href","");
			
			context.currentBuffer().append("<" + tag.getName() + " ");
			
			for (int i = 0; i < tag.getAttributeCount(); i++){
				if ( tag.getAttributeName(i).equalsIgnoreCase("href")){
					if(tag.getAttributeValue(i) != null){
						
						params.put("original.a.href",tag.getAttributeValue(i));
						
						context.currentBuffer().append("href=\"" + 
								processHrefAttribute(tag, tag.getAttributeValue(i)) + 
								"\"");
					}
				}else{
					processAttribute(tag, 
							tag.getAttributeName(i), tag.getAttributeValue(i));
				}
			}
			context.currentBuffer().append(">");
		}else if(tag.getType() == Tag.CLOSE){
			
			int counter = Integer.parseInt((String)params.get("linkcounter"));
			params.put("linkcounter",Integer.toString(counter -1));
			String imageLocation = params.getProperty("link.to.image");
			if(params.get("image.in.anchor").equals("true"))
			{
				int index = context.currentBuffer().toString().lastIndexOf('>');
				if(!context.currentBuffer().toString().substring(index).matches(".*\\w.*"))
					// this if is to avoid ending up with something like <a href="blah"></a>
					// where blah is the link where clicking on the image should have taken us.
				{
					context.currentBuffer().append((String)params.get("image.in.anchor.description"));
				}
			}
			context.currentBuffer().append("</" + tag.getName() + ">");
			
			if (imageLocation != null && !imageLocation.equals("") 
					&& !imageLocation.equals("badimage")){
				if (params.get("display.link.to.image").equals("true")){
					
					imageLocation = rewriteUrl( tag, imageLocation,(String)params.get("aidbase.param.type.binary"));
				
					if(params.get("image.in.anchor").equals("true"))
					{
						context.currentBuffer().append("<a title =\"Click here to view " +
								"image\" href=\""+ imageLocation +"\" target=\"_new\">[i-"
								+ (String)params.get("image.in.anchor.description") + "]</a>");
					} else
						context.currentBuffer().append("<a title =\"Click here to view " +
							"image\" href=\""+ imageLocation +"\" target=\"_new\">[i]</a>");
				}
			}
			params.put("image.in.anchor", "false");
			params.put("link.to.image","");
		}
	}
	
	/**
	 * This method will process any attribute that is not an href
	 * attribute.
	 * 
	 * @param	tag   The input 'a' tag
	 * @param	name  The name of the attribute
	 * @param	value The value of the attirbute
	 */
	public void processAttribute(Tag tag, String name, String value) 
	{
		super.processAttribute(tag, name, value);
	}
	
	/**
	 * This method will change the href attribute into a loband link.
	 * 
	 * @param tag 	The input 'a' tag
	 * @param value	The value of the href attribute
	 * @return The mangled href attribute as a String
	 */
	private String processHrefAttribute(Tag tag, String value){
		if(value.startsWith("#"))		// do not rewrite internal links
			return value;
		value = rewriteUrl(tag, value, "");
		return value;
	}
	
	/**
	 * This method will check if there is a body tag before
	 * this rule's input tag or else add
	 * one. This is to ensure that we get a proper Aidbar.
	 * 
	 * @param tag The input tag of the rule
	 */
    private void processBody(Tag tag){
    	if (this.params.get("bodyProcessed").equals("false"))
    		BodyRule.processBody(tag, this.params, context.currentBuffer());
    }
}
