/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.util.Map;
import java.util.Properties;

import com.opensymphony.module.sitemesh.html.Tag;

/**
 * This rule will rewrite every link inside an imagemap to
 * be a loband link. The rewriten links will be placed in an
 * Map object so that they later on can be placed at the original
 * point in the document (the image with the right usemap attribute).
 */
public class ImageMapRule extends AttributeFilterRule {
	
	private Properties params = null;
	private Map imageMap = null;
	private boolean map = false;
	private String mapName="";
	
	/**
	 * Main constructor
	 * 
	 * @param params	The general properties object which is made in RuleFilter
	 * @param imageMap	The Map object where all the links will be put
	 * @param special	Name of the special site if we are working on one, e.g. "hotmail"
	 */
	public ImageMapRule(Properties params, Map imageMap, String special)
	{
		super(new String[]{"map","area"}, params, special);
		this.params = params;
		this.imageMap = imageMap;
	}
	
	/**
	 * This method will rewrite all the links and then place 
	 * them inside the Map object.
	 * 
	 * @param tag	The input map/area tag
	 */
	public void process(Tag tag) {
		String href = "";
		if (tag.getName().equalsIgnoreCase("map")) {
			if (tag.getType() == Tag.OPEN){
				if (tag.hasAttribute("name")) {
					mapName = tag.getAttributeValue("name");
					map = true;
					imageMap.put(mapName,"");
				}
			} else if (tag.getType() == Tag.CLOSE) {
				map = false;
			}
		} else if (tag.getName().equalsIgnoreCase("area")){
			if (tag.hasAttribute("href") && map){
				href = tag.getAttributeValue("href");
				href = rewriteUrl(tag, href, "");
				String value = (String) imageMap.get(mapName);
				value = value + href + " ";
				imageMap.remove(mapName);
				imageMap.put(mapName,value);
			}
		}
	}

}
