/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.io.IOException;
import java.util.Properties;

import org.aidworld.aidbase.webui.util.WebUtils;

import com.opensymphony.module.sitemesh.html.Tag;
import com.opensymphony.module.sitemesh.html.util.CharArray;

/**
 * This rule is only used for special sites which have 
 * their own CSS stylsheet. It will replace the original
 * link to the stylesheet with a link to our stripped version.
 */
public class LinkFilterRule extends AttributeFilterRule{
	private String special = "";
	private boolean filtering = true;
	/**
	 * Main constructor
	 * 
	 * @param special The name of the special site
	 */
	public LinkFilterRule(Properties params,
			String special) {
		super(new String[] {"link"}, params, special);
		if(params.get("aidbase.filter.disable")!=null)
		{
			filtering=false;
		}
		this.special = special;
	}
	
	/**
	 * This method will change the link tag to point 
	 * it to our version of the stylesheet. The name of 
	 * our stylesheet is the name of the special site 
	 * with .css at the end.
	 * 
	 * @param tag The original link tag
	 */
	public void process(Tag tag) {
		CharArray buffer = context.currentBuffer();
		
		if(filtering && (special.length() >0)) {
			buffer.append("<");
			buffer.append(tag.getName());
			buffer.append(" rel=\"stylesheet\"");
			try{
			buffer.append(" href=\"" + WebUtils.getApplicationUrl() +"/css/"+special+".css\" />");
			} catch (IOException e) {}
		}
		else
		{
			super.process(tag);			
		}
	}


	public void processAttribute(Tag tag, String name, String value) 
	{
		if (name.equalsIgnoreCase("href")) {
			value = rewriteUrl(tag, value, (String)params.get("aidbase.param.type.binary"));		
		}
		super.processAttribute(tag, name, value); 
	}

}
