/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.util.Properties;

import com.opensymphony.module.sitemesh.html.BasicRule;
import com.opensymphony.module.sitemesh.html.HTMLProcessorContext;
import com.opensymphony.module.sitemesh.html.StateChangeListener;
import com.opensymphony.module.sitemesh.html.Tag;
import com.opensymphony.module.sitemesh.html.util.CharArray;

/**
 * This rule will removetags and their complete contents 
 * including any other tags.
 */
public class RemoveTagAndContentsRule extends BasicRule 
 implements StateChangeListener {
	private Properties params = null;
	
	/**
	 * Main constructor
	 * 
	 * @param args		A list of tags you want to remove
	 * @param params 	The general properties object which is made in RuleFilter
	 */
    public RemoveTagAndContentsRule(String[] args, Properties params) 
    {
    	super(args);
    	this.params = params;
    }
    
    private int pushCounter = 0;
    
    
    /**
     * Pushes a new buffer on the context. So that the 
     * content of the tag ends up in the new buffer. And pops
     * the buffer when it finds the closing tag.
     * 
     * @param	tag The input tag
     */
    public void process(Tag tag) {
    	if (tag.getType() == Tag.OPEN) {
    		//write all that follows to a new discardable buffer
    		//this seems a bit wasteful since all we want to do is
    		//ignore processing until we hit the end tag ... but instead
    		//we're processing everything and then discarding it when we
    		//hit the end tag ...
    		//but I guess normally there shouldn't be too many tags inside
    		//the ones we want to discard
    		context.pushBuffer(new CharArray(1024));
    		pushCounter++;
        	processBody(tag);

    	} else if (tag.getType() == Tag.CLOSE) {
    		//when we hit the closing tag we switch back to the previous
    		//buffer, discarding everything since the opening one
    		if (pushCounter > 0) {
    			context.popBuffer();
    			pushCounter--;
    		}
    	}
    }
    
    /**
     * This method gets called when the all the html processing
     * is finished. If there still is a pushed buffer write it 
     * to the main buffer.
     * 
     * @param context The pipeline context
     */
    public void stateFinished(HTMLProcessorContext context) {
    	while(pushCounter>0){
    		CharArray buffer = context.popBuffer();
    		context.currentBuffer().append(buffer);
    		pushCounter--;
    	}
    }
    
	/**
	 * This method will check if there is a body tag before
	 * this rule's input tag or else add
	 * one. This is to ensure that we get a proper Aidbar.
	 * 
	 * @param tag The input tag of the rule
	 */
    private void processBody(Tag tag){
    	if (this.params.get("bodyProcessed").equals("false"))
    		BodyRule.processBody(tag, this.params, context.currentBuffer());

    }
}

