/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import com.opensymphony.module.sitemesh.html.Tag;

/**
 * This rule will replace images with a link to the original image.
 * The link description will be the alt attribute of the tag. If there
 * is no alt attribute this rule will try to make a useful decscription
 * of the href attribute.
 */
public class ReplaceImageWithAltTextRule extends AttributeFilterRule {
	private static String TAG_NAMES[] = {"img","image"};
    private Properties params = null;
    private Map imageMap = null;
    private String special = null;
    private boolean processImage;

    /**
     * Main constructor
     * 
     * @param params	The general properties object which is made in RuleFilter
     * @param imageMap  A Map object with all the imagemap links in it. See ImageMapRule
     * @param special  	Name of the special site if we are working on one, e.g. "hotmail".
     */
    public ReplaceImageWithAltTextRule(Properties params, Map imageMap, 
			String special) {
		super(TAG_NAMES, params, special);
		this.params   = params;
		this.imageMap = imageMap;
		this.special  = special;
	}

    /**
     * This method will replace the image with a link to the original image
     * or remove the image if it is useless. If the image is inside a <a> tag,
     * the link to the original image will be written in HrefRwritingRule.process(Tag tag).
     * 
     * @param tag 	The input img/image tag
     */
	public void process(Tag tag) {
 
		processBody(tag);
		params.put("link.to.image", "");
		
		if (tag.getType() == Tag.CLOSE) return;
		if (specialRemove(tag)) return;
		
		int linkCount = Integer.parseInt(params.getProperty("linkcounter"));
		processImage = processImage(tag);
		
		if (tag.hasAttribute("usemap")){
			processUseMap(tag);
			return;
		}
		
		if ( linkCount > 0 || processImage){
			
			//See if the image has JavaScript Attributes
			String javaScriptAttributes = "";
			for (int i = 0; i < tag.getAttributeCount(); i++){
				if (tag.getAttributeName(i).startsWith("on")){
					javaScriptAttributes += tag.getAttributeName(i) + "=\"" + 
					tag.getAttributeValue(i) + "\" ";
				}
			}
			
			if (!javaScriptAttributes.equals("")){
				if(tag.hasAttribute("alt"))
					context.currentBuffer().append("<button ")
					.append(javaScriptAttributes)
					.append(">")
					.append(tag.getAttributeValue("alt"))
					.append("</button>");					
				else
					context.currentBuffer().append("<button ")
						.append(javaScriptAttributes)
						.append(">")
						.append(getLabelFromHref(tag.getAttributeValue("src")))
						.append("</button>");
				return;
			}
			
			if(linkCount > 0)
			{
				params.put("image.in.anchor","true");
			}

			if (tag.hasAttribute("alt")){
				
				String alt = tag.getAttributeValue("alt");
				if (alt.length() > 0 && !alt.equals("/")){
					if (linkCount > 0){
						if(tag.getAttributeValue("src")!=null)
						{
							params.put("link.to.image", tag.getAttributeValue("src"));
							//writeLabelForOriginalLink(alt);
							params.put("image.in.anchor.description",alt);
						}
					}
					else 
						writeLinkToImage(tag, "[i-" + alt + "]");
				}else if (linkCount > 0){ 
					params.put("link.to.image", tag.getAttributeValue("src"));
					//writeLabelForOriginalLink(tag);
					setParamWithLabelForOriginalLink(tag);
				}
				else if (linkCount <= 0) 
					writeLinkToImage(tag, "default");
				return;
			}
			
			if (linkCount > 0) {
				
				if (processImage){
					if(tag.getAttributeValue("src")!=null) {					
						params.put("link.to.image", 
								tag.getAttributeValue("src"));
					}
				}
				else
					params.put("link.to.image", "badimage");
				
				//writeLabelForOriginalLink(tag);
				setParamWithLabelForOriginalLink(tag);
				
				return;
				
			}
			
			writeLinkToImage(tag, "default");
		}
	}
	
	/**
	 * This method processes any image maps encountered and replaces
	 * them with text links.
	 * 
	 * @param tag	The input img/image tag
	 */
	private void processUseMap(Tag tag){
		String usemap = tag.getAttributeValue("usemap");
		if (usemap.startsWith("#"))
			usemap = usemap.substring(1);

		String imageMapLinks = (String) imageMap.get(usemap);
		if(imageMapLinks != null){
			String[] linkswithHref = imageMapLinks.split(" ");
			for (int i = 0 ; i < linkswithHref.length; i++ ){
				String href = linkswithHref[i];

				try{
					String linkName = 
						getLabelFromHref(URLDecoder.decode(href,"UTF-8"));
				
				context.currentBuffer().append("<a href=\"" + href + "\">[" +
						linkName + "]</a> ");
                
				}catch(UnsupportedEncodingException e){}
			}
		}
		if(params.get("display.link.to.image").equals("true"))
            writeLinkToImage(tag, "default");

	}
	
	/**
	 * If there is an image link this method will add  
	 * the orignal link description (not the link itself) 
	 * to the list of params. This description will then be used
	 * in HrefRewritingRule.process(Tag tag).
	 * 
	 * @param tag	The input img/image tag
	 */
	private void setParamWithLabelForOriginalLink(Tag tag)
	{
		if(tag.hasAttribute("src"))
			params.put("image.in.anchor.description",getLabelFromHref(tag.getAttributeValue("src")));
		else
			params.put("image.in.anchor.description","link");
	}

	
	/**
	 * If there is an image link this method will write  
	 * the orignal link description (not the link itself) 
	 * to the main buffer.
	 * 
	 * @param tag	The input img/image tag
	 */
	private void writeLabelForOriginalLink(Tag tag){
		if(tag.hasAttribute("src"))
			context.currentBuffer().append(getLabelFromHref(tag.getAttributeValue("src")));
		else
			context.currentBuffer().append("link");
	}

	/**
	 * If there is an image link this method will write  
	 * the orignal link description (not the link itself) 
	 * to the main buffer.
	 * 
	 * @param label	The description of the link
	 */
	private void writeLabelForOriginalLink(String label){
			context.currentBuffer().append(label);
	}
	
	/**
	 * This method will write the link to the original image
	 * to the main buffer.
	 * 
	 * @param tag	The img/image input tag
	 * @param label The description of the link
	 */
	private void writeLinkToImage(Tag tag, String label){
		if (label.equals("default") && tag.hasAttribute("src")){
			writeLinkToImage(tag, "[i-" + 
				getLabelFromHref(tag.getAttributeValue("src")) + "]");
			return;
		}
		else if (label.equals("default")){
			writeLinkToImage(tag, "[i]");
			return;
		}
		
		if(tag.hasAttribute("src")){
            String src = tag.getAttributeValue("src");
            
            src=rewriteUrl(tag, src, (String)params.get("aidbase.param.type.binary"));
            src = "<a title=\"click here to view image\" href=\""
            	+ src + "\" target=\"_new\">" + label + "</a>";
            
            context.currentBuffer().append(src);
            // params.put("link.to.image", tag.getAttributeValue("src"));
		}
	}
	
	/**
	 * This method will retrieve a useful label/description
	 * from an href attribute
	 * 
	 * @param href	The href attribute
	 * @return The label as string
	 */
	private String getLabelFromHref(String href){
        boolean absoluteLink = false;
        
        if( href.endsWith("/") ){
                href = href.substring(0, href.length() - 1 );
        }
        if(href.startsWith("http://") || href.startsWith("https://")){
        	
        	absoluteLink = true;
            
        	if (href.startsWith("https")) href = href.substring(8);
            else href=href.substring(7);

        }
        
        if (href.startsWith("www.")){
        	absoluteLink = true;
        	href = href.substring(4);
        }

        if( href.indexOf("/") != -1 && href.indexOf(".") != -1 ){
        	
        	if (href.lastIndexOf(".") > href.lastIndexOf("/"))
        		href = href.substring(href.lastIndexOf("/") + 1 ,href.lastIndexOf("."));
        	else{
        		href = href.substring(href.lastIndexOf("/") + 1 );
        	}

        	if (href.indexOf("?") != -1 || href.indexOf("&") != -1
        			|| href.indexOf("=") != -1 || href.length() > 20)
        		href = "link";
        	
        } else if(href.indexOf('/') != -1) {
        	href = href.substring(href.lastIndexOf('/') + 1);
        
		}else if(href.indexOf(".") != -1 && !absoluteLink) {
        	href = href.substring(0,href.indexOf("."));
        	
        }else if( !absoluteLink || href.indexOf("?") != -1 || href.indexOf("&") != -1
        		|| href.indexOf("=") != -1 || href.length() > 20){
        	href = "link";
        }

        return href;
}

	/**
	 * This method will check if the image is useful by using
	 * its height and width attributes. It also checks it against a 
	 * list in FilterList.
	 * 
	 * @param tag	The input img/image tag
	 * @return True if the image is usefull
	 */
	private boolean processImage(Tag tag){
		if (tag.hasAttribute("src")){
			
			String src = tag.getAttributeValue("src");

			for ( int i = 0; i < FilterList.IMAGES_REMOVE.length; i++ )
				if (src.indexOf(FilterList.IMAGES_REMOVE[i]) != -1)
					return false;
				
			if (src.startsWith("http") || src.indexOf("/") != -1){
				src = src.substring(src.lastIndexOf("/") + 1 );
			}
			if (src.length() < 7){
				return false;
			}
		}

		try{
			if (tag.hasAttribute("height") && 
				(tag.getAttributeValue("height").indexOf("%") == -1))
				if (Integer.parseInt(tag.getAttributeValue("height"))
						<= 5)
				return false;
		
		
			if (tag.hasAttribute("width") && 
				(tag.getAttributeValue("width").indexOf("%") == -1))
				if (Integer.parseInt(tag.getAttributeValue("width")) <= 5)
					return false;
					
		}catch(NumberFormatException e){/*If parsing of integer goes wrong*/}	

		return true;
	}
	
	/**
	 * Checks if the image is on the one of the special lists in
	 * Filterlist. 
	 * 
	 * @param tag	The input img/image tag
	 * @return True if the image is on one of the lists
	 */
	private boolean specialRemove(Tag tag){
		
		if (!tag.hasAttribute("src")) return false;
		
		Map specialImageMap = null;
				
		if(special.equals("hotmail")){
			specialImageMap = FilterList.HOTMAIL;
		}else if(special.equals("yahoo_mail")){
			specialImageMap = FilterList.YAHOO_MAIL;
		}else if (special.equals("google")){
			specialImageMap = FilterList.GOOGLE;
		}
		
		if (specialImageMap == null) return false;

		for (Iterator specialNames = specialImageMap.keySet().iterator();
				specialNames.hasNext(); ){
			String specialImage = (String)( specialNames.next() );
			if ( tag.getAttributeValue("src").indexOf(specialImage) != -1){
				context.currentBuffer().append((String)(specialImageMap.get(specialImage)));
				return true;
			}
		}
		return false;
	}
	
	/**
	 * This method will check if there is a body tag before
	 * this rule's input tag or else add
	 * one. This is to ensure that we get a proper Aidbar.
	 * 
	 * @param tag The input tag of the rule
	 */
    private void processBody(Tag tag){
    	if (this.params.get("bodyProcessed").equals("false"))
    		BodyRule.processBody(tag, this.params, context.currentBuffer());
    }

}
