/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.util.Properties;

import org.aidworld.aidbase.filter.JavaScriptRewriter;
import org.aidworld.aidbase.webui.util.WebUtils;

import com.opensymphony.module.sitemesh.html.Tag;
import com.opensymphony.module.sitemesh.html.util.CharArray;

/**
 * This filter will pass on any internal or external javascript to
 * the JavaScriptRewriter.
 */
public class ScriptRule extends AttributeFilterRule {
	private final static String SCRIPT_TAG = "script";
	private Properties params = null;
	private String special = "";
	
    /**
     * Main constructor
     * 
     * @param	params	The general properties object which is made in RuleFilter
	 * @param	special	Name of the special site if we are working on one, e.g. "hotmail". 
	 */
    public ScriptRule(Properties params,String special) {
    	super(new String[]{SCRIPT_TAG}, params, special);
    	this.params  = params;
    	this.special = special;
    }
    
    int pushCounter = 0;
    
    /**
     * Redirect all the javascript to the JavaScriptRewriter
     * 
     * @param tag The input script tag 
     */
    public void process(Tag tag) {
    	if (tag.getType() == Tag.OPEN) {
    		params.put("pageMightDisplayBadly","true");
    		context.currentBuffer().append("<script ");
    		writeAttributes(tag);
    		context.currentBuffer().append(">");
    		context.pushBuffer(new CharArray(1024));
    		pushCounter++;
    		params.put("script.tag", Integer.toString(pushCounter));
    	} else if (tag.getType() == Tag.CLOSE) {
    		String scriptcontents = null;
    		if (pushCounter > 0) {
    			scriptcontents = context.popBuffer().toString();

    			scriptcontents = JavaScriptRewriter.rewrite(scriptcontents,params);
    			context.currentBuffer().append(scriptcontents);
    			context.currentBuffer().append(tag.getContents());
    			pushCounter--;
    			params.put("script.tag", Integer.toString(pushCounter));
    		}
    		String javascript = (String) ( params.get("javascript") );
    		if(!javascript.equals("")){
    			try{
    				context.currentBuffer().append("<script type=\"text/javascript\" " +
    							"src=\""+WebUtils.getApplicationUrl()+ "/javascript/" + 
								javascript + ".js\"> </script>");
    			}catch(Exception e){}
    			params.put("javascript","");
    		}
    	}
    	else if (tag.getType() == Tag.EMPTY) {
    		context.currentBuffer().append("<script ");
    		writeAttributes(tag);
    		context.currentBuffer().append("/>");
    	}
    }
        
    /**
     * Rewrites the src attribute so that it will go through loband.
     * If it is a special site it will add a rewritten version of the 
     * javascript.
     * 
     * @param tag	The input script tag
     */
    private void writeAttributes(Tag tag){
		for (int i = 0; i < tag.getAttributeCount(); i++) {
			if (tag.getAttributeName(i).equalsIgnoreCase("src")){
				String value = tag.getAttributeValue(i);
				if(value.indexOf("mailcommonlib.js") >-1
						&& special.equals("yahoo_mail")){
					
					params.put("javascript","yahoo_mail");
					
				}else if(value.indexOf("hotmail_") >-1 
						&& special.equals("hotmail")){
					
					params.put("javascript","hotmail");
					
				}else if(value.indexOf("010main.js") >-1
						&& special.equals("reliefweb")){
					
					params.put("javascript","reliefweb");
				}
				context.currentBuffer().append("src=\"" 
						+ rewriteUrl(tag, tag.getAttributeValue("src"),
						        (String)params.get("aidbase.param.type.script"))
								+ "\" ");
			}else
				context.currentBuffer().append(tag.getAttributeName(i) + "=\""
						+ tag.getAttributeValue(i) + "\" ");
				
		}
    }
    
}

