/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.util.Properties;

import com.opensymphony.module.sitemesh.html.Tag;

/**
 * This rule will rewrite the src attribute from img, image, frame and 
 * iframe tags so that they will go through loband.
 */
public class SrcRewritingRule extends AttributeFilterRule {
	private Properties params = null;
	
	/**
	 * Main constructor
	 * 
	 * @param	params	The general properties object which is made in RuleFilter
	 * @param	special	Name of the special site if we are working on one, e.g. "hotmail".
	 */
	public SrcRewritingRule(Properties params,String special) 
	{
		
		super( (params.get("aidbase.filter.disable")==null) ? 
			new String[]{"frame","iframe","img","image"} : 
			new String[]{"frame","iframe", "img","image","input","td"} 
			, params, special);

		this.params = params;
	}
	
	/**
	 * This method will change the src attribute and pass other attributes
	 * on to AttributeFilterRule.
	 * 
	 * @param	tag 	The input frame/iframe tag
	 * @param   name	The name of the attribute
	 * @param 	value	The value of the attribute
	 */
	public void processAttribute(Tag tag, String name, String value) 
	{
		if (name.equalsIgnoreCase("src") || name.equalsIgnoreCase("background")) {

			if(tag.getName().equalsIgnoreCase("frame")
					|| tag.getName().equalsIgnoreCase("iframe")) 
			{				
				value = rewriteUrl(tag, value, "");
			}
			else
			{	// image / img
				value = rewriteUrl(tag, value, (String)params.get("aidbase.param.type.binary"));
			
			}			
		}
		super.processAttribute(tag, name, value); 
	}
	
	/**
	 * This method will check if there is a body tag before
	 * this rule's input tag or else add
	 * one. This is to ensure that we get a proper Aidbar.
	 * 
	 * @param tag The input tag of the rule
	 */
    private void processBody(Tag tag){
    	if (this.params.get("bodyProcessed").equals("false"))
    		BodyRule.processBody(tag, this.params, context.currentBuffer());
    }

}
