/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.filter.rulefilter;

import java.util.Properties;

import com.opensymphony.module.sitemesh.html.Tag;
import com.opensymphony.module.sitemesh.html.util.CharArray;


/**
 * Remove a tag replacing it with some substitute text and optionally the contents
 * of one or more attributes (e.g. ALT, TITLE etc)
 */
public class SubstituteTagWithText extends RemoveTagAndContentsRule {
	String[] attributesToUse;
	String substitutionText;

	/**
	 * Main constructor
	 * @param tagName the name of the tag to apply this rule to
	 * @param substitutionText the name of the object / element that has been removed
	 * 						this will be placed into the generic substitution text 
	 * @param attributesToUse names of any attributes whose contents should be added on to
	 * 						the end of the generic substitution text
	 * @param params The general properties object which is made in RuleFilter
	 */
	public SubstituteTagWithText(String tagName, String substitutionText, 
			String[] attributesToUse, Properties params) {
		super(new String[]{tagName}, params);
		this.substitutionText = substitutionText;
		this.attributesToUse = attributesToUse;
	}

	/**
	 * This method will remove the tag and replace it with text
	 * from the specified attributes in the contsructor.
	 * 
	 * @param tag The input tag
	 */
	public void process(Tag tag) {
        if (tag.getType() == Tag.OPEN) {
        	CharArray buffer = context.currentBuffer();
        	buffer.append("[<i>Loband: ");
            buffer.append(substitutionText);
            buffer.append(" Removed -");
            
            for (int i=0;i<attributesToUse.length;i++) {
            	if (tag.hasAttribute(attributesToUse[i])) {
            		buffer.append(" ");
            		buffer.append(tag.getAttributeValue(attributesToUse[i]));
            	}
            }

            buffer.append("</i>]");
        }
		super.process(tag);
	}
	
}
