/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.request;

import java.net.URL;
import java.util.Map;

import javax.servlet.ServletRequest;

import org.aidworld.aidbase.core.Timer;
import org.aidworld.aidbase.error.RecursionDepthException;
import org.aidworld.aidbase.error.RecursionNestingException;
import org.aidworld.aidbase.util.PartInfo;
import org.apache.commons.httpclient.HttpState;

/**
 *TODO: Javadoc
 */
public interface Request {
    Object getParameter(String name);
    Map getParameters();
    void setParameters(Map params);
    String[] getOrderedParamNames();
    
    Map getHeaders();
    void setHeaders(Map headers);
    
    String getContextPath();
    ServletRequest getRequest();
    
    URL getURL();
    void setURL(URL url);
    
    void increaseDepth() throws RecursionDepthException;
    void decreaseDepth() throws RecursionNestingException;
    int getDepth();
    
    HttpState getState();
    void setState(HttpState state);
    
    public final class Method {
    	private final String name;
    	private Method(String name) { this.name = name; }
    	public String toString()    { return name; }
    };

    public static final Method POST_METHOD = new Method("POST");
    public static final Method GET_METHOD  = new Method("GET");
    
    void setMethod(Method method);
    Method getMethod();
  
    public PartInfo[] getParts();
    public void setParts(PartInfo[] parts);
    
    public Timer getTimer();
    
    public void   setFilterParams(Map params);
    public Map    getFilterParams();
    public String getHttpsHost();
    public String getHttpHost();
    
    // The Simulator (old name: Throttler) restricts bandwidth to
    // bps setting from _ab_bps url parameter
    public boolean getSimulatorOnOff();
    public void setSimulatorOnOff(boolean on);
   
    public int getSimulatorBps();
    public void setSimulatorBps(int bps);
    
    public int getSimulatorKey();
    public void setSimulatorKey(int key); 

    public String getRequestType();
    public void setRequestType(String s);

    public void setFiltering(boolean x);
	public boolean getFiltering();
	
	public Request copy();
}
