/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.request;

import java.net.URL;
import java.util.Hashtable;
import java.util.Map;

import javax.servlet.ServletRequest;

import org.aidworld.aidbase.webui.util.WebUtils;

/**
 * This kind of Request represents a request that was generated
 * arbitrarily, i.e not via a Servlet. No ServletRequest object
 * is required. Suitable for use by unit tests, or any use of Aidbase
 * code outside of a servlet environment. 
 */
public class TestRequest extends BasicRequest 
{
	public TestRequest ()
	{
		super(true);
	}
	
	/**
	 * Main constructor
	 * 
	 * @param url			The url of the requested page
	 * @param contextPath	The context path of the application
	 * @param parameters	The parameters
	 * @throws Exception If it has problems reading the properties file
	 */
	public TestRequest (URL url, String contextPath, Map parameters) 
	throws Exception
	{
        super(url, contextPath);
        setRequestType("page");
        if (parameters == null) 
        	parameters = new Hashtable();
        parameters.put(WebUtils.getAidbaseUrlParam(), 
        		new String[]{url.toString()});
		setParameters(parameters);
        setHeaders(new Hashtable());
	}
	
	/**
	 * Returns the servlet request, but because this is a TestRequest
	 * used for outside servlets it will return null
	 * 
	 * @return null
	 */
	public ServletRequest getRequest() {
		return null;
	}
	
	public Request copy() {
		TestRequest req = new TestRequest();
		super.copyImpl(req);
		return req;
	}
}
