package org.aidworld.aidbase.util;
import java.awt.Color;
import java.io.File;
import java.io.IOException;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.jrobin.core.FetchData;
import org.jrobin.core.FetchRequest;
import org.jrobin.core.RrdDb;
import org.jrobin.core.RrdDef;
import org.jrobin.core.RrdException;
import org.jrobin.core.Sample;
import org.jrobin.graph.RrdGraph;
import org.jrobin.graph.RrdGraphDef;

/*
 * This class accesses the JRobin RRD database.
 * writetoBuffer submits data
 * createDB creates an RRD database
 * makeGraph creates a graph
 * rrdupdate commits an update to the db
 * run prepares everything and calls rrdupdate every heartbeat 
 */

public class CompressCounter extends Thread{
	
	private static boolean ready = false;
	
	private static long Buffer[] = new long[3];
	
	private static Logger LOG = Logger.getLogger(CompressCounter.class);
	private int graphcount = 1;
	private long heartbeat;
	private int graphbeat;
	private static RrdDb rrd;
	private static RrdDb rrdc;
	private boolean running=true; 	//refers to infinite loop
	private int nservers=0;
	private String[] servernames;
	private Sample sample;
	private Sample csample;
	
	private static long lasttime=0;
	private static long lrequests=0;
	private static long lbytesin=0;
	private static long lbytesout=0;
	
	private static CompressCounter instance=new CompressCounter();
		
	private CompressCounter(){		
		super("CompressCounter");
		start();
	}
	
	public static CompressCounter getInstance()
	{
		return instance;
	}
	
	public static void writetoBuffer(int type, long value){
		synchronized(Buffer)
		{
			Buffer[type] += value;
		}
	}

	public void rrdupdate() throws RrdException, IOException{
		synchronized(Buffer){
			//Set sample
			
			long timenow=(System.currentTimeMillis() + 500L)/ 1000L;
//			Send data to remote dbs
			propagate(timenow, Buffer[0], Buffer[1], Buffer[2]);
			sample.setTime(timenow);
			sample.setValue("requests",Buffer[0]);
			Buffer[0]=0;
			sample.setValue("bytesin",Buffer[1]);
			Buffer[1]=0;
			sample.setValue("bytesout",Buffer[2]);
			Buffer[2]=0;
			//Update local db
			sample.update();
			
		}
	}

	//Pass data to other servers
	public void propagate(long timenow, long requests, long bytesin, 
			long bytesout){
		//Send to other servers
		try {
			int i;
			for(i=0; i<nservers; i++)
			{
				Utils.geturl(servernames[i] 
								+ "/loband/ccservlet?"
								+ "requests=" + requests
								+ "&bytesin=" + bytesin
								+ "&bytesout=" + bytesout);
			}
		} catch (Exception e)
		{
			
		}
	}
	
//	Write data from other servers to the combined database
	// merge together samples from the same second
	// since cannot call csample.update() more than once for the same
	// time value
	public void receiveRemoteData(long timenow, long requests, long bytesin, 
			long bytesout){
		
		if (ready)
		{
			synchronized(csample)
			{
				if(timenow != lasttime)
				{
					if(lasttime != 0)
					{
						try {
							csample.setTime(lasttime);
							csample.setValue("requests",lrequests);
							csample.setValue("bytesin",lbytesin);
							csample.setValue("bytesout",lbytesout);
							csample.update();
						}
						catch (Exception e) {
							LOG.error("Problem updating combined database:" + e);
						}
					}
					lasttime=timenow;
					lrequests=requests;
					lbytesin=bytesin;
					lbytesout=bytesout;
				}
				else
				{
					lrequests+=requests;
					lbytesin+=bytesin;
					lbytesout+=lbytesout;
				}
			}
		}
	}
	
	public void run(){
		try{
			//Initialise database and sample
			Properties props = Utils.getAppProps();
			if(props.getProperty("application.rrdlogging").equals("true")){
				try{heartbeat = Long.parseLong(props.getProperty("application.rrdheartbeat"));}
				catch(NumberFormatException e){heartbeat=30;}
				
				//Local db
				String RRD_FILE = props.getProperty("application.rrdfile");
				if(!(new File(RRD_FILE).exists())){createDB(RRD_FILE);}
				rrd = new RrdDb(RRD_FILE);
				sample=rrd.createSample();
				
				//Combined db
				String RRD_COMBO_FILE = props.getProperty("application.rrdcombofile");
				if(!(new File(RRD_COMBO_FILE).exists())){createDB(RRD_COMBO_FILE);}
				rrdc = new RrdDb(RRD_COMBO_FILE);
				csample=rrdc.createSample();
							
				String s="application.rrdserver1";
				String name;
				while((name=props.getProperty(s))!=null)
				{
					nservers++;
					s="application.rrdserver" + (nservers + 1);					
				}
				int i;
				servernames = new String[nservers];
				for(i=0; i<nservers; i++)
				{
					s="application.rrdserver" + (i+1);
					name=props.getProperty(s);
					servernames[i]=name;					
				}
				
				ready = true;
				
				//Set shutdown hook to update system shutdown
				Runtime.getRuntime().addShutdownHook(new Thread(){
					public void run() {
						try{
							LOG.debug("Compression Counter shutdown " +
									"hook called.");
							rrdupdate();
							rrd.close();
						}
						catch(IOException e){
							LOG.error("File IO error whilst shutting " +
									"down in hook. Very recent data " +
									"may have been lost: "+e);
						}
						catch(RrdException e1){	
							LOG.error("Database access error when " +
									"writing during hook shutdown. " +
									"Very recent data may have been " +
									"lost: "+e1);
						}
						catch(NullPointerException e2){
							LOG.error("Null pointer exception whilst " +
									"shutting down in hook - rrd " +
									"deregistered? Very recent data " +
									"may have been lost: "+e2);
						}
					}	
				});
				
				//Write to the database every heartbeat seconds
				while(running){
					LOG.debug("CC Waiting for "+heartbeat+" seconds.");
					Thread.sleep(heartbeat*1000);
					LOG.debug("CC Waiting Over");
					try{rrdupdate();}
					catch(RrdException e1){	
						LOG.warn("Database access error when writing. " +
								"This data may have been lost: "+e1);
					}
					catch(IOException e2){
						LOG.error("File IO error whilst writing. " +
								"This data may have been lost: "+e2);
					}
				}
			}
		}
		catch(RrdException e1){
			LOG.error("Compression Counter died: database access error " +
					"whilst initialising: "+e1);
		}
		catch(IOException e2){
			LOG.error("Compression Counter died: file IO error: "+e2);
		}
		catch(InterruptedException e3){
			LOG.error("Compression Counter died: thread interrupted: "+e3);
		}
		finally{
			try{if(rrd!=null){
					rrdupdate();
					rrd.close();
				}
			}
			catch(IOException e){
				LOG.error("Compression Counter file IO error whilst " +
						"shutting down after error. Very recent data " +
						"may have been lost: "+e);
			}
			catch(RrdException e1){	
				LOG.error("Database access error when writing during " +
						"shutdown after error. Very recent data may " +
						"have been lost: "+e1);
			}
		}
	}
	
	public void shutdown() {
		LOG.error("got shutdown message");
		running=false;
		interrupt();
	}

/**Function to count totals from the database for display on a jsp
 * 
**/	
	public static long total(int type){
		
		if(!ready){return -1;}
		long tot=0;
		String name="";
		switch (type){
			case 0:
				name="bytesin";
				break;
			case 1:
				name="bytesout";
				break;
			case 2:
				name="requests";
				break;
		}
	
		try{
			Properties props = Utils.getAppProps();
			long currtime = System.currentTimeMillis()/1000L;
			FetchRequest request = rrd.createFetchRequest("AVERAGE", currtime - 30000, currtime);
			request.setFilter(name);
			FetchData fetchData = request.fetchData();
			
			double value[]=fetchData.getValues(name);
			for(int i = 0; i < value.length ; i++) {
				LOG.warn("CC data - "+name+": "+ value[i]);
				if(!Double.isNaN(value[i])){
					tot += (long) value[i];
				}
			}
		}
		catch(Exception e){LOG.error("CC error:"+e);}
		return tot;
	}

	
	/* Create RRD database
	 * 
	 * RrdDef def = new RrdDef("filename", step length);
	 * def.addDatasource("name", "TYPE", heartbeat, min, max);
	 * def.addArchive("CONSOLIDATION TYPE", max. unknown, generation length, no. of generations);
	 * RrdDb rrd = new RrdDb(def);
	 *
	 * step length = frequency with which to build a primary data point
	 * 
	 * TYPE = type of data (GAUGE, COUNTER, DERIVE, ABSOLUTE)
	 * heartbeat = max. interval between samples before data becomes "unknown"
	 * min, max = minimum, maximum value for data (can be set to unknown)
	 * 
	 * CONSOLIDATION TYPE = how to consolidate (AVERAGE, MIN, MAX, LAST)
	 * max. unknown = max. proportion of unknown data in CDP before CDP
	 *                becomes unknown
	 * generation length = number of PDPs to include in a consolidated data point
	 * no. of generations = number of CDPs to store before overwriting.
	 *  
	 */	

	
	public void createDB(String rrdfile) throws IOException, RrdException{
	
	RrdDef def = new RrdDef(rrdfile, 30);
	def.addDatasource("bytesin", "ABSOLUTE", 60, 0, Double.NaN);
	def.addDatasource("bytesout", "ABSOLUTE", 60, 0, Double.NaN);
	def.addDatasource("requests", "ABSOLUTE", 60, 0, Double.NaN);
	def.addArchive("AVERAGE", 0.95, 2, 720); //minute (store 12 hrs)
	def.addArchive("AVERAGE", 0.95, 20, 288); //10 mins (store 48 hrs)
	def.addArchive("AVERAGE", 0.95, 120, 168); //hour (store 1 wk)
	def.addArchive("AVERAGE", 0.95, 2880, 112); //day (store 16 wks)
	def.addArchive("AVERAGE", 0.95, 20160, 209); //week (store 4 yrs)
	def.addArchive("MAX", 0.95, 2, 720); //minute (store 12 hrs)
	def.addArchive("MAX", 0.95, 20, 288); //10 mins (store 48 hrs)
	def.addArchive("MAX", 0.95, 120, 168); //hour (store 1 wk)
	def.addArchive("MAX", 0.95, 2880, 112); //day (store 16 wks)
	def.addArchive("MAX", 0.95, 20160, 209); //week (store 4 yrs)

	RrdDb rrdmake = new RrdDb(def);
	rrdmake.close();
	}

	/*
	 * days = number of days to graph
	 * offset = number of days to offset from current day
	 */
	
	public static byte[] makeGraph(long startseconds,
			long endseconds, boolean localfile)
	throws IOException, RrdException{ 
		RrdGraphDef def = new RrdGraphDef();
		String RRD_FILE;
	
		Properties props = Utils.getAppProps();
		if(localfile){RRD_FILE = props.getProperty("application.rrdfile");}
		else{RRD_FILE = props.getProperty("application.rrdcombofile");}
		def.setTimePeriod(startseconds, endseconds);
		def.setTitle("Loband graph"); //Main title
		def.setVerticalLabel("Bytes"); //y-axis label
		// How does it choose which average?
		def.datasource("in", RRD_FILE, "bytesin", "AVERAGE");
		def.datasource("out", RRD_FILE, "bytesout", "AVERAGE");
		def.area("out", Color.GREEN, "output traffic");
		def.line("in", Color.BLUE, "input traffic");
		def.gprint("in", "AVERAGE", "AvgIn=@2 @sbytes");
		def.gprint("in", "MAX", "MaxIn=@2 @sbytes@r");
		def.gprint("out", "AVERAGE", "AvgOut=@2 @sbytes");
		def.gprint("out", "MAX", "MaxOut=@2 @sbytes@r");
		def.setAntiAliasing(false);
		RrdGraph g = new RrdGraph(def);
		return g.getGIFBytes(400,200);
	}
	
	public static byte[] makeratioGraph(long startseconds,
			long endseconds, boolean localfile)
	throws IOException, RrdException{ 
		RrdGraphDef def = new RrdGraphDef();
		String RRD_FILE;
	
		Properties props = Utils.getAppProps();
		if(localfile){RRD_FILE = props.getProperty("application.rrdfile");}
		else{RRD_FILE = props.getProperty("application.rrdcombofile");}
		def.setTimePeriod(startseconds, endseconds);
		def.setVerticalLabel("%age"); //y-axis label
		def.datasource("in", RRD_FILE, "bytesin", "AVERAGE");
		def.datasource("out", RRD_FILE, "bytesout", "AVERAGE");
		def.datasource("comp","100,out,100,*,in,/,-");
		def.area("comp", Color.BLUE,"compression ratio");
		def.setAntiAliasing(false);
		RrdGraph g = new RrdGraph(def);
		return g.getGIFBytes(400,80);
	}
		
	public void destroy(){
		try{LOG.error("Compression Counter destroy method called.");
			rrdupdate();
			rrd.close();}
		catch(IOException e){
			LOG.warn("Compression Counter file IO error whilst shutting " +
					"down in destroy. Very recent data may have been " +
					"lost: "+e);
			}
		catch(RrdException e1){	
			LOG.warn("Database access error when writing in destroy. " +
					"Very recent data may have been lost: "+e1);
		}
		catch(NullPointerException e2){
			LOG.warn("Null pointer exception whilst shutting down " +
					"in destroy - rrd presumably deregistered. " +
					"Very recent data may have been lost.");
		}
		super.destroy();
	}
}