/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.util;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;
import java.util.Vector;

import org.aidworld.aidbase.handler.RequestHandler;
import org.apache.log4j.Logger;

/**
 * Utils class to perform certain utility functions.
 * Used by both the application and also directory by the JSP pages.
 */
public class Utils {

    private static Logger LOG = Logger.getLogger(Utils.class);
	
    /**
     * Write a file to the file system
     * 
     * @param filename	The name of the file to be created (including path)
     * @param data		The content that you want in the file
     */
    public static void dumpFile(String filename, String data) {
        try {
            FileWriter fw = new FileWriter(filename);
            fw.write(data);
            fw.close();
        } catch (Exception e) {
            LOG.warn("Error writing "+filename+": "+e);
        }
    }

    /**
     * Write a file to the file system
     * 
     * @param filename	The name of the file to be created (including path)
     * @param data		The content that you want in the file
     */
    public static void dumpFile(String filename, byte [] data) {
        try {
            FileOutputStream fos = new FileOutputStream(new File(filename));
            fos.write(data);
            fos.close();
        } catch (Exception e) {
            LOG.warn("Error writing "+filename+": "+e);
        }
    }

    /**
     * Read a file from the file system
     * 
     * @param filename	The name of the file (including path)
     * @return	The content of the file
     */
    public static byte [] loadFile(String filename) {
        byte [] data = null;
        
        try {
            File file = new File(filename);
            FileInputStream fis = new FileInputStream(file);
            data = new byte [(int)file.length()];
            fis.read(data);
            fis.close();
        } catch (Exception e) {
            LOG.warn("Error writing "+filename+": "+e);
        }
        
        return data;
    }
    
    /**
     * Load a properties file from the properties folder
     * 
     * @param name	Name of the properties file
     * @return	The properties in the properties file
     * @throws FileNotFoundException If the file is not found
     * @throws IOException	If any file operation fails
     */
    public static Properties loadPropertiesFile(String name) 
    throws FileNotFoundException, IOException {
    	
     	Class cl = Utils.class;
    	URL url = cl.getResource(name);
    	if (url == null) {
    		throw new FileNotFoundException(name);
    	}
    	InputStream is = url.openStream(); 
    	Properties props = new Properties();
    	props.load(is);
    	return props;
    }
    
    private static Properties appProps = null;
    
    /**
     * Loads the application.properties properties file
     * 
     * @return The properties inside the properties file
     * @throws Exception If any file operation fails
     */
    public static Properties getAppProps() 
    throws FileNotFoundException, IOException {
    	if (appProps == null)
    		appProps = loadPropertiesFile("/application.properties");
    	return appProps;
    }
    
    /**
     * Gives the standard java temp directory
     * 
     * @return Path to the temp directory
     */
    public static String getTempDirectory () {
    	return System.getProperty("java.io.tmpdir");
    }
    
    /**
     * Returns the directory where the user has asked for logs to be stored,
     * using the application.logsdirectory property in 
     * application.properties.
     * 
     * If no directory has been specified, we return the temp directory,
     * for backwards compatibility.
     * 
     * @return path to the tomcat logs directory, 
     */
    public static String getLogsDirectory () throws IOException{
    	return getAppProps().getProperty("application.logsdirectory", 
                getTempDirectory());
    }
    

	private static int getIntProperty(String name, int def, 
			int min, int max) {
		String valueStr = null;
		
		try {
			valueStr = Utils.getAppProps().getProperty(
					name, ""+def);

			int value = Integer.parseInt(valueStr);
			if (value < min && value > max) {
				LOG.warn("Invalid value for "+name+": "+value);
			} else {
				return value; 
			}
		} catch (NumberFormatException e) {
			LOG.warn("Invalid value for "+name+": "+valueStr);
		} catch (Exception e) {
			LOG.warn("Error reading application.properties: "+e);
		}
		
		return def;
	}
	
	public static int getIoTimeoutMillis() {
		return getIntProperty("application.iotimeout", 10, 0, 120) * 1000;
	}

	public static void setIoTimeoutSeconds(int seconds) throws Exception {
		Utils.getAppProps().setProperty("application.iotimeout", ""+seconds);
	}
	
	public static int getMaxTimeoutMillis() {
		return getIntProperty("application.maxtimeout", 30, 0, 120) * 1000;
	}
		
	public static void setMaxTimeoutSeconds(int seconds) throws Exception {
		Utils.getAppProps().setProperty("application.maxtimeout", ""+seconds);
	}

	private static Vector allHandlers = new Vector();
	
	public static RequestHandler[] getHandlers() {
		return (RequestHandler[])(
				allHandlers.toArray(new RequestHandler[allHandlers.size()])
				);
	}
	
	public static void addHandler(RequestHandler rh) {
		allHandlers.add(rh);
	}
	
	public static void removeHandler(RequestHandler rh) {
		allHandlers.remove(rh);
	}


	public static StringBuffer geturl(String u) throws Exception {
		StringBuffer s = new StringBuffer();
		try {
			// Create an URL instance
			URL url = new URL(u);

			// Get an input stream for reading
			InputStream in = url.openStream();

			// Create a buffered input stream for efficency
			BufferedInputStream bufIn = new BufferedInputStream(in);

			// Repeat until end of file
			for (;;) {
				int data = bufIn.read();

				// Check for EOF
				if (data == -1)
					break;
				else
					s.append((char) data);
			}
		} catch (MalformedURLException mue) {
			LOG.error("Invalid URL: " + u);
		} catch (IOException ioe) {
			LOG.error("I/O Error: " + ioe);
		}
		return s;
	}

	
}





