/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.webui.servlet;

import java.io.IOException;
import java.net.URLDecoder;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import mumail.net.SmtpClient;

import org.aidworld.aidbase.util.Utils;
import org.aidworld.aidbase.webui.util.JSPManager;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;


/**
 * This servlet handles all the feedback that users
 * give.
 */
public class BadPageServlet extends AidbaseServlet{

    private PatternLayout layout = new PatternLayout("%d{dd/MM HH:mm:ss} %-5p %c{1}: %m%n");
	private FileAppender file = null;
    private static Logger LOG = Logger.getLogger(BadPageServlet.class);
	       
	    
    /** Overrides doGet in HttpServlet to delegate to doABGet */
    protected void doABGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
    	String page_url=request.getParameter("_page");
    	String additional=request.getParameter("additional");
    	LOG.debug("Additional :" +additional);
    	Properties props = null;
    	boolean showDefaultError = true;
    	
    	try {
    		props = Utils.getAppProps();
    	
    		String smtphost = props.getProperty("application.smtphost");
    		String smtpport = props.getProperty("application.smtpport");
    		String to		= props.getProperty("application.to");
			String from		= props.getProperty("application.from");
			boolean error 	= false;
		    int smtpportint = 0;
			String subject = "";
			String message = "";
		    
			try {
				if (smtphost == null || smtphost.equals(""))
					throw new Exception( // I18N 3001
						"<code>application.smtphost</code> is not configured");	

				if (smtpport == null || smtpport.equals(""))
					throw new Exception( // I18N 3002
						"<code>application.smtpport</code> is not configured");	

			    try {
			    	smtpportint = Integer.parseInt(smtpport);
			    } catch (Exception x) {
			    	throw new Exception( // I18N 3003
			    		"<code>application.smtpport</code> is not a valid integer");
			    }

			    if (smtpportint < 1 || smtpportint > 65535)
			    	throw new Exception( // I18N 3004
			    		"<code>application.smtpport</code> must be a " +
			    		"valid IPv4 port number, between 1 and 65535");
			    	
				if (from == null || from.equals(""))
					throw new Exception( // I18N 3005
						"<code>application.from</code> is not configured");	
						
				if (to == null || to.equals("")) 
					throw new Exception(
						"<code>application.to</code> is not configured");
			} catch (Exception ex2) {
				
		    	error = true;
		    }

			
	        if (page_url.length() != 0) {	        	
	        	if (additional == null) {
	        		String referrer = request.getHeader("referer");
	        		if (referrer == null){
	        			referrer = "referrer not found"; // I18N 3007
	        		}else {
	        			referrer = referrer.substring(referrer.indexOf("_ab_url=")+8);
	        			referrer = URLDecoder.decode(referrer,"UTF-8");
	        		}
	        		// I18N 3008
	        		message = "Bad page report\n" +
							  "URL: " + page_url + "\n" +
	        				  "Referrer: " + referrer + "\n" +
							  "User: " + request.getRemoteAddr() + "\n\n"; 
	        		subject = "[Bad page report]";
	        		
	        		file = new FileAppender(layout, Utils.getLogsDirectory()
	                		+ "/badpagereports");
	       		 	JSPManager.showJSP(request, response, "/badpagereport/report.jsp");
	       		 	showDefaultError = false;
	        	} else {
	        		String comment = request.getParameter("comment");
	        		String sender = request.getParameter("sender");
	        		String referrer = request.getParameter("referrer");
	        		// I18N 3009
	        		message = "Additional comment\n" +
							  "URL: "+page_url+ "\n" +
							  "Referrer: "+referrer+ "\n"+
							  "Sender: "+sender+"\n" +
							  "User: " + request.getRemoteAddr() + "\n" +
    						  "Comment:\n"+comment + "\n\n";
	        		subject = "[Additional comment]";
	        		file = new FileAppender(layout,Utils.getLogsDirectory()
	        				+ "/additional_report");
	        		JSPManager.showJSP(request, response, 
	        				"/badpagereport/additional.jsp");
	        		showDefaultError = false;
	        	}
	        }

        	LOG.addAppender(file);
        	LOG.setLevel(Level.ERROR);
        	LOG.error(message);
        	LOG.removeAppender(file);

        	if (!error){
				SmtpClient smtp = new SmtpClient(smtphost, smtpportint);
				smtp.submitMail(from,to,subject, message, "UTF-8");
				smtp.Disconnect();
			}
	    } 
    	catch (Exception e) 
		{
    		// I18N 3010
	    	LOG.error("Couldn't send report email: "+e);
	    	if (showDefaultError){
	    		JSPManager.showInternalError(request, response, e, 
	    			"Couldn't send report email");
	    	}
	    	showDefaultError = false;
	    }

	    if (showDefaultError) {
	    	JSPManager.showInternalError(request, response, 
	    			"Couldn't send report email");
	    }
     }
}
    
  

