/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.webui.servlet;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.Iterator;

import org.aidworld.aidbase.core.DocumentTypeChecker;
import org.aidworld.aidbase.error.AidbaseException;
import org.aidworld.aidbase.error.GoDirectException;
import org.aidworld.aidbase.error.InternalFaultException;
import org.aidworld.aidbase.factory.URLContents;
import org.aidworld.aidbase.webui.util.WebUtils;
import org.apache.commons.httpclient.Header;
import org.apache.log4j.Logger;

/**
 * This class checks if a document is a HTML document
 */
public class HtmlChecker implements DocumentTypeChecker{
	private Logger log;
	
	/**
	 * Main constructor
	 * 
	 * @param log	A logger for debugging/logging
	 */
	public HtmlChecker(Logger log) {
		this.log = log;
	}
	
	/**
	 * This method will check if the content type of the document
	 * is text/html
	 * 
	 * @param c	The content of the url
	 */
	
	/* TODO change this so that is checks the content-type against
	 * the path requested - and redirects to the correct path
	 * 
	 * so if the path is /loband/page? but the content-type is
	 * text/javascript then redirect to /loband/script?
	 * 
	 * likewise for binary content types
	 * 
	 * then get  to be called in all cases even when
	 *  not filtering
	 */
	
	public void checkContents(URLContents c) 
	throws AidbaseException
	{
		Header[] headers = c.allHeaders;
		String sMethod= "page";
		String sUrl = "";
		
		for (int i = 0; i < headers.length; i++)
		{
			if (headers[i].getName().equalsIgnoreCase("content-type"))
			{
				String contentType = headers[i].getValue();
				if (!contentType.startsWith("text/html"))
				{
					if (contentType.indexOf("javascript") != -1)
					{
						sMethod = WebUtils.getAidbaseScriptType();
					}
					else
					{
						sMethod = WebUtils.getAidbaseBinaryType();
					}
				}
				if (!sMethod.equals(c.request.getRequestType()))
				{
					try 
					{
						String requestUrl = c.request.getURL().toString();
						sUrl = WebUtils.getLobandCallUrl(requestUrl, sMethod);

						Iterator keys = c.request.getParameters().keySet().iterator();
						
						boolean hasQuestionMark = (sUrl.indexOf("?") != -1);
						
				        while (keys.hasNext())
				        {
				            String param = keys.next().toString();
				            if(param.equals(WebUtils.getAidbaseUrlParam()))
				            	continue;
				            String val = ((String []) c.request.getParameter(param))[0];
				            if(hasQuestionMark)
				            	sUrl += "&";
				            else
				            {
				            	sUrl += "?";
				            	hasQuestionMark = true;
				            }
				            sUrl  += param + "=" +	URLEncoder.encode(val, "UTF-8");
				        }
					} 
					catch (IOException e) 
					{
						throw new InternalFaultException(
								"Unsupported encoding UTF-8", e);
					}

					log.warn("Content type '"+  contentType
		                			+ "': requesting to go to " + sUrl);
			                	
					c.cleanup();
					throw new GoDirectException(sUrl);
				}				
			}
		}
	}
}