/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.webui.simulator;

import java.io.ByteArrayOutputStream;
import java.io.CharArrayWriter;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

import org.apache.log4j.Logger;

/**
 * A wrapper to capture and buffer a Servlet Filter response.
 * @see http://java.sun.com/products/servlet/Filters.html
 */

public class CharResponseWrapper extends HttpServletResponseWrapper {

    private static Logger LOG = Logger.getLogger(CharResponseWrapper.class);
    private boolean isBinary = false;
    private ByteArrayOutputStream m_dataoutput;
    private ServletOutputStreamWrapper m_outputstreamwrapper;
    private CharArrayWriter m_charoutput;

    public CharResponseWrapper(HttpServletResponse response) {
        super(response);
        if (LOG.isDebugEnabled()) LOG.debug("constructor: creating new CharResponseWrapper");

        m_dataoutput = new ByteArrayOutputStream();
        m_outputstreamwrapper = new ServletOutputStreamWrapper(m_dataoutput);
        m_charoutput = new CharArrayWriter();

    }

    public PrintWriter getWriter() throws java.io.IOException{
        isBinary = false;
        return new PrintWriter(m_charoutput);
    }

    public ServletOutputStream getOutputStream() throws java.io.IOException{
        isBinary = true;
        return m_outputstreamwrapper;
    }

	public byte[] getData(){
		if (isBinary){
			return m_dataoutput.toByteArray();
		}
		
		return m_charoutput.toString().getBytes();
	}

    class ServletOutputStreamWrapper extends ServletOutputStream{

        DataOutputStream m_output;

        public ServletOutputStreamWrapper(OutputStream output){
            m_output = new DataOutputStream(output);
        }
        public void write(int b) throws IOException{
            m_output.write(b);
        }
        public void write(byte[] b,int off,int len)throws IOException{
            m_output.write(b, off, len);
        }
        public void flush()throws IOException{
            m_output.flush();
        }
        public void close() throws IOException{
            m_output.close();
        }

        public String toString(){
            return m_output.toString();
        }
    }
}
