/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.webui.util;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.apache.log4j.Logger;

/**
 * Helper class for calling JSP pages from Servlets.
 * Based on code from MIT's DSpace project.
 */

public class JSPManager {
	/* log4j logger */
	private static Logger m_log = Logger.getLogger(JSPManager.class);
	
    /** Display the specified JSP page. */
    public static void showJSP(ServletRequest request, 
    		ServletResponse response, String jsp)
    throws ServletException, IOException
    {
        // For the moment, a simple forward
        request.getRequestDispatcher(jsp).forward(request, response);
    }

    
    /**
     * Display an internal server error message.
     *
     * @param request   the HTTP request
     * @param response  the HTTP response
     * @param e         the Exception
     * @throws ServletException
     * @throws IOException
     */
    public static void showInternalError(ServletRequest request,
       ServletResponse response, Exception e)
        throws ServletException, IOException
    {
        showInternalError(request, response, e, null);
    }
    
    /**
     * Display an internal server error message.
     * 
     * @param request            the HTTP request
     * @param response           the HTTP response
     * @param message            a message to display to the user
     * @throws ServletException
     * @throws IOException
     */
    public static void showInternalError(ServletRequest request,
        ServletResponse response, String message) 
        throws ServletException, IOException {
            
        showInternalError(request, response, null, message, null);    
    }
    
    /**
     * Display an internal server error message.
     * 
     * @param request            the HTTP request
     * @param response           the HTTP response
     * @param message            a message to display to the user
     * @param longMessage        a verbose explanation of the error
     * @throws ServletException
     * @throws IOException
     */
    public static void showInternalError(ServletRequest request,
       ServletResponse response, String message, String longMessage) 
        throws ServletException, IOException {
            
        showInternalError(request, response, null, message, longMessage);    
    }


    /**
     * Display an internal server error message.
     * 
     * @param request            the HTTP request
     * @param response           the HTTP response
     * @param e                  the Exception
     * @param message            a message to display to the user
     * @throws ServletException
     * @throws IOException
     */
    public static void showInternalError(ServletRequest request,
        ServletResponse response, Throwable e, String message) 
        throws ServletException, IOException {
            
        showInternalError(request, response, e, message, null);
    
    }
    
    /**
     * Display an internal server error message.
     * 
     * @param request            the HTTP request
     * @param response           the HTTP response
     * @param e                  the Exception
     * @param message            a message to display to the user
     * @param longMessage        a verbose explanation of the error
     * @throws ServletException
     * @throws IOException
     */
    public static void showInternalError(ServletRequest request,
        ServletResponse response, Throwable e, 
        String message, String longMessage) 
        throws ServletException, IOException 
	{
    	m_log.warn("Showing JSP internal error page in response to "+e);
    
        // Pass the exception to report as an argument to the JSP
        // in case it wants to display the details.
        request.setAttribute("exception", e);

        //Pass in a message to display to the user
        request.setAttribute("message", message);
        
        //Pass in the verbose description of the error
        request.setAttribute("longMessage", longMessage);

        // Show the internal error JSP page
        showJSP(request, response, "/error/internal.jsp");
    }

}