/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.core;

import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;

import junit.framework.TestCase;

import org.aidworld.aidbase.document.Document;
import org.aidworld.aidbase.factory.TestDocumentFactory;
import org.aidworld.aidbase.filter.RuleFilter;
import org.aidworld.aidbase.request.TestRequest;
import org.aidworld.aidbase.webui.util.WebUtils;
import org.xml.sax.SAXException;

import com.meterware.httpunit.ClientProperties;
import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.HttpUnitOptions;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebForm;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;

/**
 * This class is intended to be used as a superclass for HttpUnit
 * tests, providing a standard setUp() method, and useful utility
 * methods.
 */

public class BasedTest extends TestCase {
	public BasedTest() {
	    super();
	}

	public BasedTest( String name ) {
	    super( name );
	}

    private static String 		aidbaseUrl = null;
    // private static Properties 	props = null;

    protected WebConversation 	conversation;
    protected WebRequest 		request;
    protected WebResponse 		response;

	/*
	 * @see TestCase#setUp()
	 */
	public void setUp() throws Exception {
		super.setUp();
	
		aidbaseUrl = WebUtils.getApplicationUrl(); 
		conversation = new WebConversation();
	    request = new GetMethodWebRequest
			(aidbaseUrl + "/main");
	    response = conversation.getResponse( request );
	    
	    HttpUnitOptions.setExceptionsThrownOnScriptError(false);
	    ClientProperties clientProperties = conversation.getClientProperties();
		clientProperties.setAutoRedirect(true);
		clientProperties.setAutoRefresh(true);
		clientProperties.setAcceptGzip(false);
	    //HttpUnitOptions.setScriptingEnabled(false);	
	}

	public String getAidbaseUrl() {
		return aidbaseUrl;
	}
	
    /**
     * Convenience method to submit an Aidbase request, getting back a
     * filtered page in a Response object.
     * @throws Exception
     */
    
    public WebResponse getThroughAidbase(String url)
    throws SAXException, IOException {
	    request = new GetMethodWebRequest
			(aidbaseUrl);
	    response = conversation.getResponse( request );
    	WebForm form = response.getForms()[0];
    	form.setParameter(WebUtils.getAidbaseUrlParam(), url);
        form.submit();
        return conversation.getCurrentPage();
    }
    
    /**
     * Filters the html contained in 'input' through loband and returns the result.
     * The url of 'input' is assumed to be "http://foo".
     * 
     * @param input         Some html
     * @return
     * @throws Exception
     */
    public String getFilteredHtmlString(String input) 
    throws Exception 
    {
        return getFilteredHtmlString(input,
                new URL("http://foo"));
    }

    /**
     * Filters the html contained in 'input' through loband and returns the result.
     * The url of 'input' is assumed to be pageUrl.
     * 
     * @param input         Some html
     * @return
     * @throws Exception
     */
    public String getFilteredHtmlString(String input, URL pageUrl) 
    throws Exception 
    {
        TestDocumentFactory fact = new TestDocumentFactory(input,
                new Hashtable());
        TestRequest req = new TestRequest(pageUrl, WebUtils.getContextPath(),
                new Hashtable());
        RuleFilter filter = new RuleFilter();
        filter.setNextStage(fact);
        Document out = filter.service(req);

        return out.toString();
    }
    
    public WebResponse getTestPageThroughAidbase()
    throws SAXException, IOException
    {
    	return getThroughAidbase(
    			WebUtils.getApplicationUrl() + "/test/test.html");
    }
    
    /**
     * Returns the content of the document at the given URL 'url'.
     * 
     * @param url
     * @return String representing content of document at the URL 'url'.
     * @throws SAXException
     * @throws IOException
     */
    public String getDocumentText(URL url)
    throws SAXException, IOException {
        request = new GetMethodWebRequest(url.toString());
        response = conversation.getResponse( request );
        return conversation.getCurrentPage().getText();
    }
}
