/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.offline.unit;

import java.util.Properties;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;
import org.aidworld.aidbase.util.Utils;
import org.aidworld.aidbase.webui.util.WebUtils;


public class BinaryPipelineTests extends BasedTest{
	
    /**
	 * Standard JUnit constructor.
	 */
	public BinaryPipelineTests(String name) {
		super(name);
	}
	
	/**
	 * Standard JUnit method to build a suite of tests consisting of
	 * all test methods within this class.
	 */
    public static Test suite() {
        return new TestSuite(BinaryPipelineTests.class);
    }
	
    /**
     * check that a page filtered normally comes out the same as a page filtered through the binary pipeline
     * @throws Exception
     */
	public void testBinaryPipeLineDoesntChangeContent() throws Exception{
		Properties applicationProperties = Utils.getAppProps();

		response = conversation.getResponse(WebUtils.getApplicationUrl()+"/test/test.html");
		String normal = response.getText();
	 
		String lobandCall = "http://"
			+ applicationProperties.getProperty("application.host",
			"localhost")
			+ ":"
			+ applicationProperties.getProperty("application.port", "8080")
			+ WebUtils.getLobandCallUrl(WebUtils.getApplicationUrl() + "/test/test.html",
				WebUtils.getAidbaseBinaryType()); 
		
		response = conversation.getResponse(lobandCall
				);
		String binary = response.getText();
		
		assertEquals("Binary pipeline changed the content",normal,binary);
		
	}
	
	public void testBinaryPipeLineSetsCookies() throws Exception{
		Properties applicationProperties = Utils.getAppProps();

		response = conversation.getResponse("http://"
				+ applicationProperties.getProperty("application.host",
				"localhost")
				+ ":"
				+ applicationProperties.getProperty("application.port", "8080")
				+ WebUtils.getLobandCallUrl(
						WebUtils.getApplicationUrl()+ "/test/cookies.jsp",
						WebUtils.getAidbaseBinaryType()));
		
		assertTrue("Cookie wasn't set",response.getText().indexOf("Cookie set OK")!=-1);
	}
	
    public void testBinaryDownloadsGetsTheContentTypeCorrect() throws Exception {
        response = getThroughAidbase(WebUtils.getApplicationUrl()
                + "/test/test.pdf");
        assertEquals("PDF file not treated as pdf: wrong content type",
                "application/pdf", response.getContentType());
    }
}
