/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.offline.unit;

import java.util.Properties;

import junit.extensions.TestSetup;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;
import org.aidworld.aidbase.util.Utils;
import org.aidworld.aidbase.webui.util.WebUtils;
import org.apache.commons.httpclient.Cookie;
import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.cookie.CookieSpecBase;
import org.apache.commons.httpclient.methods.GetMethod;

import com.meterware.httpunit.ClientProperties;
import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.WebConversation;

/**
 * Unit tests for the CookieFilter.
 */

public class CookieFilterTest extends BasedTest {

    private static String deployedUrl = null;
    
	public static Test suite() {
	  	    return new TestSetup(new TestSuite(CookieFilterTest.class)) {
	        protected void setUp() throws Exception{
	    		deployedUrl = WebUtils.getApplicationUrl();
            }
	    };
	}

	public void testCookieResetShouldWork() throws Exception
	{
		ClientProperties cp = conversation.getClientProperties();
    	cp.setAutoRedirect(true);
		
    	response = getThroughAidbase(
				getAidbaseUrl() + "/test/resetCookie.jsp");
		response = conversation.getCurrentPage();
		
//		System.out.print(response.getText());
		assertTrue("Cookie value wasn't reset",
				response.getText().indexOf("dummy = b") != -1);

	}
	
	
	public void testShouldBeAbleToReceiveCookies() throws Exception
	{
		String url = deployedUrl + "/test/cookies.jsp";
		HttpClient client = new HttpClient();
		HttpMethod method = new GetMethod(url);
		
		method.setFollowRedirects(false);
		client.executeMethod(method);
		Cookie[] cookies = client.getState().getCookies();
		
		assertTrue("No cookies set",cookies !=null);
		assertTrue("Empty cookie array",cookies.length > 0);
		
		Header[] response = method.getResponseHeaders();
		boolean set = false;
		
		for (int i = 0; i < response.length; i++) {
			if (response[i].getName().equalsIgnoreCase("set-cookie")){
				set = true;
			}
		}
		assertTrue("Set-cookie not found",set);
	}
	

	/**
	 * HttpClient doesn't allow you to set cookies with a Cookie header,
	 * since it deleted that header and replaces it. You have to mess
	 * about with the HttpState object instead.
	 */

	public void testShouldNotBeAbleToSetMultipleCookiesWithCustomHeader() 
	throws Exception
	{
		String url = deployedUrl + "/test/cookies2.jsp";
		HttpClient client = new HttpClient();
		HttpMethod method = new GetMethod(url);
		
		method.setFollowRedirects(false);
		method.addRequestHeader(new Header("cookie", "a=b;c=d"));
		
		client.executeMethod(method); 
		
		String body = method.getResponseBodyAsString();
		assertFalse("Setting cookie header worked when it should have failed",
				body.equals("Cookies are set"));
			
	}
	
	public void testHttpMethodStrictModeShouldSendOnlyOneCookieHeader()
		throws Exception{
		String url = deployedUrl + "/test/CheckCookies.jsp";
		HttpClient client = new HttpClient();
		HttpMethod method = new GetMethod(url);
		
		Properties props = Utils.getAppProps();
    	String host = props.getProperty("application.host", "dev.aidworld.org");
    	String path = props.getProperty("application.path", "/loband");
    	path = path + "/test/CheckCookies.jsp";
    	
		HttpState state = client.getState();
		state.addCookie(new Cookie(host, "a","true", path, 60, false));
		state.addCookie(new Cookie(host, "b","false",path, 60, false));
		method.setStrictMode(false);
		
		
		client.executeMethod(method); 
		
		String body = method.getResponseBodyAsString();
		
		
		assertEquals("Received wrong number of cookie headers",
				"\n$Version=0; a=true\n"+"$Version=0; b=false\n",body.replaceAll("\\r", ""));
		
		method = new GetMethod(url);
		state = new HttpState();
		client.setState(state);
		client.setStrictMode(true);
		state.addCookie(new Cookie(host, "a","true", path, 60, false));
		state.addCookie(new Cookie(host, "b","false",path, 60, false));
		
		client.executeMethod(method); 
		
		body = method.getResponseBodyAsString();
		
		assertEquals("Received wrong number of cookie headers",
				"\n$Version=0; a=true; b=false\n",body.replaceAll("\\r", ""));
			
	}
	
	public void testHttpClientCookieParsingShouldHandleMultipleCookies() 
	throws Exception 
	{
		Properties props = Utils.getAppProps();
    	String host = props.getProperty("application.host", "dev.aidworld.org");
    	String port = props.getProperty("application.port", "8080");
    	String path = props.getProperty("application.path", "/loband");
    	
    	
	    String cookieHeader = "JSESSIONID=A902AC21AE4F694F7DD7AAA56020F9B9; Path=/aidbase, tested=true; Expires=Tue, 10-Aug-2004 15:09:42 GMT; Path=/aidbase/test/cookies.jsp";
		Cookie[] cookiesOut;
		
		CookieSpecBase spec = new CookieSpecBase();
		cookiesOut = spec.parse(host, Integer.parseInt(port) , path + "/test/cookies.jsp", false, cookieHeader);
		
		assertTrue("Wrong number of cookies",cookiesOut.length == 2);
		assertEquals("Wrong name of first cookie", "JSESSIONID",cookiesOut[0].getName());
		assertEquals("Wrong name of second cookie", "tested",cookiesOut[1].getName());
	}
	
	public void testCookieTesterPageShouldRespondCorrectly()
	throws Exception {
        conversation = new WebConversation();
        request = new GetMethodWebRequest
			(getAidbaseUrl() + "/test/cookies.jsp");
        response = conversation.getResponse( request );
        assertEquals("cookies.jsp reports that cookie was not set",
        		"<html><body>Cookie set OK</body></html>", response.getText());
	}
	
	
	public void testCookieTesterPageShouldWorkThroughAidbase()
	throws Exception {
        response = getThroughAidbase(getAidbaseUrl() + "/test/cookies.jsp");
        assertTrue("cookies.jsp reports that cookie was not set: " 
        		+ response.getText(),
        		response.getText().indexOf("Cookie set OK") != -1);
	}
	
	public void testCookieSecurityShouldNotSeeOtherCookie()
	throws Exception {
        conversation = new WebConversation();
        request = new GetMethodWebRequest
			(getAidbaseUrl() + "/test/cookies.jsp");
        response = conversation.getResponse( request );
        request = new GetMethodWebRequest
			(getAidbaseUrl() + "/test/CookieSecurity.jsp");
        response = conversation.getResponse( request );
        assertEquals("Cookie Security failed",
        		"Cookie not found", response.getText());
	}
	
	public void testCookieStateTestShouldWork()
	throws Exception {
		response = getThroughAidbase(getAidbaseUrl() + "/test/cookies.jsp");
		   assertTrue("Cookie State lost",
        		 response.getText().indexOf("Cookie set OK") != -1);
	}
	
	public void testCookiesAndMetaRefreshShouldWorkThroughAidbase()
	throws Exception {
        response = getThroughAidbase(getAidbaseUrl() + "/test/cookiesMeta.jsp");
        assertFalse("Meta refresh wasn't executed", 
        		response.getText().indexOf("Meta refresh wasn't executed.") != -1);
        assertTrue("cookiesMeta.jsp reports that cookie was not set",
        		response.getText().indexOf("Cookie set OK") != -1);
	}

	public void testCookiesWithSameNamesShouldNotBeOverwritingEachOther()
	throws Exception {
        response = getThroughAidbase(getAidbaseUrl() + "/test/cookiesNamesTest1.jsp");
        if (response.getText().indexOf("Cookies set OK") == -1) {
            fail("cookiesNamesTest1.jsp reports that: "
                    + response.getText());
        }
        
        HttpClient client = new HttpClient();
		HttpMethod method = new GetMethod(
				deployedUrl + "/test/cookiesNamesTest1.jsp");
		
		method.setFollowRedirects(true);
		client.executeMethod(method);
		Cookie[] cookies = client.getState().getCookies();
        
		assertNotNull("No cookies found", cookies);
		
		boolean first = false, second = false;
		for (int i = 0; i< cookies.length; i++){
			if (cookies[i].toString().equals("test=value1"))
				first = true;
			if (cookies[i].toString().equals("test=value2"))
				second = true;
		}
		
        assertTrue("First cookie not set", first);
        assertTrue("Second cookie not set", second);
	}
	
}
