/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.offline.unit;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;
import org.aidworld.aidbase.webui.util.WebUtils;

import com.meterware.httpunit.WebForm;
import com.meterware.httpunit.WebLink;

public class FeedbackTests extends BasedTest {


	public static Test suite() {
		return new TestSuite( FeedbackTests.class );
	}
	

	public void testFeedbackLinkIsGenerated() throws Exception{
		response = getTestPageThroughAidbase();
		assertTrue("No Report Error link generated",
				response.getText().indexOf("Report error") != -1);
	}

	public void testErrorLogging() throws Exception 
	{
    	response = conversation.getResponse(WebUtils.getApplicationUrl()+ "/test/ReadErrorReports.jsp");
    	long oldfilesize = response.getText().length();
    	
    	conversation.setExceptionsThrownOnErrorStatus(false);
    	response = getThroughAidbase("badger");
    	response = conversation.getResponse(WebUtils.getApplicationUrl()+ "/test/ReadErrorReports.jsp");
    	
    	long newfilesize = response.getText().length();
    	
    	assertTrue("Error not logged",newfilesize > oldfilesize);
    }
    
    public void testReportErrorFeedback() throws Exception 
	{
    	response = conversation.getResponse(WebUtils.getApplicationUrl()
    			+ "/test/ReadBadPageReports.jsp");   	
        int oldfilesize = response.getText().length();
               
        response = getThroughAidbase(WebUtils.getApplicationUrl() 
        		+ "/test/test.html");
        WebLink link = response.getLinkWith("Report error");
        link.click();
        
        response = conversation.getResponse(WebUtils.getApplicationUrl()
        		+ "/test/ReadBadPageReports.jsp");
        int newfilesize = response.getText().length();
        assertTrue("BadPage Report not logged",newfilesize > oldfilesize);
        
        String temp = response.getText().substring(oldfilesize,newfilesize); 
        int ref=temp.indexOf("Referrer: ");
        assertTrue(ref!=-1);
        int ref2=temp.indexOf("test.html", ref);
        
        assertTrue(ref2!=-1);
    }
    
    public void testDetailedFeedback() throws Exception 
	{
    	String feedbackComment = "I couldn't see the page about badgers.";
    	
    	response = conversation.getResponse(WebUtils.getApplicationUrl()
    			+ "/test/ReadAdditionalComments.jsp");
    	
        long oldfilesize = response.getText().length();
        
       
        response = conversation.getResponse(WebUtils.getApplicationUrl() 
        		+ "/badpagereport?_page="+ WebUtils.getApplicationUrl()+ "/test/test.html");
        
        
        
        WebForm form = response.getFormWithName("commentform");
        form.setParameter("comment",feedbackComment);
        form.submit();
        response = conversation.getResponse(WebUtils.getApplicationUrl()
        		+ "/test/ReadAdditionalComments.jsp");
        long newfile = response.getText().length();
        assertTrue("Detailed feedback not logged",newfile > oldfilesize);
        
        String temp = response.getText().substring(0,(int)(newfile)-3);
        assertTrue(temp.indexOf(feedbackComment)>-1);
    }
}