package org.aidworld.aidbase.offline.unit;

import java.io.IOException;

import junit.framework.TestCase;

import org.jrobin.core.FetchData;
import org.jrobin.core.FetchRequest;
import org.jrobin.core.RrdDb;
import org.jrobin.core.RrdDef;
import org.jrobin.core.RrdException;
import org.jrobin.core.Sample;

public class JRobinTests extends TestCase {

	/*
	 * This test ensures that consolidation is working correctly
	 * for the "AVERAGE" data type, which is now a totalling function.
	 * 
	 * The first assertion checks that totalling works.
	 * The second assertion checks that consolidation to a second archive
	 * works.
	 * 
	 * TODO: Work out why the first value on each archive is low.
	 */
	
	public void testConsolidateTotals() 
	throws IOException, RrdException
	{	
		String rrdfile = "/tmp/test.rrd";
		RrdDef def = new RrdDef(rrdfile, 1);
		def.addDatasource("bytes", "ABSOLUTE", 60, 0, Double.NaN);
		def.addArchive("AVERAGE", 0.99, 20,  9); // 20 seconds (store 3 mins)
		def.addArchive("AVERAGE", 0.99, 60, 20); // 1 min (store 20 mins)
		RrdDb db = new RrdDb(def);
		
		Sample sample = db.createSample();
		long offset = System.currentTimeMillis() / 1000L;
		
		for (int time = 0; time < 540; time++)
		{
			sample.setTime(time + offset);
			sample.setValue("bytes", 1);
			sample.update();
		}

		FetchRequest request = 
			db.createFetchRequest("AVERAGE", offset+440, offset+520);
		FetchRequest request2 = 
			db.createFetchRequest("AVERAGE",offset, offset+540);
		FetchData fetchData = request.fetchData();
		FetchData fetchData2 = request2.fetchData();
		
		double value[] = fetchData.getValues("bytes");
		double value2[] = fetchData2.getValues("bytes");
		double valsum=0;
		double valsum2=0;
		
		for (int time = 0; time < value.length; time++) 
		{
			// System.out.println(time*20 + ": " + value[time]);
			if(!Double.isNaN(value[time])){valsum+=value[time];}
		}
		assertEquals("Wrong value sum retrieved",120.0, valsum, 0.5);
		
		for (int time = 0; time < value2.length; time++) 
		{
			// System.out.println(time*60 + ": " + value2[time]);
			if(!Double.isNaN(value2[time])){valsum2+=value2[time];}
		}
		assertEquals("Wrong value sum retrieved",540.0, valsum2, 0.5);
		
		db.close();	
	}
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(JRobinTests.class);
	}

}
