package org.aidworld.aidbase.offline.unit;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;
import org.aidworld.aidbase.core.Version;
import org.aidworld.aidbase.webui.util.WebUtils;
import org.apache.log4j.Logger;

/**
 * Tests that check if the layout of loband either on the main page
 * or on filtered pages is correct, should be put here.
 * 
 * @author Sebastien
 *
 */
public class LobandLayoutTests extends BasedTest{
    
    private Logger LOG = Logger.getLogger (LobandLayoutTests.class);

    public static Test suite() {
        return new TestSuite(LobandLayoutTests.class );
    }
    
	/**
	 * Aidbase used to add a header bar inside each body tag.
	 * Pages with multiple body tags get multiple headers.
	 */
	public void testShouldNotAddMultipleHeaderBarsWhenMultipleBodyTagsPresent() 
	throws Exception
	{
	    response = getTestPageThroughAidbase(); 
	    // System.out.println(output);
	    assertTrue("The content of a body tag was deleted",
	            response.getText().indexOf("badger") != -1);
	    assertFalse("Multiple headers added",
	    		response.getText().matches("(?s).*originally.*originally.*"));
	    assertFalse("Multiple footers added",
	    		response.getText().matches("(?s).*<hr.*<hr.*"));	    
	}

	public void testAidbaseHasCSS() throws Exception{
		response = getTestPageThroughAidbase();
        if (response.getText().indexOf("loband.css") == -1) {
            LOG.debug(response);
            fail ("No CSS inserted");
        }
	}

	/**
     * Verifies that the filtered page has the correct version, as specified in
     * org.aidworld.aidbase.core.Version. and link to the CSS file, and the CSS
     * downloads correctly and looks valid.
     */
    public void testHeaderOnFilteredHasCssClass() throws Exception {
        response = getThroughAidbase(
                "file:/www.reliefweb.int.html");
        if (response.getText().indexOf("<table class=\"aidbar\">") == -1) {
            fail("Aidbar isn't set as expected");
        }

        String CssUrl = WebUtils.getContextPath() + "/loband.css";
        assertTrue("missing reference to CSS file in filtered page", response
        		.getText().indexOf(CssUrl) != -1);

        if (response.getText().indexOf(
                "title=\"Version: " + Version.getVersion() + "\"") == -1) {
            System.out.println(response);
            fail("Aidbar doesn't contain the correct version");
        }

        response = getThroughAidbase(
                WebUtils.getApplicationUrl()
                + "/loband.css");
        assertTrue("No background colour found in css",
                response.getText().indexOf("background-color:") != -1);
        assertTrue("No definition found for aidbar in css",
                response.getText().indexOf("TABLE.aidbar") != -1);
    }

    public void testShouldInsertHeaderAndFooter() throws Exception {
        String input = "<html><head><title>hello</title></head>"
                + "<body>blah</body></html>";
        String expected = "<html><head>" + WebUtils.getPageHeadHtml()
                + "<title>" + Version.getShortNameHtml() + " hello</title></head>"
                + "<body>" + WebUtils.getPageHeader("http://foo")
                + "blah" + WebUtils.getPageFooter(false)
                + "</body></html>";

        assertEquals(getFilteredHtmlString(input), expected);
    }
}
