/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.offline.unit;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.aidworld.aidbase.document.Document;
import org.aidworld.aidbase.error.AidbaseException;
import org.aidworld.aidbase.error.GoDirectException;
import org.aidworld.aidbase.factory.StandardDocumentFactory;
import org.aidworld.aidbase.filter.SimplifyPipeline;
import org.aidworld.aidbase.request.Request;
import org.aidworld.aidbase.request.TestRequest;
import org.aidworld.aidbase.webui.servlet.HtmlChecker;
import org.aidworld.aidbase.webui.util.WebUtils;
import org.apache.log4j.Logger;

/**
 * Check that we handle binary documents correctly, by throwing
 * an exception.
 */

public class NonHtmlTest extends TestCase {

    /**
     * Constructor for NonHtmlTest.
     * @param arg0
     */
    public NonHtmlTest(String arg0) {
        super(arg0);
    }
    
    private Document FilterDocument(String urlString) throws Exception
	{
        URL url = new URL(urlString);
        StandardDocumentFactory sdf = new StandardDocumentFactory(
        		false, new HtmlChecker(Logger.getLogger(NonHtmlTest.class)));
        SimplifyPipeline pipeline = new SimplifyPipeline(sdf);
        Map map = new HashMap();
        map.put(WebUtils.getAidbaseUrlParam(), urlString);
        Request request = new TestRequest(url, "http://www.aidbase.org", map);
        return pipeline.service(request);
    }
    
    public void testPDFRejection() throws Exception {
        String urlString = 
            WebUtils.getApplicationUrl() + "/test/test.pdf";

        String exceptionThrown = "";
        boolean expectedExceptionThrown = false;
        try {
            FilterDocument(urlString);
        } catch (AidbaseException e) {
            if(e instanceof GoDirectException) {
                expectedExceptionThrown = true;
            }
            exceptionThrown = e.toString();
        }
            
        assertTrue("GoDirectException was not thrown for a PDF - exception" +
        		" thrown: " + exceptionThrown,
            expectedExceptionThrown);
    }


    public void testGIFRejection() throws Exception {
        String urlString = 
        	WebUtils.getApplicationUrl() + "/test/dot.gif";

        boolean expectedExceptionThrown = false;
        try {
        	FilterDocument(urlString);
        } catch (AidbaseException e) {
            if(e instanceof GoDirectException) {
                expectedExceptionThrown = true;
            }
        }
            
        assertTrue("GoDirectException was not thrown for a GIF",
            expectedExceptionThrown);

    }


    public void testJPGRejection() throws Exception {
        String urlString = 
            WebUtils.getApplicationUrl() + "/test/dot.jpg";

        boolean expectedExceptionThrown = false;
        try {
            FilterDocument(urlString);
        } catch (AidbaseException e) {
            if(e instanceof GoDirectException) {
                expectedExceptionThrown = true;
            }
        }
        
        assertTrue("GoDirectException was not thrown for a JPG",
            expectedExceptionThrown);
    }

    public void testDOCRejection() throws Exception {
        String urlString = 
            WebUtils.getApplicationUrl() + "/test/small_doc.doc";

        boolean expectedExceptionThrown = false;
        try {
            FilterDocument(urlString);
        } catch (AidbaseException e) {
            if(e instanceof GoDirectException) {
                expectedExceptionThrown = true;
            }
        }
        
        assertTrue("GoDirectException was not thrown for a DOC",
            expectedExceptionThrown);
    }

    
    /**
     * Standard JUnit method to build a suite of tests consisting of
     * all test methods within this class.
     */
    public static Test suite() {
        return new TestSuite(NonHtmlTest.class);
    }
}
