/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.offline.unit;

import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.DocumentSpec;
import org.aidworld.aidbase.core.Version;


/**
 * The DocumentSpecs class encapsulates some
 * specifications for 'standard' test pages, making it
 * easier to see which tests are available, and to add
 * new test pages to potentially a large number of
 * unit tests.
 */

public class OfflineDocumentSpecs extends TestCase {
	public static final String RESOURCE_PREFIX = "/data/"; 

	public static DocumentSpec getReliefwebHomeSpec() {
	    return new DocumentSpec(
	            "Reliefweb Home Page (static)",
	            // URL of page to test
	            "file:/www.reliefweb.int.html",
				// Array of strings expected to be present in the filtered output
		        new String[] {
						"IRIN",
			        	"OCHA",
			        	"Complex Emergencies",
			            "Latest Updates",
		                "Map Centre",
		                "ReliefWeb All rights reserved" },
				// Array of strings expected to be absent
		        new String[] {
						// should be filtered:
			        	"<!-- Lotus-Domino",
						// wrong URL translation:
			        	"http://www.reliefweb.int/w/rwb.nsf" }
				);
	}


	public void testReliefwebHome() throws Exception {
	    TestResult result = getReliefwebHomeSpec().run();
	    assertTrue(result.wasSuccessful());
	}
	

	public final DocumentSpec [] allSpecs = new DocumentSpec [] {
	        getReliefwebHomeSpec(),
	        

		    new DocumentSpec(
		            "BBC Home Page (static)",
					// URL of page to test
					"file:/bbcnexfp.html",
					// Array of strings expected to be present in the filtered output
					new String[] {
						"The Windsor and Spencer families",
						"BBC",
		                "Where&nbsp;I&nbsp;Live",
		                "Privacy &amp; Cookies Policy",
		                "Poorly? Prove it",
		                "Radio"
					},
					// Array of strings expected to be absent
					new String[] {}                 
			),


		    new DocumentSpec(
		            "CNN Home Page (static)",
					// URL of page to test
					"file:/CNN.com.html",
					// Array of strings expected to be present in the filtered output
					new String[] {
						"cnn.com",
						"Democratic presidential nominee",
						"CNN",
		                "International Edition",
		                "CNN.com does not endorse external sites",
		                "Cable News Network"
					},
					// Array of strings expected to be absent
					new String[] {}                 
			),

		    new DocumentSpec(
		            "Yahoo.co.uk Home Page (static)",
					// URL of page to test
					"file:/www.yahoo.co.uk.html",
					// Array of strings expected to be present in the filtered output
					new String[] {
						"<title>" + Version.getShortNameHtml() 
						+ " Yahoo! UK & Ireland</title>",
						"Privacy Policy",
						Version.getName()
					},
					// Array of strings expected to be absent
					new String[] {
						"An invalid XML character"
//						new String(new byte[] {0})
					}					
			),

		    new DocumentSpec(
		            "Yahoo.com Home Page (static)",
					// URL of page to test
					"file:/www.yahoo.com.html",
					// Array of strings expected to be present in the filtered output
					new String[] {
						"<title>" + Version.getShortNameHtml() 
						+ " Yahoo!</title>",
						"Privacy Policy",
						Version.getName()
					},
					// Array of strings expected to be absent
					new String[] {
						"The content of elements must consist of well-formed "+
						"character data or markup",
						"<![if"
					}					
			),
			
			new DocumentSpec(
					"ITDG Home Page (static)",
					"file:/www.itdg.org.html",
					new String[] {
							"ITDG helps people to use technology " +
							"in the fight against poverty.",
							"Practical information on water, " +
							"sanitation and construction",
							"Make Poverty History",
							"15 February 2005 &nbsp;11:05",
							"Food Chain"
					},
					new String[]{}
			),
			
			new DocumentSpec(
					"CABI Home Page (static)",
					"file:/www.cabi.org.html",
					new String[]{
							"Welcome to CAB",
							"Privacy Policy"
					},
					new String[]{}
			),

			new DocumentSpec(
					"ALIN Home Page (static)",
					"file:/www.alin.or.ke.html",
					new String[]{
							"Open Knowledge Network launched in Geneva",
							"on dryland agricultural practices",
							"activities"
					},
					new String[]{}
			),
			
			new DocumentSpec(
					"DGroups OKN Group (static)",
					"file:/www.dgroups.org_groups_okn_index.html",
					new String[]{
							"<title>&gt;0- Open Knowledge Workspace</title>",
							"OKN Progress Reports",
							"Read about the terms of Copyright, Privacy, User agreements"
					},
					new String[]{}
			),

			new DocumentSpec(
					"Oneworld Home Page (static)",
					"file:/www.oneworld.net.html",
					new String[]{
							"We bring together the latest news and views from over",
							"Penguin, Kids Channel",
					},
					new String[]{}
			),

			new DocumentSpec(
					"CCK Home Page (static)",
					"file:/www.cck.go.ke.html",
					new String[]{
							"Spectrum Management",
							"is the regulatory body"
					},
					new String[]{}
			),
			
			new DocumentSpec(
					"KACE Kenya Home Page (static)",
					"file:/www.kacekenya.com.html",
					new String[]{
							"Commodity Prices",
							"Website Policy",
							"Copyright &copy; 2005, KACE",
					},
					new String[]{}
			),
			
			new DocumentSpec(
					"KENET Main Page (static)",
					"file:/www.kenet.org.main.html",
					new String[]{
							"The KENET Newsletter is a definitive source",
							"<form target=\"popuppoll\" method=\"post\"",
							"<P>Building a Strong Education Network",
							"KENET | All Rights Reserved</span>",
					},
					new String[]{}
			),
	};

	public static TestSuite suite() throws Exception {
	    TestSuite suite = new TestSuite();
	    OfflineDocumentSpecs specs = new OfflineDocumentSpecs();
	    DocumentSpec[] all = specs.allSpecs;
	    
	    for (int i = 0; i < all.length; i++) {
	        suite.addTest(all[i].getSuite());
	    }
	    return suite;
	}
}
