/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.offline.unit;

import java.net.URL;
import java.util.Hashtable;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;
import org.aidworld.aidbase.core.Version;
import org.aidworld.aidbase.document.Document;
import org.aidworld.aidbase.factory.TestDocumentFactory;
import org.aidworld.aidbase.filter.RuleFilter;
import org.aidworld.aidbase.request.TestRequest;
import org.aidworld.aidbase.webui.util.WebUtils;

import com.opensymphony.module.sitemesh.html.BasicRule;
import com.opensymphony.module.sitemesh.html.HTMLProcessor;
import com.opensymphony.module.sitemesh.html.State;
import com.opensymphony.module.sitemesh.html.Tag;
import com.opensymphony.module.sitemesh.html.util.CharArray;

/**
 * A collection of tests for the RuleFilter rules running on top of sitemesh's 
 * HTML parser
 */
public class ParserFilterTest extends BasedTest {
	
	public boolean rule_processed = false;
    private static String aidbaseUrl;
    private static String head = null;
    
    public void setUp() throws Exception {
    	super.setUp();
        aidbaseUrl = WebUtils.getApplicationUrl();
        head = "<head>" + WebUtils.getPageHeadHtml() 
        + "<title>" + Version.getShortNameHtml() + " </title></head>";
    }

	public void testAidbaseShouldPlaceHeadTagIfNotFound() throws Exception{
		response = getThroughAidbase(
				WebUtils.getApplicationUrl() + "/test/testNoHeadTag.html");
		assertTrue("No head tag found", response.getText().indexOf("<head>") != -1);
	}

	public void testImagesShouldBeReplacedWithLinks() 
	throws Exception {
		 response = getTestPageThroughAidbase(); 
		 assertTrue("Image link not found",
		 		response.getText().indexOf("[i-badger]")>-1);
	}

	public void testShouldWorkWithWrongSpelledCharacterSet() 
	throws Exception{
		response = getThroughAidbase(
				WebUtils.getApplicationUrl()+
				"/test/charset.jsp");
		
		assertTrue("Problems with charset in meta tag",
				response.getText().indexOf("This is a test page with meta.")>-1);
		
		response = getThroughAidbase(
				WebUtils.getApplicationUrl()+
				"/test/charset.jsp?header=foo");
		
		assertTrue("Problems with charset in header",
				response.getText().indexOf("This is a test page with header.")>-1);
	}

	/**
     * XML processing instructions, and things that look like them,
     * should be removed because they cause Xerces to die.
     * For example, <?mysql?> in a page would cause an exception.
     * @see http://www.statusq.org
     * @throws Exception
     */
    public void testXmlProcessingInstructionsShouldNotCauseExceptions() 
    throws Exception
    {
        response = getTestPageThroughAidbase();
        assertTrue("XML Processing Instructions were not removed",
                response.getText().indexOf("<?") == -1);
    }

    /**
     * When a <body> tag is missing, this tests if it, together
     * with an aidbase bare, is added.
     * 
     * @throws Exception
     */
    public void testABodyTagIsInserted() throws Exception{
        String input = "<html><head><title>No need for a title</title></head>\n"
            + "<p>A body should be inserted before this line</p></body></html>";
        String expected = "<html><head>" + WebUtils.getPageHeadHtml()
            + "<title>" + Version.getShortNameHtml()
            + " No need for a title</title></head>\n"
            + "<body>" + WebUtils.getPageHeader("http://foo")
            + "<p>A body should be inserted before this line</p>"
            + WebUtils.getPageFooter(false) + "</body></html>";
        internalTestHelper(input, expected);
    }
    
    public void testBodyInsertedOnBodylessPage() throws Exception
    {
    	response = getThroughAidbase(
    			WebUtils.getApplicationUrl()+"/test/nobody.html"); 
    	assertTrue("No body inserted", response.getText().indexOf("<body")!=-1);
    	
    }

	public void testRepeatedQuoteRemoval() throws Exception {
		
		response = getThroughAidbase(
				WebUtils.getApplicationUrl()
				+"/test/QuoteTest.html");
		
		assertTrue("Repeated quotes not removed properly"
				, response.getText().indexOf("</textarea>")!=-1);
	}

	public void testBaseHrefRewrite() throws Exception {
		
		response = getThroughAidbase(
				WebUtils.getApplicationUrl()
				+"/test/BaseHref.html");
		
		assertTrue("Base href rewriting not handled properly"
				, response.getText().indexOf("/com/cnn/www/%20/image.gif")!=-1);
	}

    public void testNobrRemoval() throws Exception {
    	response = getTestPageThroughAidbase();
         
        assertTrue("Nobr tags not cleared as expected" ,  
        		response.getText().indexOf("<nobr>Text inside nobr tag</nobr>") == -1);               
    }
    
	public void testStylesheetLinkRemoval() throws Exception {
		
    	response = getTestPageThroughAidbase();
		assertTrue("StylesheetLinkTags are not removed as expected.", 
				response.getText().indexOf("<link rel=\"stylesheet\" type=\"" +
						"text/css\" href=\"styles.css\">") == -1);
	}



	public void testCentreTagRemoval() throws Exception{
    	response = getTestPageThroughAidbase();
		
		assertFalse("CentreTags are not removed as expected.", 
            response.getText().indexOf("Test Page Text.</center>") != -1);
	}


    public void testFontRemoval() throws Exception {
 
    	response = getTestPageThroughAidbase();
        
        assertFalse("FONT tag not removed as expected.", 
        		response.getText().indexOf("<font face=\"Tahoma\" size=\"2\">") != -1);   
    }

	
    public void testLayerRemoval() throws Exception {

    	response = getTestPageThroughAidbase();
       
        assertTrue("Layer tags not removed as expected.", 
        		response.getText().indexOf("Text in layer</layer>")== -1);
        
    }

	public void testEmbedRemoval() throws Exception {
		
    	response = getTestPageThroughAidbase();

		assertTrue("Object tag is not removed as expected.",
				response.getText().indexOf("Loband: Object Removed") != -1);
		assertTrue("Embed tag is not removed as expected.",
				response.getText().indexOf("Loband: Embedded Object Removed") != -1);
	}

	/**
	 * Example: @link http://www.bbc.co.uk navigation links run onto one line
	 * @link http://www.w3.org/TR/html4/struct/global.html#h-7.5.4
	 * The W3C says: "Visual user agents generally place a line break 
	 * before and after DIV elements".
	 * Therefore they cannot be removed without affecting presentation.
	 */
	public void testDivTagsShouldNotBeRemoved() 
	throws Exception {
	    response = getTestPageThroughAidbase(); 
	    
	    assertTrue("Div tag was deleted",
	    		response.getText().indexOf("<div>Div Text</div>") !=-1);
	}

	public void testFormModification() throws Exception {

    	response = getTestPageThroughAidbase();
        /*
        System.out.println("------------");
        System.out.println(response.getText());
        System.out.println("------------");
        */
    	assertTrue("Didn't change image submit to submit button.",
    			response.getText().indexOf("<input type=\"submit\"")!= -1);    
    	assertTrue("Added empty name attribute to submit button.",
    			response.getText().indexOf("<input type=\"submit\" name=\"\"")== -1);
    	assertTrue("Form action doesn't contain the \"Aidbase\" url.",
        		response.getText().indexOf("<form action=\"/loband/page\"") != -1);
        assertTrue("Form's action isn't set as expected", 
        		response.getText().indexOf("action=\""
        				+ WebUtils.getContextPath() + WebUtils.getRealFilterServletPath()
						+ "/uk/co/bbc/www/%20/cgi-bin/" +
        				"search/results.pl") != -1);
    }
    
  
    public void testRemoveUnnecessaryAttributes() throws Exception {
    	response = getTestPageThroughAidbase();
    	assertTrue("Unnecessary attribute not removed.",
    			response.getText().indexOf("background=\"blah\"") == -1);
    }
    
    public void testKeepNecessaryAttributes() throws Exception {
    	response = getTestPageThroughAidbase();
    	assertTrue("Rowspan attribute removed.",
    			response.getText().indexOf("rowspan") != -1);
    }

    public void testTitleTag() throws Exception{
    	response = getTestPageThroughAidbase();
    	assertTrue("No default title inserted", 
    			response.getText().indexOf("&gt;0-") != -1);
    	
    	response = getThroughAidbase(
    			WebUtils.getApplicationUrl() 
    			+ "/test/testNoHeadTag.html");
    	assertTrue("No title in default head generated", 
    			response.getText().indexOf("&gt;0-") != -1);
    	
    	response = getThroughAidbase(
    			WebUtils.getApplicationUrl() 
    			+ "/test/postDataTest.jsp");
    	assertTrue("Title not mangled as expected", 
    			response.getText().indexOf("&gt;0- PostDataTest") != -1);
    	
    }

    public void testMetaTagRemoval() throws Exception {
    	response = getTestPageThroughAidbase();
    	assertTrue("Meta tag not removed", response.getText().indexOf("<META")== -1);
    }

    public void testNoEmbedTagShouldWork() throws Exception {
        response = getThroughAidbase(
                aidbaseUrl + "/test/noembed.html");
        assertTrue("Failed to load complete page", response.getText().indexOf(
                "</html>") > -1);
    }

    public void testDoubleTitleTagsShouldNotBePresent() throws Exception {
        response = getThroughAidbase(
                "file:/www.reliefweb.int.html");
        assertTrue(response.getText().indexOf("<title.*<title") == -1);
    }
    
    public void testHTMLCommentsInScriptTagsAreHandledInASensibleWay()
    throws Exception {
        String expected = "below script";

        response = getThroughAidbase(
                aidbaseUrl + "/test/textarea.html");

        assertTrue("HTML comments occuring within script tags " +
                "are not being handled correctly",
                response.getText().indexOf(expected) != -1);
    }

    public void testImageDeletionOnSizeAttributes() throws Exception{
        response = getThroughAidbase(
                aidbaseUrl +"/test/imageDeletionOnSizeTest.html");
        assertTrue("Image with width of 5px and height of 5px not removed",
                response.getText().indexOf("5w5himage.jpg") == -1);
        assertTrue("Image with width of 5px and height of 10px not removed",
                response.getText().indexOf("5w10himage.jpg") == -1);
        assertTrue("Image with width of 5px and without height attribute" +
                " not removed",
                response.getText().indexOf("5wimage.jpg") == -1);
        assertTrue("Image without width or height removed",
                response.getText().indexOf("none.jpg") != -1);
        assertTrue("Image with width of 5% and height of 5px not removed",
                response.getText().indexOf("5pw5himage.jpg") == -1);
        assertTrue("Image with width of 5% and height of 10px removed",
                response.getText().indexOf("5pw10himage.jpg") != -1);
        assertTrue("Image with width of 10px and height of 10px removed",
                response.getText().indexOf("10w10himage.jpg") != -1);
    }

    public void testImgTagsWithNoSRCDontBreakPage() throws Exception{
        response = getThroughAidbase(
                WebUtils.getApplicationUrl() 
                + "/test/imgtagnosrc.jsp");
        assertTrue("Image Tag with no SRC causing error",
                response.getText().indexOf("img")>-1);
    }
    
    public void testCommentRemoval() throws Exception
    {
        response = getThroughAidbase(
                aidbaseUrl + "/test/test.html");
      
        assertTrue("Comment not removed", 
                response.getText().indexOf("<!-- Comment") == -1);
        
        
        assertFalse("Text outside comments also removed", 
                response.getText().indexOf("Text outside comments") == -1);
        
        response = getThroughAidbase(
                aidbaseUrl +"/test/commentRemoval.html");
        
        assertTrue("Comment <!----> not treated as expected"
                , response.getText().indexOf("Blah")!=-1);
    }

//    public void testShouldCombineMultipleBodies() throws Exception {
//        String input = "<html><body><p>hi</p></body><body><p>bye</p></body></html>";
//        String expected = "<html>"+head+"<body>" 
//        		+ WebUtils.getPageHeader("http://foo") 
//				+ "<p>hi</p><p>bye</p>" 
//				+ WebUtils.getPageFooter("http://foo");
//
//        internalTestHelper(input, expected);
//    }
    
    public void testShouldFixMissingCloseHtmlAndBody() throws Exception {
        String input = "<html><body><p>hi</p></body><body><p>bye</p>";
        String expected = "<html>" + head + "<body>" 
            + WebUtils.getPageHeader("http://foo") 
			+ "<p>hi</p></body><body><p>bye</p>" 
			+ WebUtils.getPageFooter(false)
			+ "</body></html>";
        
        internalTestHelper(input, expected);
    }
    
    public void testShouldRemoveUnwantedTags() throws Exception {
        String input = "<html><body>Tags should <layer>go</layer>, but <font><center>text should</center></font> stay ...</body></html>";
        String expected = "<html>" + head + "<body>" 
            + WebUtils.getPageHeader("http://foo") 
			+ "Tags should go, but text should stay ..." 
			+ WebUtils.getPageFooter(false)
        	+ "</body></html>";
        internalTestHelper(input, expected);
    }
  /*  
    public void testShouldRemoveUnwantedTagsAndContents() throws Exception {
        String input = "<html><body><style>all </style><meta>this</meta><img>should</img><link>go</link><iframe>completely</iframe> but this text should stay</body></html>";
        String expected = "<html>"+head+"<body>" 
            + WebUtils.getPageHeader("http://foo") 
			+ " but this text should stay" + WebUtils.PAGE_FOOTER;
        
        internalTestHelper(input, expected);
    }
    */
    public void testShouldSubstituteAppletWithText() throws Exception {
        String input = "<html><body>The <applet alt=\"appletName\">applet sitting here</applet> should be removed</body></html>";
        String expected = "<html>" + head + "<body>" 
            + WebUtils.getPageHeader("http://foo") 
			+ "The [<i>Loband: Applet Removed - appletName</i>] " 
			+ "should be removed" 
			+ WebUtils.getPageFooter(false)
			+ "</body></html>";
        
        internalTestHelper(input, expected);
    }
    
    public void testShouldSubstituteObjectWithText() throws Exception {
        String input = "<html><body>The <object alt=\"alt text\" type=\"type text\">object sitting here</object> should be removed</body></html>";
        String expected = "<html>" + head + "<body>" 
            + WebUtils.getPageHeader("http://foo") 
			+ "The [<i>Loband: Object Removed - alt text type text</i>] " 
			+ "should be removed" 
			+ WebUtils.getPageFooter(false)
			+ "</body></html>";
        
        internalTestHelper(input, expected);
    }
    
    public void testShouldSubstituteEmbedWithText() throws Exception {
        String input = "<html><body>The <embed src=\"src text\">embedded object sitting here</embed> should be removed</body></html>";
        String expected = "<html>"+head+"<body>"  
        	+ WebUtils.getPageHeader("http://foo") 
			+ "The [<i>Loband: Embedded Object Removed - src text</i>] "
			+ "should be removed" 
			+ WebUtils.getPageFooter(false)
			+ "</body></html>";
        
        internalTestHelper(input, expected);
    }
    
    public void testShouldRemoveTableAttributes() throws Exception {
        String input = "<html><body><table blah=\"bob\" foo=\"bar\"></table></body></html>";
        String expected = "<html>"+head+"<body>" 
            + WebUtils.getPageHeader("http://foo") 
			+ "<table></table>" 
			+ WebUtils.getPageFooter(false)
			+ "</body></html>";
        
        internalTestHelper(input, expected);
    }
    
    public void testShouldRemoveBRAttributes() throws Exception {
        String input = "<html><body><br clear=\"all\" /><br clear=\"all\"></body></html>";
        String expected = "<html>"+head+"<body>" 
            + WebUtils.getPageHeader("http://foo") 
			+ "<br /><br>" 
			+ WebUtils.getPageFooter(false)
			+ "</body></html>";
        
        internalTestHelper(input, expected);
    }
    
    public void testShouldChangeImageSubmitToButton() throws Exception {
        String input = "<html><body><input type=\"image\" name=\"foo\" value=\"bar\" src=\"SRC\" height=\"123\" width=\"234\"></body></html>";
        String expected = "<html>"+head+"<body>" 
            + WebUtils.getPageHeader("http://foo") 
			+ "<input type=\"submit\" name=\"foo\" value=\"bar\"/>" 
			+ WebUtils.getPageFooter(false)
			+ "</body></html>";
        
        internalTestHelper(input, expected);
    }
            
    public void testFormActionRewritten() throws Exception {
    	String input = "<html><body><form action=\"http://foo.com/mypage.htm\"" +
    			" name=\"form1\">blah<input type=\"submit\" /></form>" +
    			"<form action=\"http://foo.com/mypage.htm\" method=\"post\"" +
    			" target=\"blah\" enctype=\"dee\" foo=\"bar\"" +
    			" /></body></html>";
    	
    	String expected = "<html>"+head+"<body>" + 
			WebUtils.getPageHeader("http://foo") +
			"<form name=\"form1\" action=\"" + WebUtils.getContextPath()
			+ WebUtils.getRealFilterServletPath() +
			"/com/foo/%20/mypage.htm\">" +
			"blah<input type=\"submit\"/></form><form method=\"post\" target=\"blah\" enctype=\"dee\"" +
			" action=\"" + WebUtils.getContextPath() + WebUtils.getRealFilterServletPath() +
			"/com/foo/%20/mypage.htm\"></form>"
			+ WebUtils.getPageFooter()
			+ "</body></html>";
        
        internalTestHelper(input, expected);
    }  
    /*
    public void testFormAlwaysHasSubmitButton() throws Exception {
    	String input = "<html><body><form action=\"http://foo.com/mypage.htm\"></form></body></html>";
    	
    	String expected = "<html>"+head+"<body>" + WebUtils.getPageHeader("http://foo") + 
			"<form action=\"/aidbase/page\"><input type=\"hidden\" name=\"_ab_url\" value=\"http://foo.com/mypage.htm\" /><input type=\"submit\" /></form>" +
			WebUtils.PAGE_FOOTER;
        
        internalTestHelper(input, expected);
    }  
    */
        
    public void testTextAreaNotEmptyTag() throws Exception {
    	String input = "<html><body><textarea rows=\"1\" cols=\"2\" name=\"foo\" /></body></html>";
    	
    	String expected = "<html>"+head+"<body>" 
			+ WebUtils.getPageHeader("http://foo") 
			+ "<textarea rows=\"1\" cols=\"2\" name=\"foo\" />"
			+ WebUtils.getPageFooter(false)
			+ "</body></html>";
        
        internalTestHelper(input, expected);
    }  
            
    
    public void testImageMapParsedCorrectly() throws Exception {
    	String input = "<html><body>"+
    		"<img src=\"http://foo.com/testimage.gif\" usemap=\"#tlMap\">"+ 
    		  "<map name=\"tlMap\">"+
    		  "<area href=\"http://foo.com/testcentre/\">"+
    		  "<area href=\"http://foo.com/testmeeting/\">"+
    		  "<area href=\"http://foo.com/testlibrary/\">"+
    		  "<area href=\"http://foo.com/testtraining/\">"+
    		  "</map></body></html>";
  		String expected = "<html>"+head+"<body>"+
  			WebUtils.getPageHeader("http://foo")+
  	  	  "<a href=\"" + WebUtils.getContextPath() + WebUtils.getRealFilterServletPath() + "/com/foo"+
  	  	  "/%20/testcentre/\">[testcentre]</a>"+
  	  	  " <a href=\"" + WebUtils.getContextPath() + WebUtils.getRealFilterServletPath() + "/com/foo"+
  	  	  "/%20/testmeeting/\">[testmeeting]</a>"+
  	  	  " <a href=\"" + WebUtils.getContextPath() + WebUtils.getRealFilterServletPath() + "/com/foo"+
  	  	  "/%20/testlibrary/\">[testlibrary]</a>"+
  	  	  " <a href=\"" + WebUtils.getContextPath() + WebUtils.getRealFilterServletPath() + "/com/foo"+
  	  	  "/%20/testtraining/\">[testtraining]</a>"+
  	  	  " <a title=\"click here to view image\" href=\""
  	  	  + WebUtils.getContextPath() + WebUtils.getRealFilterServletPath() +
  	  	  "/com/foo/%20/testimage.gif?" + WebUtils.getAidbasePipelineParam() + "=" +
  	  	  WebUtils.getAidbaseBinaryType() +
  	  	  "\" target=\"_new\">[i-testimage]</a>"+
  	  	  "<map name=\"tlMap\">"+
  	  	  "<area href=\"http://foo.com/testcentre/\">"+
  	  	  "<area href=\"http://foo.com/testmeeting/\">"+
  	  	  "<area href=\"http://foo.com/testlibrary/\">"+
  	  	  "<area href=\"http://foo.com/testtraining/\">"+
  	  	  "</map>" + WebUtils.getPageFooter(false)
  	  	  + "</body></html>";
  		
  		// System.out.println(input);
  
  		internalTestHelper(input, expected);
    }
    
    public void testCommentsAreRemovedButJavascriptIsNot() throws Exception
    {
        String input = "<html><script type=\"text/javascript\">\n" +
            "<!-- function javascriptIsNotRemoved(){}\n" +
            "//--></script>\n" +
            "<body><!-- This should be removed -->But not this</body></html>";
        TestDocumentFactory fact = new TestDocumentFactory(input,
                new Hashtable());
        TestRequest req = new TestRequest(new URL("http://foo"), "/loband",
                new Hashtable());
        RuleFilter filter = new RuleFilter();
        filter.setNextStage(fact);
        Document out = filter.service(req);
        
        assertTrue("Comments were not removed",
                out.toString().indexOf("This should be removed") == -1);
        assertTrue("Javascript was removed",
                out.toString().indexOf("function javascriptIsNotRemoved()") != -1);
    }        

    
    public void internalTestHelper(String input, String expected) throws Exception 
    {
        internalTestHelper(input, expected, new URL("http://foo"));
    }

  
    public void internalTestHelper(String input, String expected, URL pageUrl) 
    throws Exception 
    {
        TestDocumentFactory fact = new TestDocumentFactory(input,
                new Hashtable());
        TestRequest req = new TestRequest(pageUrl, "/loband",
                new Hashtable());
        RuleFilter filter = new RuleFilter();
        filter.setNextStage(fact);
        Document out = filter.service(req);

//        System.out.println("\nOUTPUT:\n" +  out.toString());
    	assertEquals(expected, out.toString());
    }
    
  /**
   * Tests for a bug in the SiteMesh parser, which fails to
   * correctly handle <!----> comments in HTML. We have a workaround
   * for this in the code base. When this test fails, it means that
   * SiteMesh has been fixed, so the sense of the test should be
   * reversed (pass when it used to fail) and the workaround removed
   * from the code.
   * 
   * @throws Exception if anything goes wrong during the test.
   */

    public void testCommentsRemoval() throws Exception 
	{
    	String input1 = 
    	"<html><script scr=\"test.js\"><!---->" +
			"<body><table width=\"100%\" bgcolor=\"red\">" +
			"</body>" +
         "</html>" +
         "<!-- bla -->";
    	
    	CharArray output = new CharArray(1024);
    	
    	HTMLProcessor htmlProcessor = 
        	new HTMLProcessor(input1.toCharArray(), output);
    	State state = htmlProcessor.defaultState();
    	state.addRule(new TestRule());
    	
    	htmlProcessor.process();
    	
    	assertFalse("Comment tag properly processed - maybe the " +
    			"SiteMesh bug has been fixed?", rule_processed);	
    }
    
    /**
     * Tests for a bug in SiteMesh, which causes parsing of pages
     * to fail when they contains double double-quotes ("") in attributes.
     * 
     * The test is currently configured to pass when SiteMesh DOES have
     * the bug. When it starts to fail, that probably means that
     * SiteMesh has been updated and the bug has been fixed, so the
     * sense of the test should be reversed.
     *  
     * @throws Exception
     */
    
    public void testUnclosedQuoteBug() throws Exception {
    	String input = 
        	"<map name=\"tlMap\">\n" + 
    		"<area href=\"http://www.sheltercentre.org/sheltercentre/\">\n" +
    		"<area href=\"http://www.sheltercentre.org/sheltermeeting/\">\n" +
    		"<area href=\"http://www.sheltercentre.org/shelterlibrary/\">\n" +
    		"<area href=\"http://www.sheltercentre.org/sheltertraining/\">\n" +
    		"</map>";
        	
    	AreaTestRule atr = new AreaTestRule();
    	CharArray output = new CharArray(1024);
    	HTMLProcessor htmlProcessor = 
        	new HTMLProcessor(input.toCharArray(), output);
    	State state = htmlProcessor.defaultState();
    	state.addRule(atr);
    	htmlProcessor.process();
    	
    	assertEquals("Test is broken", 4, atr.areaCount);

    	input = 
	    	"<map name=\"tlMap\">\n" + 
			"<area href=\"http://www.sheltercentre.org/sheltercentre/\"\">\n" +
			"<area href=\"http://www.sheltercentre.org/sheltermeeting/\">\n" +
			"<area href=\"http://www.sheltercentre.org/shelterlibrary/\">\n" +
			"<area href=\"http://www.sheltercentre.org/sheltertraining/\">\n" +
			"</map>";
    	
    	atr = new AreaTestRule();
    	output = new CharArray(1024);
    	htmlProcessor = 
        	new HTMLProcessor(input.toCharArray(), output);
    	state = htmlProcessor.defaultState();
    	state.addRule(atr);
    	htmlProcessor.process();
    	
    	// assertEquals("Unclosed quote mark not handled properly", 
    	// 4, atr.areaCount);
    	assertFalse("Unclosed quote mark handled properly: " +
    			"SiteMesh bug fixed?", 4 == atr.areaCount);
    }
    
    // test does not terminate due to suspected bug in httpunit/jtidy,
    // if it ever gets fixed then restore test, it should pass as viewing the
    // pages works
        
    
// public void testFramesWithinFramesDoNotGetMultipleAidBars()
// throws Exception {
//
// response = getThroughAidbase(getAidbaseUrl() +
// "/test/frames/busyinternet/fm_services.htm?_ab_noheader=1");
//
// assertTrue("Aidbar being shown when _ab_noheader=1",
// response.getText().indexOf("name=\"_aidbar\"")==-1);
// assertTrue("noframes and body tags mis-nested",
// response.getText().matches("(?s)</body>.*</noframes>"));
//
// }
    
	public static Test suite() {
		return new TestSuite(ParserFilterTest.class );
	}
	
	public class TestRule extends BasicRule {
	    public TestRule() {
	        super(new String[]{"table"});
	    }
	    
	    public void process(Tag tag) {
	    	if( tag.getType() == Tag.OPEN)
	    		rule_processed = true;
	    }

	}
	
	public class AreaTestRule extends BasicRule {
		public int areaCount = 0;
		public AreaTestRule() {
			super(new String[]{"area"});
		}
	    public void process(Tag tag) {
	    	if( tag.getType() == Tag.OPEN)
	    		areaCount++;
	    }
	}

	class MockTag implements Tag {
		public String getContents() {return "CONTENTS";}
		public void writeTo(CharArray arg0) {}
		public String getName() {return "NAME";}
		public int getType() {return Tag.OPEN;}
		public int getAttributeCount() {return 0;}
		public String getAttributeName(int arg0) {return "ATT_NAME";}
		public String getAttributeValue(int arg0) {return "ATT_VALUE";}
		public String getAttributeValue(String arg0) {return "ATT_VALUE";}
		public boolean hasAttribute(String arg0) {return true;}
	}
}