/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.offline.unit;

import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;
import org.aidworld.aidbase.webui.util.WebUtils;
import org.xml.sax.SAXException;

import com.meterware.httpunit.WebForm;
import com.meterware.httpunit.WebResponse;


public class RedirectTests extends BasedTest {
        
	/**
	 * Standard JUnit constructor.
	 */
	public RedirectTests(String name) {
		super(name);
	}
	
	/**
	 * Standard JUnit method to build a suite of tests consisting of
	 * all test methods within this class.
	 */
    public static Test suite() {
        return new TestSuite(RedirectTests.class);
    }
		
    private WebResponse getFilteredPage(String url) 
    throws SAXException, IOException {
    	WebForm form = response.getForms()[0];
    	form.setParameter(WebUtils.getAidbaseUrlParam(), url);
        form.submit();
        return conversation.getCurrentPage();
    }
    
    public void testRedirectWithDifferentProtocolAndPort() throws Exception {
        WebResponse response = getThroughAidbase(getAidbaseUrl()
                + "/test/redirect_host.jsp");
        assertTrue("Redirect failed",
                response.getText().indexOf("VeriSign") > -1);
    }
    
	public void testHTTPHeaderRedirect() throws Exception {
		response = getFilteredPage(WebUtils.getApplicationUrl()+"/test/redirect_HTTPHeader.jsp");
		assertTrue("Redirection didn't work.",response.getText().indexOf("Badger")!=-1);
	}
	
    public void testPostToRedirectShouldChangeMethodToGet() throws Exception {
        response = getThroughAidbase(getAidbaseUrl()
                + "/test/redirectMethodTest.jsp");
        response.getFormWithName("testform").submit();
        response = conversation.getCurrentPage();
        String resp = response.getText().replaceAll("(?s).*</table>\n", "")
                .replaceAll("(?s)</body></html>.*", "");
        assertTrue("Text content is wrong", resp.indexOf("Test passed!") != -1);
    }

    public void testPostToRefreshShouldChangeMethodToGet() throws Exception {
        response = getThroughAidbase(getAidbaseUrl()
                + "/test/refreshMethodTest.jsp");
        response.getFormWithName("testform").submit();
        response = conversation.getCurrentPage();
        String resp = response.getText().replaceAll("(?s).*</table>\n", "")
                .replaceAll("(?s)</body></html>.*", "");
        assertFalse("Meta refresh doesn't work", resp
                .indexOf("Meta refresh wasn't executed") != -1);
        assertTrue("Text content is wrong: " + resp, resp
                .indexOf("Test passed!") != -1);
    }

    public void testChangingMethodsWhenRedirected() throws Exception{
		response = getFilteredPage(WebUtils.getApplicationUrl() + "/test/post_to_get.jsp");
		WebForm form = response.getFormWithName("test");
		form.submit();
		response = conversation.getCurrentPage();
		assertTrue("Method not changed",response.getText().indexOf("GET")>-1);	
	}
		
	public void testMetaRedirect() throws Exception {
		response = getFilteredPage(WebUtils.getApplicationUrl()+"/test/redirect_meta.html");
		assertTrue("Redirection didn't work.",response.getText().indexOf("Join the badger")!=-1);
	}
		
	/* 
	 * Broken by lack of support for JavaScript in HttpUnit? 
	 * Removed by Chris 050426
	 */
	/*
	public void testJavaScriptRedirect() throws Exception {
		HttpUnitOptions.setScriptingEnabled(true);
		response = getFilteredPage(WebUtils.getApplicationUrl()
				+ "/test/redirect_javascript.html");
		if (response.getText().indexOf("Join the badger") == -1) {
			fail("Redirection didn't work: "+response.getText());
		}
	}
	*/
	
	
}
