/*
 * Created on 04-Feb-2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.aidworld.aidbase.offline.unit;

import java.net.URL;
import java.util.Hashtable;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.aidworld.aidbase.document.Document;
import org.aidworld.aidbase.document.HtmlDocument;
import org.aidworld.aidbase.error.AidbaseException;
import org.aidworld.aidbase.error.DocumentException;
import org.aidworld.aidbase.error.InternalFaultException;
import org.aidworld.aidbase.factory.StandardDocumentFactory;
import org.aidworld.aidbase.factory.TestDocumentFactory;
import org.aidworld.aidbase.filter.Stage;
import org.aidworld.aidbase.handler.RequestHandler;
import org.aidworld.aidbase.handler.RetryHandler;
import org.aidworld.aidbase.handler.StandardRequestHandler;
import org.aidworld.aidbase.request.Request;
import org.aidworld.aidbase.request.TestRequest;
import org.aidworld.aidbase.util.Utils;
import org.aidworld.aidbase.webui.util.WebUtils;

/**
 * @author chris
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class RetryHandlerTests extends TestCase {
	private RequestHandler doRetryRequest(Request request, Stage factory) 
	throws Throwable {
		Object monitor = new Object();
		RequestHandler retrier = new RetryHandler("test", request, 
				false, factory, monitor);
		retrier.start();
		synchronized(monitor) {
			monitor.wait();
		}
		if (retrier.getException() != null) {
			throw retrier.getException();
		}
		return retrier;
	}
	
	private Request request;
	
	public void setUp() throws Exception {
		request = new TestRequest(new URL("http://foo"),
			"/foo", new Hashtable());
	}
	
	public void testSimpleRequest() throws Throwable {
		Stage factory = new TestDocumentFactory(
				"This is a test document", new Hashtable());
		Request request = new TestRequest(new URL("http://foo"), 
				"/foo", new Hashtable());
		StandardRequestHandler srh = new StandardRequestHandler(
				"test", request, false, factory, null);
		srh.handleRequest();
		
		RequestHandler retrier = doRetryRequest(request, factory);
		Document doc = retrier.getDocument();
		assertEquals("This is a test document", doc.toString());
	}
	
	/** 
	 * Tests that nowhere in the code contains synchronized or blocking
	 * sections which would hold up subsequent requests (e.g. HttpClient
	 * appears to do so).
	 */
	
	public void testEndToEndReentrancy() throws Throwable {
		Stage factory = new StandardDocumentFactory();

		
		URL testUrl = new URL(
				WebUtils.getApplicationUrl() + 
				"/test/delay_first_hit.jsp?delayNext=1");
		Request realRequest = new TestRequest(testUrl, "/loband", 
				new Hashtable());
		Document doc = doRetryRequest(realRequest, factory).getDocument();
		assertEquals("Wrong output from script", 
				"\n\n\n\n\n\nDelaying next hit.", 
				doc.toString().replaceAll("\\r", ""));
		
		testUrl = new URL(
				WebUtils.getApplicationUrl() +
				"/test/delay_first_hit.jsp");
		realRequest = new TestRequest(testUrl, "/loband", 
				new Hashtable());

		long startTime = System.currentTimeMillis();
		doc = doRetryRequest(realRequest, factory).getDocument();
		long elapsedTime = System.currentTimeMillis() - startTime;
		
		assertEquals("Wrong output from test page", 
				"\n\n\n\n\n\nReturned immediately.",
                doc.toString().replaceAll("\\r", ""));
		
		assertTrue("Response came back too quickly: " +
				"first hit succeeded when it should have failed", 
				elapsedTime > 1000);

		assertTrue("Response came back too slowly: " +
				"second thread blocked by first", elapsedTime < 10000);
	}

	interface Response {
		Document respond() throws AidbaseException;
		boolean hit();
	}

	class BlockResponse implements Response {
		private boolean hit = false;
		public boolean hit() { return hit; }

		public Document respond() throws AidbaseException {
			hit = true;
			while (!Thread.interrupted()) {
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					// do nothing
				}
			}
			return null; 
		}
	}

	class DelayResponse implements Response {
		private final int delay;
		private final Response next;
		
		public DelayResponse(int delay, Response next) {
			this.delay = delay;
			this.next  = next;
		}
		
		private boolean hit = false;
		public boolean hit() { return hit; }

		public Document respond() throws AidbaseException {
			hit = true;
            /*
			System.out.println(Thread.currentThread().getName() + " waiting for "
					+delay+" ms");
			*/
            
			long deadline = System.currentTimeMillis() + delay;
			long remainder = deadline - System.currentTimeMillis();
			
			while (remainder > 0) {
				try {
					Thread.sleep(remainder);
				} catch (InterruptedException e) {
					System.err.println(e);
				}
				remainder = deadline - System.currentTimeMillis(); 
			}
		
			return next.respond(); 
		}
	}
	
	class NormalResponse implements Response {
		private final String document;
		public NormalResponse(String document) {
			this.document = document;
		}

		private boolean hit = false;
		public boolean hit() { return hit; }

		public Document respond() {
			hit = true;
			return new HtmlDocument(document, new Hashtable());
		}
	}

	class FailResponse implements Response {
		private final AidbaseException exception;
		public FailResponse(AidbaseException exception) {
			this.exception = exception;
		}

		private boolean hit = false;
		public boolean hit() { return hit; }

		public Document respond() throws AidbaseException {
			hit = true;
			throw exception;
		}
	}

	class AwkwardTestFactory implements Stage {
		private final Response[] responses;
		
		private int try_num = 0;
		
		public AwkwardTestFactory(Response[] responses) {
			this.responses = new Response [responses.length];
			System.arraycopy(responses, 0, this.responses, 0, responses.length);
		}
		
		public Document service (Request request) throws AidbaseException {
			int this_try = try_num++;
			if (this_try >= responses.length)
				this_try  = responses.length - 1;
			Response r = responses[this_try];
			return r.respond();
		}
		
		public String getName() { return "AwkwardTestFactory"; }
		public Stage getNextStage() { return null; }
		public void setNextStage(Stage next) { 
			throw new UnsupportedOperationException(""); 
		}
	}
	
	public void testDocumentShouldBeRetrievedWhenRequestFailsFourTimesThenPasses() 
	throws Throwable {
		Response failNow = new FailResponse(
				new InternalFaultException("deliberate"));
		Response passNow = new NormalResponse(
				"hello world");
		Response[] responses = new Response[]{
				failNow, failNow, failNow, passNow
		};
		
		Stage factory = new AwkwardTestFactory(responses); 
		RequestHandler retrier = doRetryRequest(request, factory);
		Document doc = retrier.getDocument();
		assertEquals("RetryHandler didn't retry enough times " +
                "to get the correct response",
                "hello world", doc.toString());
	}

	public void testDocumentShouldFailWhenRequestFailsFiveTimes() 
	throws Throwable {
		InternalFaultException result = new InternalFaultException("deliberate");
		Response failNow = new FailResponse(result);
		Response passNow = new NormalResponse(
				"hello world");
		Response[] responses = new Response[]{
				failNow, failNow, failNow, failNow, failNow, passNow
		};
		
		Stage factory = new AwkwardTestFactory(responses); 
		boolean thrown = false;
		
		try {
			doRetryRequest(request, factory);
		} catch (Exception e) {
			assertEquals(result, e);
			thrown = true;
		}
		assertTrue("expected exception not thrown", thrown);
	}

	public void testOneThreadThrowingExceptionShouldNotPreventReturningDocument()
	throws Throwable {
		Response passNow = new NormalResponse("hello world");
		Response failNow = new FailResponse(new DocumentException("deliberate"));
		Response waitForever = new BlockResponse();
		Response [] responses = new Response [] {
				waitForever, failNow, passNow
		};
		Stage factory = new AwkwardTestFactory(responses); 
		RequestHandler retrier = doRetryRequest(request, factory);
		Document doc = retrier.getDocument();
		assertEquals("RetryHandler didn't retry enough times " +
                "to get the correct response", 
                "hello world", doc.toString());
	}

	public void testExceptionShouldBeReturnedIfNoThreadsSucceeded()
	throws Throwable {
		DocumentException result = new DocumentException("deliberate");
		Response failNow = new FailResponse(result);
		Response waitForever = new BlockResponse();
		Response [] responses = new Response [] {
				waitForever, failNow, waitForever
		};
		Stage factory = new AwkwardTestFactory(responses); 
		boolean thrown = false;
		
		try {
			doRetryRequest(request, factory);
		} catch (Exception e) {
			assertEquals(result, e);
			thrown = true;
		}
		assertTrue("expected exception not thrown", thrown);
	}

	public void testDocumentShouldBeRetrievedWhenRequestTimesOutThenPasses() 
	throws Throwable {
		Response waitForever = new BlockResponse();
		Response passNow = new NormalResponse(
				"hello world");
		Response[] responses = new Response[]{
				waitForever, waitForever, passNow
		};
		
		Stage factory = new AwkwardTestFactory(responses); 
		RequestHandler retrier = doRetryRequest(request, factory);
		Document doc = retrier.getDocument();
		assertEquals("hello world", doc.toString());
	}

	public void testDocumentShouldFailWhenRequestTimesOutEveryTime() 
	throws Throwable {
		Response waitForever = new BlockResponse();
		Response passNow = new NormalResponse(
				"hello world");
		Response[] responses = new Response[]{
				waitForever, waitForever, waitForever, waitForever, 
				waitForever, passNow
		};
		
		Stage factory = new AwkwardTestFactory(responses); 
		boolean thrown = false;

		try {
			doRetryRequest(request, factory);
		} catch (Exception e) {
			assertEquals(DocumentException.class, e.getClass());
			thrown = true;
		}
		
		assertTrue("expected timeout exception not thrown", thrown);
	}
	
	public void testShouldReturnWhicheverThreadReturnsFirst() 
	throws Throwable {
		Response [] responses = new Response[3];
		int deadline = Utils.getMaxTimeoutMillis();
		
		for (int i = 0; i < responses.length; i++) {
			// thread number i will pass this time
			
			for (int j = 0; j < responses.length; j++) {
				Response finalResponse = new NormalResponse("thread "+j);
				int delay = deadline;
				if (i == j) {
					// ensure that thread i finishes first
					delay = 500;
				}
				responses[j] = new DelayResponse(delay, finalResponse);
			}

			// run a request
			Stage factory = new AwkwardTestFactory(responses);
			Document doc = doRetryRequest(request, factory).getDocument();
			
			// check that all threads were hit
			for (int j = 0; j < i; j++) {
				assertTrue("Thread "+j+" was never hit", responses[j].hit());
			}
			// check that the correct one responded
			assertEquals("Answer came from "+doc.toString()+
					" instead of thread "+i, "thread "+i, doc.toString());
		}
	}
	
	public static void main(String[] args) {
		junit.textui.TestRunner.run(RetryHandlerTests.class);
	}

    public static Test suite() {
        return new TestSuite(RetryHandlerTests.class);
    }
    
    public static class StaticHelper {
    	public static boolean delayNext = false;
    }
}
