/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.offline.unit;

import java.io.ByteArrayInputStream;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;
import org.aidworld.aidbase.webui.util.WebUtils;
import org.apache.log4j.Logger;

import com.meterware.httpunit.HttpInternalErrorException;
import com.meterware.httpunit.UploadFileSpec;
import com.meterware.httpunit.WebForm;

/**
 * A test case for the SimpleHtmlTransformer. It runs a simple test
 * HTML document through the transformer, and checks the output document.
 */

public class SimpleHtmlPipelineTest extends BasedTest{

    private Logger LOG = Logger.getLogger (SimpleHtmlPipelineTest.class);
    
	/**
	 * Standard JUnit constructor.
	 */
	public SimpleHtmlPipelineTest(String name) {
		super(name);
	}

	public void testFileUploadShouldWork() throws Exception
	{
		String reader = new String("Hello world");
		ByteArrayInputStream input = new ByteArrayInputStream(reader.getBytes());
		
		response = getThroughAidbase(WebUtils.getApplicationUrl()+"/test/file_upload.jsp");
		WebForm form = response.getFormWithName("upload");
		
		UploadFileSpec[] fspec = {new UploadFileSpec("file_upload",input,"text/plain")};
		
		form.setParameter("test",fspec);
		form.submit();
		response = conversation.getCurrentPage();
		
		assertTrue("File not uploaded",response.getText().indexOf("Upload_") != -1);
	}
    	
	public void testUnsupportedProtocolsShouldBeBlocked() throws Exception {
		boolean exceptionThrown = false;
		
		try {
			response = getThroughAidbase("ftp://foo.com");
		} catch (HttpInternalErrorException e) {
			exceptionThrown = true;
		}
		
		assertTrue("Didn't Block FTP Protocol", exceptionThrown);
		
		exceptionThrown = false;
		
		try {
			response = getThroughAidbase("irc://ircserver.com/#test");
		} catch (HttpInternalErrorException e) {
			exceptionThrown = true;
		}

		assertTrue("Didn't Block IRC Protocol", exceptionThrown);
	}
	
	public void testPostMethod() throws Exception{
		response = getThroughAidbase(WebUtils.getApplicationUrl()+"/test/postDataTest.jsp");
		
		WebForm testForm = response.getFormWithName("testform");
		testForm.setParameter("bigparam", "badger");
		testForm.submit();
		
		response = conversation.getCurrentPage();
		assertFalse("Method changed to GET",
				response.getText().indexOf("GET instead of POST!") !=-1);
	}	
		
	/**
	 * Standard JUnit method to build a suite of tests consisting of
	 * all test methods within this class.
	 */
    public static Test suite() {
        return new TestSuite(SimpleHtmlPipelineTest.class);
    }
}