package org.aidworld.aidbase.offline.unit;

import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;
import org.aidworld.aidbase.util.Utils;
import org.aidworld.aidbase.webui.util.WebUtils;

import com.meterware.httpunit.HttpInternalErrorException;

public class TimeoutTests extends BasedTest{
	
	public static TestSuite suite()
	{
		return new TestSuite(TimeoutTests.class);
	}

	public void testTimeoutDuringHeaders() throws Exception {
		int timeoutMillis = Utils.getMaxTimeoutMillis();
		int timeoutSecs   = timeoutMillis / 1000;
		
		assertTrue("Cannot test when timeout is zero (no timeout)", 
				timeoutSecs != 0);
		String testPageUrlBase = WebUtils.getApplicationUrl() + "/test/delay.jsp";
		String testPageUrl = null;
		
		try {
			testPageUrl = testPageUrlBase + "?delay=" + (timeoutSecs - 1);
			response = getThroughAidbase(testPageUrl);
		} catch (HttpInternalErrorException e) {
			fail("Page timed out too early: "+testPageUrl);
		}
		
		assertEquals("\n\n\nWaited for "+(timeoutSecs-1)+" seconds.", 
				response.getText().replaceAll("\\r", ""));
		
		/**
		 * Deliberately try to cause an exception by requesting a page
		 * which takes too long to load, and check that the exception
		 * is actually thrown.
		 */
		
		boolean exception = false;
		
		try {
			testPageUrl = testPageUrlBase + "?delay=" + (timeoutSecs + 1);
			response = getThroughAidbase(testPageUrl);
		} catch (HttpInternalErrorException e) {
			exception = true;
		}
		
		assertTrue("Request did not time out as expected after "+
				(timeoutSecs + 1) + " seconds: "+testPageUrl, exception);
	}
	
	/* TODO: fix the timeouts so that retries occur after the io timeout.
	 * This will make the previous test succeed and should render the next one
	 * worthless.
	 */
	
	public void testIOTimeout() throws Exception {
		int timeoutMillis = Utils.getIoTimeoutMillis();
		int timeoutSecs   = timeoutMillis / 1000;
		
		assertTrue("Cannot test when timeout is zero (no timeout)", 
				timeoutSecs != 0);
		String testPageUrlBase = getAidbaseUrl() + "/test/delay.jsp";
		String testPageUrl = testPageUrlBase + "?delay=" + (timeoutSecs - 1);

		response = getThroughAidbase(testPageUrl);
		assertEquals("\n\n\nWaited for "+(timeoutSecs-1)+" seconds.", 
				response.getText().replaceAll("\\r", ""));
		
		testPageUrl = testPageUrlBase + "?delay=" + (timeoutSecs + 1);

		boolean exception = false;
		try {
			response = getThroughAidbase(testPageUrl);
		} catch (HttpInternalErrorException e) {
			exception = true;
		}
		assertTrue("Request did not time out as expected after "+
				(timeoutSecs + 1) + " seconds: "+testPageUrl, exception);
	}
}
