package org.aidworld.aidbase.offline.unit;

import java.io.IOException;
import java.net.URL;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;
import org.aidworld.aidbase.core.Version;
import org.aidworld.aidbase.webui.util.WebUtils;
import org.xml.sax.SAXException;

import com.meterware.httpunit.ClientProperties;
import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.HttpUnitOptions;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebResponse;

/**
 * Tests concerning URLs (such as URL rewritting and corrections)
 * should be put in this class. Eventhough, some of these tests
 * such as those concerning handling of javascript, might be found
 * elsewhere.
 * 
 * @author Sebastien
 *
 */
public class URLTests extends BasedTest {
    private static String aidbaseUrl = null;
    private static String head = null;
    
    public void setUp() throws Exception {
        super.setUp();
    
        head = "<head>" + WebUtils.getPageHeadHtml()
        + "<title>" + Version.getShortNameHtml() + " </title></head>";

        aidbaseUrl = WebUtils.getApplicationUrl(); 
        
        conversation = new WebConversation();
        request = new GetMethodWebRequest
            (aidbaseUrl + "/main");
        response = conversation.getResponse( request );
      
        HttpUnitOptions.setExceptionsThrownOnScriptError(false);
        ClientProperties clientProperties = conversation.getClientProperties();
        clientProperties.setAutoRedirect(true);
        clientProperties.setAutoRefresh(true);
        clientProperties.setAcceptGzip(false);
        clientProperties.setUserAgent("Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7) Gecko/20040803 Firefox/0.9.3");
        //HttpUnitOptions.setScriptingEnabled(false);   
    }

    public static Test suite() {
        return new TestSuite(URLTests.class );
    }

    public static void main(String args[]) {
        junit.textui.TestRunner.run( suite() );
    }

	public void testDoubleHttpIsRemoved() throws Exception 
	{	
		String url = "http://" + getAidbaseUrl() + "/instruction.jsp";
		WebResponse response = getThroughAidbase(url);
        if (response.getText().indexOf("Instructions")==-1)
        {
            System.out.println(response.getText());
            assertTrue(response.getText().indexOf("Instructions")!=-1);
        }
	}

	/*
	 * tests that links on the testpage are rewritten to the desired 
	 * form. At the moment that is the domain in reverse with / 
	 * instead of the . then a %20 then the path
	 */
	public void testLinkRewriting() throws Exception
	{
		response = getTestPageThroughAidbase();
		
		String base = WebUtils.getContextPath() + WebUtils.getRealFilterServletPath();
		
		String page = response.getText();
		
		assertTrue("Links not being rewritten properly1", page.indexOf(base + "/uk/co/google/www")>-1);
		assertTrue("Links not being rewritten properly2", page.indexOf(base + "/com/cnn")>-1);
		assertTrue("Links not being rewritten properly3", page.indexOf(base + "/org/slashdot")>-1);
		assertTrue("Links not being rewritten properly4", page.indexOf(base + "/org/aidworld/www")>-1);
		assertTrue("Links not being rewritten properly5", page.indexOf(base + "/uk/co/bbc/news")>-1);
		assertTrue("Links not being rewritten properly6", page.indexOf(base + "/com/google/maps")>-1);
		assertTrue("Links not being rewritten properly7", page.indexOf(base + "/com/google/b/a")>-1);
		assertTrue("Links not being rewritten properly8", page.indexOf(base + "/uk/co/bbc")>-1);
		assertTrue("Links not being rewritten properly9", page.indexOf(base + "/uk/co/bbc/www/%20/news/as")>-1);
		assertTrue("Links not being rewritten properly10", page.indexOf(base + "/uk/co/google/www")>-1);
		assertTrue("Links not being rewritten properly11", page.indexOf(base + "/uk/co/google/www")>-1);
		assertTrue("Links not being rewritten properly12", page.indexOf(base + "/uk/co/bbc/www/%20/%20")>-1);
		assertTrue("Uppercase Https links not being rewritten properly", response.getText().indexOf("https://wiki.aidworld.org")!=-1);
    	assertTrue("Uppercase Http links not being rewritten properly", response.getText().indexOf(base +"/com/uppercasehttp/www")!=-1);
    	assertTrue("Https links not being rewritten properly", response.getText().indexOf("https://wiki.aidworld.org")!=-1);
    	assertTrue("Https links not being rewritten properly", response.getText().indexOf("https://wiki2.aidworld.org")!=-1);
  	}

	public void testHttpsLinksRewrittenProperly() throws Exception
	{
    	response = getThroughAidbase(WebUtils.getApplicationUrl()+"/test/test.html"); 
    	assertTrue("Https links rewrites failed", response.getText().indexOf("https://wiki.aidworld.org")!=-1);
    	assertTrue("Https links rewrites in form failed", response.getText().indexOf("https://wiki2.aidworld.org")!=-1);
    }

	/**
     * Where the website contains more ..'s than there are path components,
     * remove the extra ..'s, don't send them back to the browser which will
     * start stripping off the server hostname, Loband servlet path, etc.
     * @throws Exception if anything goes wrong
     * @author chris
     */
    public void testShouldRewriteRelativeDotDotLinksProperly() throws Exception
    {
        URL baseUrl = new URL("http://foobar.baz.info/some/path/");
        
        String input = "<html><body><a href=\"whee\">whee</a>" +
                "</body></html>";
        String expected = "<html><head>" + WebUtils.getPageHeadHtml()
                + "<title>" + Version.getShortNameHtml() + " </title></head><body>"
                + WebUtils.getPageHeader(baseUrl.toString())
                + "<a href=\"/loband/filter/info/baz/foobar/%20/some/path/whee\">whee</a>"
                + WebUtils.getPageFooter()
                + "</body></html>";
        assertEquals("test 1 failed", getFilteredHtmlString(input, baseUrl), expected);

        input = "<html><body><a href=\"../whee\">whee</a>" +
                "</body></html>";
        expected = "<html><head>" + WebUtils.getPageHeadHtml()
                + "<title>" + Version.getShortNameHtml() + " </title></head><body>"
                + WebUtils.getPageHeader(baseUrl.toString())
                + "<a href=\"/loband/filter/info/baz/foobar/%20/some/whee\">whee</a>"
                + WebUtils.getPageFooter(false) 
                + "</body></html>";
        assertEquals("test 2 failed", getFilteredHtmlString(input, baseUrl), expected);

        input = "<html><body><a href=\"../../whee\">whee</a>" +
                "</body></html>";
        expected = "<html><head>" + WebUtils.getPageHeadHtml()
                + "<title>" + Version.getShortNameHtml() + " </title></head><body>"
                + WebUtils.getPageHeader(baseUrl.toString())
                + "<a href=\"/loband/filter/info/baz/foobar/%20/whee\">whee</a>"
                + WebUtils.getPageFooter(false)
                + "</body></html>";
        assertEquals("test 3 failed", getFilteredHtmlString(input, baseUrl), expected);

        input = "<html><body><a href=\"../../../whee\">whee</a>" +
                "</body></html>";
        expected = "<html><head>" + WebUtils.getPageHeadHtml()
                + "<title>" + Version.getShortNameHtml() + " </title></head><body>"
                + WebUtils.getPageHeader(baseUrl.toString())
                + "<a href=\"/loband/filter/info/baz/foobar/%20/whee\">whee</a>"
                + WebUtils.getPageFooter(false)
                + "</body></html>";
        assertEquals("test 4 failed", getFilteredHtmlString(input, baseUrl), expected);
    }
    
    public void testHrefUrlRewritten() throws Exception {
        String input = "<html><body><a href=\"http://foo/mypage.htm\">blah</a>" +
            "<a href=\"/mypage.htm\">blah</a></body></html>"+
            "<a href=\"https://foo/mypage.htm\">blah</a></body></html>";
        
        String expected = "<html>"+head+"<body>" 
            + WebUtils.getPageHeader("http://foo") 
            + "<a href=\""+ WebUtils.getContextPath() + WebUtils.getRealFilterServletPath()
            + "/foo/%20/mypage.htm\">blah</a>"
            + "<a href=\""+ WebUtils.getContextPath() + WebUtils.getRealFilterServletPath()
            +"/foo/%20/mypage.htm\">blah</a>"
            + "<a href=\"https://foo/mypage.htm"
            //+ "<a href=\""+ WebUtils.getApplicationUrlSSL() + WebUtils.getRealFilterServletPath()
            //+"/foo/%20/mypage.htm?"
            //+ WebUtils.getAidbaseSecurityParam() + "=yes"
            + "\">blah</a>"
            + WebUtils.getPageFooter(false)
            + "</body></html>";
        
        assertEquals("href url wrongly rewritten",
                getFilteredHtmlString(input), expected);
    }  

    public void testInternalLinkAreNotRewritten() throws Exception {
        response = getThroughAidbase(aidbaseUrl
                + "/test/test.html");
        assertTrue("Internal link rewritten, it shouldn't. It wastes bandwidth",
                response.getText().indexOf("\"#internal_link\"") != -1);
    }

    public void testLinksContainingPipeCharactersShouldWork() throws Exception {
        WebResponse response = getThroughAidbase(aidbaseUrl
                + "/test/test.html?a|badger");
        assertTrue("Pipes caused problems...", response.getText().indexOf(
                "escaped absolute path") == -1);
    }

    public void testEmptyLinkRemoval() throws Exception {

        response = getTestPageThroughAidbase();
        assertFalse("EmptyLinks are not removed as expected.", 
                response.getText().indexOf("href=\"\">This is an Empty Link.</a>") != -1);
    }

    /**
     * check for URL passing scripts and make them external
     * this should fix webguide and all other script based
     * URL passers for whom a href is just too easy... :)
     * note that URLs automatically end if an & is present.
     * note that this is a hack to get around an old bbc-specific
     * problem, and should probably now be removed, since it surely
     * breaks interoperability with other scripts.
     *
     * handle fully qualified links
     * Modify url.    not completed yet... there are many cases left.
     */
    public void testFullyQualifiedLinkModification() throws Exception {
        
        response = getTestPageThroughAidbase();
        
        String checkstr= WebUtils.getContextPath()  + WebUtils.getRealFilterServletPath()
         + "/com/cnn/www\">CNN</a>";
        if (response.getText().indexOf(checkstr) == -1)
        {
            System.out.println(response.getText());
            fail("FullQualifiedLink of CNN is not modified as expected");
        }
        
        checkstr= WebUtils.getContextPath() + WebUtils.getRealFilterServletPath()
         + "/com/microsoft/www/%20/intro.htm#Game\">Play games</a>";
        
        if (response.getText().indexOf(checkstr) == -1)
        {
            System.out.println(response.getText());
            fail("FullQualifiedLink of Microsoft is not modified as expected");
        }
    }

    /**
     * now the rest of them
     * Keep mailto and ftp  etc... 
     * 
     * @throws Exception if the page request failed for some reason
     */
    
    public void testOtherLinkModification() throws Exception {
        response= getTestPageThroughAidbase();

        assertTrue("Ftp is not remained as expected.", 
                response.getText().indexOf("href=\"ftp://server/path/filename.ext\">") != -1);
        
        assertTrue("Mailto is not remained as expected.", 
                response.getText().indexOf("href=\"mailto:info@aidworld.org\">" +
                        "Send mail to AidWorld</a>") != -1);
        
        assertTrue("Irc link is not remained as expected.",
                response.getText().indexOf("irc://ircserver.com/#badger") != -1);


    }

    /**
     * Deal with any "text-only" links that reference the filter location
     * Keep <a href> because jumping in the same page.
     * 
     * @throws Exception if the page request failed for some reason
     */
    public void testTextOnlyLinkModification() throws Exception
    {
        response = getTestPageThroughAidbase();

        assertFalse("Links that already go through aidbase " +
                "are being modified.", 
        response.getText().indexOf("http://www.aidbase.org/page?_ab_url=" +
                "http://www.reliefweb.int") == -1);
    }

    /**
     * Deals with # links inside the document. They should be left the same by
     * the transformation 
     * @throws Exception
     */

    public void testBookmarkLinkModification() throws Exception 
    {
        response = getTestPageThroughAidbase();
        
        assertTrue("Internal Links are not remained as expected."
                , response.getText().indexOf("<a href=\"#C4\">Go to Chapter 4</a>") != -1); 
        assertTrue("Internal Links are not remained as expected.",response.getText().indexOf("<a  name=\"C4\">Chapter 4</a>") != -1);  
    }

    public void testBlankURLGoesToMainPage() throws Exception
    {
        conversation.setExceptionsThrownOnErrorStatus(false);
        response = getThroughAidbase("");
        
        assertTrue("Blank URL in aidbar not redirected to main page", response.getText().indexOf(WebUtils.getMainPageTestString())>-1);
    
        conversation.setExceptionsThrownOnErrorStatus(true);
    }

    public void testHttpGetsCaughtByLoband() throws Exception 
    {
        conversation.setExceptionsThrownOnErrorStatus(false);
        response = getThroughAidbase("http");
        
        assertTrue("Entering http in loband is not handled well", response.getText().indexOf(WebUtils.getFileNotFoundPageTestString())>-1);
    
        conversation.setExceptionsThrownOnErrorStatus(true);
    }
    
    public void testHttpColonFlashFlashGetsRedirectedToMainPage() throws Exception
    {
        conversation.setExceptionsThrownOnErrorStatus(false);
        response = getThroughAidbase("http://");
        
        assertTrue("http:// in aidbar not redirected to main page", response.getText().indexOf(WebUtils.getMainPageTestString())>-1);
    
        conversation.setExceptionsThrownOnErrorStatus(true);        
    }

    //test if all spaces have been removed in the input url
    public void testSpacesInUrlShouldBeRemoved() throws Exception {
        response = getThroughAidbase("   " + aidbaseUrl
                + "/test /test2.html    ");
        assertTrue("Spaces in the test URL are not removed",
                response.getText().indexOf("A file to test loband - 2") != 1);
    }
    
    public void testCallsToGoLobandServerAreProperlyMadeOne()
    {
        String originalUrl = "http://www.alpha.beta.org:5783/abcd/:497/z?ab=:23";
        String calledUrl = "";
        try
        {
            calledUrl = WebUtils.getLobandCallUrl(originalUrl, "");
        } catch (IOException e) {}
        assertTrue(calledUrl, calledUrl.indexOf("org/beta/alpha/www/%20/abcd/:497/z") != -1
                && calledUrl.indexOf(WebUtils.getAidbasePortParam() + "=5783") != -1);
    }
    //test URL starts with HTTP gets caught by loband
    public void testUpperCaseHttpGetsCaught()
    {
        String originalUrl = "HTTP://www.uppercasehttp.com";
        String originalUrl2 = "HTTPS://www.uppercasehttps.com";
        
        String calledUrl = "";
        String calledUrl2 = "";
        try
        {
            calledUrl = WebUtils.getLobandCallUrl(originalUrl, "");
            calledUrl2 = WebUtils.getLobandCallUrl(originalUrl2, "");
        } catch (IOException e) {}
        assertTrue(calledUrl, calledUrl.indexOf("com/uppercasehttp/www") != -1);
        assertTrue(calledUrl2, calledUrl2.indexOf("https://www.uppercasehttps.com") != -1);
    }
    
    public void testCallsToGoLobandServerAreProperlyMadeTwo()
    {
        String originalUrl = "http://www.alpha.org/abcd?ab=:23";
        String calledUrl = "";
        try
        {
            calledUrl = WebUtils.getLobandCallUrl(originalUrl, "_ab_type=binary");
        } catch (IOException e) {}
        assertTrue(calledUrl, calledUrl.indexOf("org/alpha/www/%20/abcd?") != -1
                && calledUrl.indexOf("_ab_type=binary") != -1);
    }
    
    public void testBaseRemovedAndHrefUrlRewritten() throws Exception {
        String input = "<html><body><base href=\"http://blah.com\" />" +
                "<a href=\"mypage.htm\">blah</a></body></html>";
        
        String expected = "<html>"+head+"<body>" 
            + WebUtils.getPageHeader("http://foo") 
            + "<a href=\"" + WebUtils.getContextPath()
            + WebUtils.getRealFilterServletPath()
            + "/com/blah/%20/mypage.htm\">blah</a>"
            + WebUtils.getPageFooter(false)
            + "</body></html>"; 
        
        assertEquals(getFilteredHtmlString(input), expected);
    }
    
    public void testUnproxyableLinksUnchanged() throws Exception {
        String input = "<html><body>" + 
            "<a href=\"mailto:jonny@leroy.com\">blah</a>" + 
            "<a href=\"ftp://blah.foo.bar/baz\">blah</a>" + 
            "<a href=\"#xxx\">blah</a>" + 
            "<a href=\"http://www.aidbase.org\">blah</a>" + 
            "</body></html>";
        
        String expected = "<html>"+head+"<body>" 
            + WebUtils.getPageHeader("http://foo") 
            + "<a href=\"mailto:jonny@leroy.com\">blah</a>"  
            + "<a href=\"ftp://blah.foo.bar/baz\">blah</a>"  
            + "<a href=\"#xxx\">blah</a>" 
            + "<a href=\"http://www.aidbase.org\">blah</a>"  
            + WebUtils.getPageFooter(false)
            + "</body></html>";
        
        assertEquals(getFilteredHtmlString(input), expected);
    }
    
    
    /**
     * test on bad URL links, make sure the pages with invalid 
     * URL links shown instead of showing an error page
     * 
     * @throws Exception
     */
    public void testBadUrlLinksDoNotCauseException() throws Exception
    {
    	
    	try {
			response=getThroughAidbase(WebUtils.getApplicationUrl() + "/test/badurllinks.html");
		} catch (Exception e) {
			fail("Bad URL links cause an exception to be thrown");
		} 
    	
    	
    }
    
    /*
    public void testInvalidUrlChangedToAlert() throws Exception {
        String input = "<html><body><a href=\"xxx://blah\">blah</a></body></html>";
        
        String expected = "<html><body>" 
            + WebUtils.getPageHeader("http://foo") 
            + "<a href=\"javascript:alert(\"Link was invalid: xxx://blah\")\">blah</a>" + 
            WebUtils.PAGE_FOOTER;
        
        internalTestHelper(input, expected);
    } 
    */ 
}
