/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.offline.unit.pipeline;

import org.aidworld.aidbase.document.Document;
import org.aidworld.aidbase.error.InternalFaultException;
import org.aidworld.aidbase.filter.BasicFilter;
import org.aidworld.aidbase.request.Request;

/**
 * This filter is used to test the pipeline. It adds its name to the request 
 * and to the document.
 */

public class DummyFilter extends BasicFilter {
    private String name = null;
    private boolean throwRequestException = false;
    private boolean throwResponseException = false;

    public DummyFilter(String name) {
        this.name = name;
    }
    
    public String getName() {
        return name;
    }
    
    public String toString() {
        return name;
    }
    
    /**
     * If true an exception will be thrown during the request processing.
     * @param requestException
     */
    public void setRequestExceptionThrown(boolean requestException) {
        throwRequestException = requestException;
    }
    
    /**
     * If true an exception will be thrown during the request processing.
     * @return
     */
    public boolean isRequestExceptionThrown() {
        return throwRequestException;
    }
    
    /**
     * If true an exception will be thrown during the response processing.
     * @param responseException
     */
    public void setResponseExceptionThrown(boolean responseException) {
        throwResponseException = responseException;
    }
    
    /**
     * If true an exception will be thrown during the response processing.
     * @return
     */
    public boolean isResponseExceptionThrown() {
        return throwResponseException;
    }

    /* (non-Javadoc)
     * @see org.aidworld.aidbase.core.Filter#filter(org.aidworld.aidbase.core.Request, org.aidworld.aidbase.core.Server)
     */
    public Document filter(Request request)
        throws Exception {

        TestFilterRecorder.addFilterRequestRecord(this);
        // TestFilterRecorder.addServerRecord(next);
        
        if(throwRequestException) {
            throw new InternalFaultException(
            		"Test Exception during Request handling");
        }

        Document document = super.service(request);
        
        TestFilterRecorder.addFilterResponseRecord(this);

        if(throwResponseException) {
            throw new InternalFaultException(
            		"Test Exception during Response handling");
        }
        
        return document;    
    }
    

}
