/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.offline.unit.pipeline;

import java.util.Iterator;
import java.util.Vector;

import org.aidworld.aidbase.filter.Stage;

/**
 * This class is used to record the function of the test filters. It is a 
 * central place where the filters can record their execution and the test
 * case can read it back. It is a static class.
 */

public class TestFilterRecorder {
    private static Vector filterRequestRecord = new Vector();
    private static Vector filterResponseRecord = new Vector();
    private static Vector servers = new Vector();
    

    /**
     * Clears all recirded results held by the TestFilterRecorder. This should
     * be done before running each test. 
     */
    public static void clearRecord() {
        filterRequestRecord = new Vector();
        filterResponseRecord = new Vector();
        servers = new Vector();
    }
    
    
    /**
     * Adds a filter to the record of filters handling requests.
     * Also adds the remaining pipeline to the record of pipelines.
     * @param filter
     */
    public static void addFilterRequestRecord(Stage stage) {
        filterRequestRecord.add(stage);
    }
        
    
    /**
     * Returns an array of the filters handling requests, 
     * in the order in which they were run.
     * @return
     */
    public static Stage[] getFilterRequestRecord() {
        return (Stage[])filterRequestRecord.toArray(new Stage[] {});
    }
    
    
    /**
     * Adds a record of server.
     * @param pipeline
     */
    public static void addServerRecord(Stage server) {
        servers.add(server);
    }
    
    
    /**
     * Returns an array of all servers in the order in which
     * the filters that contain them were run. Should be in sync with the
     * filter request handling array.
     * @return
     */
    public static Stage[] getServerRecord() {
        return (Stage[])servers.toArray(new Stage[] {});
    }
    
    
    /**
     * Adds a filter to the response record in the order in which the responses
     * are generated. This should be in the reverse order of the request 
     * record.
     * @param filter
     */
    public static void addFilterResponseRecord(Stage stage) {
        filterResponseRecord.add(stage);
    }
    
    
    /**
     * Returns an array of filters in the order in which they generated
     * responses. This should be the reverse order of the request record.
     * @return
     */
    public static Stage[] getFilterResponseRecord() {
        return (Stage[])filterResponseRecord.toArray(new Stage[] {});
    }
    
    public static String printRecord() {
        StringBuffer buff = new StringBuffer();
        buff.append("TestFilterRecord\n");
        buff.append("Request Filters: \n");
        
        Iterator iter = filterRequestRecord.iterator();
        while(iter.hasNext()) {
            buff.append(iter.next().toString());
            buff.append("\n"); 
        }
        
        buff.append("\n");
        buff.append("Response Filters: \n");
        
        iter = filterResponseRecord.iterator();
        while(iter.hasNext()) {
            buff.append(iter.next().toString());
            buff.append("\n"); 
        }
        
        buff.append("\n");
        buff.append("Pipelines: \n");
        iter = servers.iterator();
        while(iter.hasNext()) {
            buff.append(iter.next().toString());
            buff.append("\n"); 
        }
        
        return buff.toString();            
    }
    

}
