/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.performance;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.DocumentSpec;
import org.aidworld.aidbase.factory.StandardDocumentFactory;
import org.aidworld.aidbase.filter.SimplifyPipeline;
import org.aidworld.aidbase.offline.unit.OfflineDocumentSpecs;
import org.aidworld.aidbase.request.Request;
import org.aidworld.aidbase.request.TestRequest;
import org.aidworld.aidbase.webui.util.WebUtils;
import org.apache.log4j.Logger;

/**
 * MemoryLeakTest runs to check for memory leaks by watching
 * the growth of the heap after running some pages through the pipeline.
 */

public class MemoryLeakTest extends TestCase{

    private static final Logger LOG = Logger.getLogger(MemoryLeakTest.class);

	static final String RESOURCE_PREFIX = "/data/";

	public static void main(String args[]) {
        junit.textui.TestRunner.run( suite() );
        
	}
	
	/**
	* Standard JUnit method to build a suite of tests consisting of
	* all test methods within this class.
	*/
	public static Test suite() {
	    return new TestSuite(MemoryLeakTest.class);
	}
  
    /**
     * Standard JUnit constructor.
     */
    public MemoryLeakTest(String name) {
        super(name);
    }
    
	public void testReliefwebHome () throws Exception
	{
		DocumentSpec spec = OfflineDocumentSpecs.getReliefwebHomeSpec();
		filterDocument(spec, 50, 0.01F);
		System.gc();
	}
	
	/*public void testAllSpecs() throws Exception {
	    DocumentSpec [] specs = DocumentSpecs.allSpecs;
	    for (int i = 0; i < specs.length; i++) {
	        filterDocument(specs[i], 5, 0.5F);
	    }
	}*/

	/* TODO reinstate these tests */
	/*
	 * another line
	public void testBBCnews () throws MalformedURLException, AidbaseException  {
		
			DocumentSpec spec = new DocumentSpec(
			        "BBC Home Page (static)",
					// URL of page to test
					"file:/bbcnexfp.html",
					// Array of strings expected to be present in the filtered output
					new String[] {
						"http://www.bbc.co.uk",
						"The Windsor and Spencer families"
					},
					// Array of strings expected to be absent
					new String[] {}                 
			);
		
			filterDocument(spec, 5, 0.5F);
			System.gc();
		}

	public void testCNNcom () throws MalformedURLException, AidbaseException  {
		
			DocumentSpec spec = new DocumentSpec(
			        "CNN Home Page (static)",
					// URL of page to test
					"file:/CNN.com.html",
					// Array of strings expected to be present in the filtered output
					new String[] {
						"cnn.com",
						"Democratic presidential nominee"
					},
					// Array of strings expected to be absent
					new String[] {}                 
			);
		
			filterDocument(spec, 5, 0.5F);
			System.gc();
		}
		*/






	private void filterDocument(DocumentSpec docSpec, int repeatRequest, float maxGrowth)
	throws Exception 
	{
		maxGrowth = maxGrowth * 1000000;
			     
		String urlString = docSpec.getUrlString();
		URL url = new URL(urlString);
		
		StandardDocumentFactory sdf = new StandardDocumentFactory();
		SimplifyPipeline pipeline = new SimplifyPipeline(sdf);
		
		Map map = new HashMap();
		map.put(WebUtils.getAidbaseUrlParam(), urlString);
		
		//build the request for the pipeline	
		Request request = new TestRequest(url, "http://www.aidbase.org", map);
		
		//stabilise the pipeline with requests before doing a mem growth test.
		for (int i = 0; i < 1; i++) {
			pipeline.service(request);
		}
			
        LOG.debug("Start of  " +this.getName().toString() + " :");
        
		//just do a dump before we do anything serious.
		grabHeapInfo();
		
		System.gc();
		Runtime r = Runtime.getRuntime();
		long startHeapSize = r.totalMemory();
		long startUsedSize = r.totalMemory()-r.freeMemory();
		
		for (int i = 0; i < repeatRequest; i++){			
			pipeline.service(request);
			System.runFinalization();
			System.gc();
		}
		
		System.gc();
		long endHeapSize = r.totalMemory();
		long endUsedSize = r.totalMemory()-r.freeMemory();
        LOG.debug("End of  " +this.getName().toString() + " :");
		grabHeapInfo();
		
		//calculate memory Growth after running the above pipeline "repeatRequest" times.
		long memoryGrowth = endHeapSize - startHeapSize;
		
        LOG.debug("Memory Growth = "+memoryGrowth);
        LOG.debug("Start Total Memory = "+startHeapSize);
        LOG.debug("Start Used Memory = "+startUsedSize);
        LOG.debug("End Total Memory = "+endHeapSize);
        LOG.debug("End Used Memory = "+endUsedSize);
		
		assertTrue("Memory leak outside performance bounds: actual "+memoryGrowth+
			" Bytes but maximum was "+maxGrowth+" Bytes", memoryGrowth < maxGrowth);
			
		//docSpec.verifyDocument(filteredDoc);
		//Document filteredDoc = pipeline.service(request);
		/* wait forever for profiler */
		/*
		while(true){
			try{
				
			
				Thread.sleep(1000);
			}
			catch(InterruptedException ex){}
			
		}
		*/	
	}

	public void grabHeapInfo() {
	    // Get current size of heap in bytes
	    long heapSize = Runtime.getRuntime().totalMemory();
	    LOG.debug("Heap Size is " + heapSize);
	    
	    // Get maximum size of heap in bytes. The heap cannot grow beyond this size.
	    // Any attempt will result in an OutOfMemoryException.
	    long heapMaxSize = Runtime.getRuntime().maxMemory();
	    LOG.debug("Max Heap Size is "+ heapMaxSize);
	    
	    // Get amount of free memory within the heap in bytes. This size will increase
	    // after garbage collection and decrease as new objects are created.
	    long heapFreeSize = Runtime.getRuntime().freeMemory();
	    LOG.debug("Heap Free Size is " + heapFreeSize);
	}
}
