/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */
package org.aidworld.aidbase.performance;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.log4j.Logger;

import com.meterware.httpunit.ClientProperties;
import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.HttpUnitOptions;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;

public class PerformanceComparisonTest extends TestCase{
	
    protected WebConversation 	conversation;
    protected WebRequest 		request;
    protected WebResponse 		response;
    
    //String abParserUrl = "http://localhost:8080/aidbase-parser/";
    String abParserUrl = "http://stage.aidworld.org/aidbase/";
    
	String pageBit = "page?_ab_url=";
    
    private Logger LOG = Logger.getLogger(PerformanceComparisonTest.class);
	
	/*
	 * @see TestCase#setUp()
	 */
	public void setUp() throws Exception 
	{		
		conversation = new WebConversation();
	    HttpUnitOptions.setExceptionsThrownOnScriptError(false);
	    ClientProperties clientProperties = conversation.getClientProperties();
		clientProperties.setAutoRedirect(true);
		clientProperties.setAutoRefresh(true);
	}
	
	public static void main(String args[]) {
		junit.textui.TestRunner.run( suite() );
	}
	
	public static Test suite() {
		return new TestSuite( PerformanceComparisonTest.class );
	}

class LogLoader {
		
		String inputLine;
						
		public LogLoader () {
			super();
		}
		
	    public Vector getLogVector () throws FileNotFoundException{
	    	
	    	File inputFile = new File("/home/tariq/biglistofurls");
		    FileReader in = new FileReader(inputFile);
		    BufferedReader logfile = new BufferedReader(in);
		    Vector logVector = new Vector();
		 
	    	
	    try {
	    	
	    	while (logfile.readLine() != null){
	    		
	    		inputLine = logfile.readLine();
	    		
	    		if (inputLine.indexOf("yahoo")==-1)
	    		{
	    			logVector.addElement(inputLine);
	    		}
	    	 }
	    	}
	    
	    catch ( IOException error ) {
            LOG.error( "Error reading file: " + error );
	    }
	    
	    try {
	        in.close();
	    }
	    catch ( IOException error ) {
	        LOG.error( "Couldn't close file reader: " + error );
	    }
	    
	    return logVector;


	    
}
}


class TestThread extends Thread {
		// private DocumentSpec spec;
		private long timeTotal;
		private long timePerRun;
		private long numPasses;
		Vector urls;
		String parser_choice;
		int offset;
		int requests;
		
		public TestThread(Vector urls, String parser_choice) {
			this.urls = urls;
			this.parser_choice = parser_choice;
			this.offset = 0;
			this.requests = 10;
		}
		
		public TestThread(Vector urls, String parser_choice, int offset, int requests) {
			this.urls = urls;
			this.parser_choice = parser_choice;
			this.offset = offset;
			this.requests = requests;
		}
		
	    public void run() {
	    	try {
		        long startMillis = System.currentTimeMillis();
				int index;
				int urlsSize = urls.size();
				
		        for(int urlcount = 0; urlcount < requests; urlcount++)
				{
					index = urlcount + offset;
					if (index >= urlsSize)
					{
						index-= urlsSize;
					}
					//String url = (String) urls.get(urlcount);
					String url = "http://www.google.com";
					//System.out.println(url);
					request = new GetMethodWebRequest(abParserUrl + pageBit + url);
	 
	   				response = conversation.getResource(request);
					++numPasses;
				}
		        
		        
				long endMillis = System.currentTimeMillis();
				     
		        this.timeTotal  = endMillis - startMillis;
		        this.timePerRun = this.timeTotal / numPasses;
	    		} 
	    		
	    		catch (Exception e) {
	    		// force test failure
	    		TestCase.assertTrue(e.toString(), false);
	    	}
	    }
	    
	    public long getTimeTotal() {
	    	return this.timeTotal;
	    }

	    public long getTimePerRun() {
	    	return this.timePerRun;
	    }

	    public long getNumPasses() {
	    	return this.numPasses;
	    }
	}
	

class TestResult {
		public int numThreads;
		public double overallTimeTaken,
			threadTimeTaken,
			avgTimePerPass;
	}
	
	public TestResult testThreads(int numThreads, String parser_choice) throws FileNotFoundException {
		
		//int numURLS = 10;
		
		
		TestThread [] threads = new TestThread[numThreads];
		
//		String [] TestURLS = {"www.bbc.co.uk",
//							"www.google.com",		
//							"www.zmag.org/weluser.htm",
//							"www.guardian.co.uk",
//							"www.cambridge.indymedia.org.uk"
//												};
		LogLoader ll = new LogLoader();
			
		
		
		for (int i = 0; i < numThreads; i++) {
			// int randomOffset; //TODO use random offset?
			int offset =i * (ll.getLogVector().size() / numThreads);  
			threads[i] = new TestThread(ll.getLogVector(), parser_choice, offset, 5 );
		}

		long overallStartMillis = System.currentTimeMillis();            

		long[] startTimes= new long [numThreads];
		
		for (int i = 0; i < numThreads; i++) {
			threads[i].start();
			startTimes[i]=System.currentTimeMillis();
		}
		
		
		
		boolean allFinished = false;

		while (!allFinished) {
			allFinished = true;
			for (int i = 0; i < numThreads; i++) {
				if (threads[i].isAlive()) {
					
//					if (System.currentTimeMillis() - startTimes[i] >10000)
//					{
//						System.out.println("oh bugger, I'm going to have to kill it\n");
//						threads[i].stop();
//					}
//					else
//					{
						allFinished = false;
//					}
					
				}
			}
		}

		long overallEndMillis = System.currentTimeMillis();            
		long overallTimeTaken = overallEndMillis - overallStartMillis;
		
		long threadTimeTaken = 0;
		long totalNumPasses  = 0;
		
		for (int i = 0; i < numThreads; i++) {
			threadTimeTaken += threads[i].getTimeTotal();
			totalNumPasses  += threads[i].getNumPasses();
		}

		long avgTimePerPass = threadTimeTaken / totalNumPasses;
		
		/*
		System.out.println("Number of threads  : " +numThreads);
		System.out.println("Overall time taken : "+
				(double)overallTimeTaken/1000+" s");
		System.out.println("Thread  time taken : "+
				(double)threadTimeTaken/1000+" s");
		System.out.println("Mean time per pass : "+
				(double)avgTimePerPass/1000+" ms");
		System.out.println();
				*/
		
		TestResult result = new TestResult();
		result.numThreads = numThreads;
		result.overallTimeTaken = (double)overallTimeTaken / 1000;
		result.threadTimeTaken  = (double)threadTimeTaken  / 1000;
		result.avgTimePerPass   = (double)avgTimePerPass   / 1000;
		
		return result;
	}
	
	

	
	public void testConcurrency() throws FileNotFoundException {
		// warm up
		//testThreads(10);

		Vector results = new Vector();
	
			
        LOG.debug(
					"Number of threads, " +
					"Overall time taken, " +
					"Thread  time taken, " +
					"Mean time per pass");
	
			double sum_x = 0.0, sum_y = 0.0;
			int n = 0;
			
//			String parser_choice;
//			// parser or xsl
//			if (parserSwitch==0)
//			parser_choice = "xsl";
//			else
//			parser_choice = "parser";
			
            LOG.debug("PARSER CHOICE = " + "Parser");
			
			for (int numThreads = 1; numThreads <= 5; numThreads++) {
				TestResult result = testThreads(numThreads, "parser"); 
				results.add(result);
                LOG.debug(
						result.numThreads + ", " +
						result.overallTimeTaken + ", " +
						result.threadTimeTaken  + ", " +
						result.avgTimePerPass);
				sum_x += result.numThreads;
				sum_y += result.avgTimePerPass;
				n++;
			}
			
			/* We 'plot' the number of threads (x) against average time per
			 * filter pass (y), and calculate a linear regression. The gradient
			 * of the average line, expressed as a fraction of the average time
			 * per pass, represents the marginal cost of another simultaneous
			 * thread. If this is zero, then extra threads are free, and we
			 * have infinite concurrency. If it's one, then extra threads cost
			 * as much as the first, and we have no benefit from concurrency.
			 * If greater than one, concurrency is worse than no concurrency.
			 * Our target is less than 0.5.    
			 */ 
			
			double mean_x = sum_x / n;
			double mean_y = sum_y / n;
			
			double gradient_sum = 0.0;
			int    gradient_n   = 0;
			
			for (int i = 0; i < results.size(); i++) {
				TestResult r = (TestResult)(results.get(i));
				/* if r.numThreads for this result equals the mean, then
				 * the gradient is undefined (division by zero) and this
				 * point must be ignored. 
				 */
				if (r.numThreads != mean_x) {
					double gradient = (r.avgTimePerPass - mean_y) / 
						(r.numThreads - mean_x);
					gradient_sum += gradient;
					gradient_n++;
				}
			}
			
			double gradient_avg = gradient_sum / gradient_n;
			double marginal_cost_factor = gradient_avg / mean_y; 
			
            LOG.debug("Marginal cost factor is "+marginal_cost_factor);
			
			assertTrue("Not enough concurrency: marginal cost factor "+
					"is greater than 0.5 ("+marginal_cost_factor+")",
					marginal_cost_factor < 0.5);
		}
	




/*
 
 	public void testLogLoaderWorks() throws IOException {
		
		
		LogLoader ll = new LogLoader();
		
		
		System.out.println (ll.getLogVector().size());
		System.out.println (ll.getLogVector().toString());
		
		assertTrue("Should Never Read This" , 1==1);
		
	}
	public void testXSLFrontPageLoads() throws IOException {
		
		request = new GetMethodWebRequest(abXSLUrl + "/main");
		response = conversation.getResource(request);
		//System.out.println("XSLT RESPONSE : \n" + response.getText());
		assertTrue("XSL Response Too Short!" , response.getText().length() > 1);
				
	}
	
	public void testGoogleThroughXSLFrontPageLoads() throws IOException {
		
		request = new GetMethodWebRequest(abXSLUrl + pageBit + "http://www.google.com");
		response = conversation.getResource(request);
		//System.out.println("XSLT RESPONSE : \n" + response.getText());
		assertTrue("Google Through XSL Response Too Short!" , response.getText().length() > 1);
				
	}
	
	public void testGoogleThroughParserFrontPageLoads() throws IOException {
		
		request = new GetMethodWebRequest(abParserUrl + pageBit + "http://www.google.com");
		response = conversation.getResource(request);
		//System.out.println("PARSER RESPONSE : \n" + response.getText());
		assertTrue("Google Through Parser Response Too Short!" , response.getText().length() > 1);
				
	}
*/	

}
