/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.performance;

import java.net.URL;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.Timer;
import org.aidworld.aidbase.document.Document;
import org.aidworld.aidbase.factory.StandardDocumentFactory;
import org.aidworld.aidbase.filter.Pipeline;
import org.aidworld.aidbase.filter.ScriptDocumentPipeline;
import org.aidworld.aidbase.filter.SimplifyPipeline;
import org.aidworld.aidbase.request.TestRequest;
import org.aidworld.aidbase.webui.util.WebUtils;

import com.meterware.httpunit.ClientProperties;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebResponse;

public class TestPerformance extends TestCase {

	/**
	 * This test will pass only if you have unabled gzip
	 * compression in Tomcat (see server.xml).
	 * 
	 * @throws Exception
	 */
	public void testAidbaseShouldUseGzipCompression() throws Exception{
		WebConversation conv = new WebConversation();
		ClientProperties properties = conv.getClientProperties();
		properties.setUserAgent(
				"Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7) " +
				"Gecko/20040803 Firefox/0.9.3");
		properties.setAcceptGzip(true);
		WebResponse resp = conv.getResponse(WebUtils.getApplicationUrl()
				+ "/test/gzip.jsp");
		String zipped = resp.getHeaderField("Content-Encoding");
		assertEquals("Aidbase didn't use gzip compression","gzip", zipped);
		assertTrue("Test page didn't retrieve remote page using gzip",
				resp.getText().indexOf("gzip") !=-1);		
	}//*/
	
	/**
	 * These tests pass reference documents through the pipeline to 
	 * check the output, and in particular that the performance is acceptable
	 * (less than 4 seconds per page filtered). They load local files.
	 */
	public void testRunBenchmark() throws Exception {
        StandardDocumentFactory  sdf = new StandardDocumentFactory();
        Pipeline nsp = new SimplifyPipeline(sdf);

    	Benchmarker bm = new Benchmarker();
    	
    	String [] columns = new String[]{ "Parser" };

    	for (int run = 0; run < 10; run++) {
	    	for (int col = 0; col < columns.length; col++) {
	    		Pipeline p = null;
	    		
	    		if (columns[col].equals("Parser")) 
	    			p = nsp;
	    		/*
	    		else if (columns[col].equals("NoParser"))
	    			p = nsp;
	    			*/
	    		
	    		/*
	        	TestRequest req = new TestRequest(new URL("http://www.reliefweb.int"),
	        			"http://www.aidbase.org", null);
	        			*/
	        	TestRequest req = new TestRequest(new URL("file:/www.yahoo.co.uk.html"),
	        			"http://www.aidbase.org", null);
	    		Document filteredDoc = p.service(req);
	    		/*
	    		assertTrue(filteredDoc.toString().indexOf(
	    				"ReliefWeb All rights reserved") != -1);
	    				*/
	    		assertTrue(filteredDoc.toString().indexOf(
	    				"Privacy Policy") != -1);
	    		
	    		// if (run >= 10) 
	    		bm.addRun(req.getTimer(), columns[col]);
	    	}
    	}
    	
    	bm.writeStats();
    }

    public static Test suite() {
        return new TestSuite(TestPerformance.class);
    }
    
    public void testJavascriptFilterSpeed() throws Exception {
    	StandardDocumentFactory  sdf = new StandardDocumentFactory();        
    	Pipeline p = new SimplifyPipeline(sdf);
    	TestRequest req = new TestRequest(new URL("file:/javascript/"),
    			"http://www.aidbase.org", null);
		Document filteredDoc = p.service(req);
		
		String docstring = filteredDoc.toString();
		
		String[] files = docstring.split("\n");
		Pipeline sp = new ScriptDocumentPipeline(sdf);
		
		for (int i=0; i<files.length; i++) {
		//	System.out.println(i + "  " + files[i]);
			long ct = System.currentTimeMillis();
			
			req = new TestRequest(new URL("file:/javascript/"+files[i]),
					"http://www.aidbase.org", null);					
			filteredDoc = sp.service(req);						
			long ct2 = System.currentTimeMillis() - ct;
			
			assertTrue("Document took longer than 1s to process", ct2 < 1000);
		}		
    }
}

class Benchmarker {
	class Stats {
		int n = 0;
		long sum_x = 0, sum_x_x = 0;
		long min = -1, max = -1;
		
		void add(long time) {
			n++;
			sum_x += time;
			sum_x_x += time * time;
			if (min > time || min == -1)
				min = time;
			if (max < time || max == -1)
				max = time;
		}
		
		int    num() { return n; }
		long   min() { return min; }
		long   max() { return max; }
		double avg() { return (double)sum_x / n; }
	}
	
	private Map allStats = new Hashtable();
	
	public void addRun(Timer newTimer, String column) {
		Map newTimes = newTimer.getTimes();
		Iterator it = newTimes.keySet().iterator();
		long totalTime = 0;
		
		Map col = (Map)( allStats.get(column) );
		if (col == null) {
			col = new Hashtable();
			allStats.put(column, col);
		}
		
		while (it.hasNext()) {
			String task = (String)( it.next() );
			Long   time = (Long)( newTimes.get(task) );
			
			Stats stats = (Stats)( col.get(task) );
			if (stats == null) {
				stats = new Stats();
				col.put(task, stats);
			}
			
			stats.add(time.longValue());
			totalTime += time.longValue();
		}
		
		Stats total = (Stats)( col.get("total") );
		if (total == null) {
			total = new Stats();
			col.put("total", total);
		}
		total.add(totalTime);
	}
	
	public void writeStats() {
		Vector sortedColumns = new Vector(allStats.keySet());
		Collections.sort(sortedColumns);
		
		Hashtable allKeys = new Hashtable();
		{
			Iterator ci = sortedColumns.iterator();
			while (ci.hasNext()) {
				String column = (String)( ci.next() );
				//System.out.print("\t" + column);
				
				Map col = (Map)( allStats.get(column) );
				Iterator ri = col.keySet().iterator();
				while (ri.hasNext()) {
					String row = (String)( ri.next() );
					allKeys.put(row, "1");
				}
			}
		}
		//System.out.println();
		
		Vector sortedKeys = new Vector(allKeys.keySet());
		Collections.sort(sortedKeys);

		Iterator ri = sortedKeys.iterator();
		while (ri.hasNext()) {
			String task = (String)( ri.next() );
			//System.out.print(task);
			
			Iterator ci = sortedColumns.iterator();
			while (ci.hasNext()) {	
				String column = (String)( ci.next() );
				Map    col    = (Map)   ( allStats.get(column) );
				Stats  stats  = (Stats) ( col.get(task) );
				//if (stats == null)
					//System.out.print("\t-/-/-");
				//else
					//System.out.print("\t" + stats.min() + "/" + stats.avg() + "/" + stats.max());
			}
			
			//System.out.println();
		}
	}
}