package org.aidworld.aidbase.specialsites;

import java.io.IOException;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;
import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

import com.meterware.httpunit.WebLink;

public class BBCTests extends BasedTest{

    private Logger LOG = Logger.getLogger (BBCTests.class); 
    
	public static Test suite() {
		return new TestSuite( BBCTests.class );
	}
	
	/**
	 * A basic liveness check verifies that the BBC front page downloads correctly
	 * and passes if the string bbc.co.uk is in the html response.
	 * @throws Exception
	 */
    public void testBBCFrontPageWorks()
    throws IOException, SAXException
    {
        response = getThroughAidbase("http://www.bbc.co.uk");
        assertTrue(response.getText().indexOf("bbc.co.uk") != -1);
    }

    /**
     * Beware! This test checks that clicking on the link 'Sport' on
     * the BBC web page does return an existing page. Unfortunately,
     * because of the lack of javascript support by httpunit and
     * because bbc.co.uk uses javascript to rewrite some on its links,
     * this not a guarantee that links work on a browser with
     * javascript enable.
     * 
     * @throws IOException
     * @throws SAXException
     */
    public void testLinksOnBBCPageWork()
    throws IOException, SAXException
    {
        response = getThroughAidbase("http://www.bbc.co.uk");
        WebLink newsLink = response.getLinkWith("Sport");
        newsLink.click();
        response = conversation.getCurrentPage();
        assertTrue("Links on bbc.co.uk do not work\n" + response.getText(),
                response.getResponseCode() == 200);
    }
}
