package org.aidworld.aidbase.specialsites;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;

import com.meterware.httpunit.SubmitButton;
import com.meterware.httpunit.WebForm;

public class GoogleTests extends BasedTest{

	public static Test suite() {
		return new TestSuite( GoogleTests.class );
	}
	
	public void testTopBarShouldWork() throws Exception{
		response = getThroughAidbase("http://www.google.com");
		WebForm[] forms = response.getForms();
		assertNotNull("form not found", forms[0]);
		forms[0].setParameter("_ab_url","http://www.yahoo.com");
		forms[0].submit();
		
		response = conversation.getCurrentPage();
		assertTrue("Topbar failed",
				response.getText().indexOf("Yahoo! Inc. All rights reserved.") > -1);
	}

	public void testGooglePostQuery() throws Exception {
    	response = getThroughAidbase("http://www.google.com");
    	
    	WebForm form = response.getFormWithName("f");
    	assertNotNull("Form f not found", form);
    	form.setParameter("q", "aidworld");
    	SubmitButton button = form.getSubmitButton("btnG");
    	assertNotNull("Button btnG not found", button);
    	button.click();
    	
    	response = conversation.getCurrentPage();
    	assertTrue("Google search failed", 
    			response.getText().indexOf("www.aidworld.org") != -1);
    }

    /**
     * Verifies that submitting the main page form with the 
     * url "http://www.google.com" results in a page containing
     * a link to the original URL.
     */
    public void testOriginalLinkProvidedAfterSubmitGoogleToURLForm()
    throws Exception {
        response = getThroughAidbase("http://www.google.co.uk");
        assertTrue("Missing expected text in output: "+response.getText(),
        		response.getText().indexOf("<a href=\"" +
        		"http://www.google.co.uk\" target=\"_top\">" +
        		"original</a>") != -1);
    }
    
    public void testRetrieveGoogleAdsenseSecureWebsiteShouldWork() throws Exception {
        response = getThroughAidbase(
        		"https://www.google.com/adsense");
    	assertTrue("Missing expected text in response: "+
    			response.getText(),
				response.getText().indexOf(
    			"Click Here to Apply")
				!= -1);
    }
}
