/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

/**
 * This set of end-to-end tests uses httpunit and is intended to run on a
 * machine with a deployed copy of aidbase running. They can also be run
 * against a remote copy of aidbase.
 */

package org.aidworld.aidbase.specialsites;
import java.net.URLDecoder;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;
import org.aidworld.aidbase.webui.util.WebUtils;

import com.meterware.httpunit.ClientProperties;
import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.HttpUnitOptions;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebForm;
import com.meterware.httpunit.WebLink;

public class HotmailTests extends BasedTest {
    private static String 		aidbaseUrl = null;

	/**
	 * Change the address of the request here to match the 
	 * instance of aidbase being tested.
	 */
	

	public static Test suite() {
		return new TestSuite( HotmailTests.class );
	}

	
	public void setUp() throws Exception {
		super.setUp();
	
		aidbaseUrl = WebUtils.getApplicationUrl(); 
		conversation = new WebConversation();
	    request = new GetMethodWebRequest
			(aidbaseUrl + "/main");
	    response = conversation.getResponse( request );
	    
	    HttpUnitOptions.setExceptionsThrownOnScriptError(false);
	    ClientProperties clientProperties = conversation.getClientProperties();
		clientProperties.setAutoRedirect(true);
		clientProperties.setAutoRefresh(true);
		clientProperties.setAcceptGzip(false);
		clientProperties.setUserAgent("Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.7) Gecko/20040803 Firefox/0.9.3");
	    //HttpUnitOptions.setScriptingEnabled(false);	
	}

/* TODO: "hiddenform" has disappeared from the site.
*  This needs to be researched and replaced with whatever there is there now.
*  It appeared to be a pre-redirect form on www.hotmail.com 
*/	
	
	public void logIn() throws Exception {
    	    	
    	response = getThroughAidbase("http://www.hotmail.com");
    	
    	WebForm form = response.getFormWithName("hiddenform");
    	assertNotNull(form);
        form.submit();
        response = conversation.getCurrentPage();
        
        form = response.getFormWithName("hotmail_com");
        assertNotNull(form);
        form.setParameter("passwd", "testing");
        form.setParameter("login", "aidbase@hotmail.com");
        form.setParameter("domain", "hotmail.com");
        form.submit();
        response = conversation.getCurrentPage();
	}
	
	public static void main(String args[]) {
        junit.textui.TestRunner.run( suite() );
        
	}
	
    public void testHotmailSigninPageShouldLoad() throws Exception {
    
    	response = getThroughAidbase("http://www.hotmail.com");
    	
    	
    	ClientProperties props = conversation.getClientProperties();
    	props.setAutoRedirect(true);
    	
    	WebForm form = response.getFormWithName("hiddenform");
    	
    	assertNotNull("form wasn't found",form);
        form.submit();
        response = conversation.getCurrentPage();
        //System.out.println(response.getText());
    	assertTrue(response.getText()
    			.indexOf("<title>&gt;0- Please sign in</title>") != -1);
    
    	
    
    }
    
    public void testHotmailSigninShouldWork() throws Exception {

    	logIn();
           	
    	assertTrue(response.getText()
    			.indexOf("onclick=\"G('/cgi-bin/compose?');") != -1);
    }

    public void testHotmailSendMessageShouldWork() throws Exception {

    	logIn();
    	
    	//Go to the compose page, original is JavaScript: "javascript:G('/cgi-bin/compose?"
    	response = getThroughAidbase("http://by20fd.bay20.hotmail.msn.com" +
    			"/cgi-bin/compose?");
    	
    	//Send an email
    	response = conversation.getCurrentPage();
    	WebForm form = response.getFormWithName("composeform");
    	form.setParameter("encodedto", "aidbase@hotmail.com");
    	form.setParameter("msg", "testmail");
    	form.setParameter("subject", "Test email from HotmailTest");
    	form.submit();
    	   	
    	//Get resultpage
    	response = conversation.getCurrentPage();
        assertTrue("Message has been sent", 
        		response.getText().indexOf(
        				"Your message has been sent to:") != -1);
    	
    }
    
    /**
     * This tests if you can reach a message. This doesn't include 
     * testing the JavaScript mangling of JavaScript G: links
     * @throws Exception
     */
    
    public void testReadMessageShouldWork() throws Exception {

    	logIn();

        //Go to the inbox folder
        WebLink link = response.getLinkWith("Mail"); // find the link
        String url = link.getURLString().replaceAll("hmhome", "HoTMaiL");
        url=url.substring(21);
        url=URLDecoder.decode(url,"UTF-8");
        response = getThroughAidbase(url);
        response = conversation.getCurrentPage();
        
       //Get first message link 
        WebLink message = null;
       for( int i = 0; i < response.getLinks().length; i++){
       	if(response.getLinks()[i].getURLString().indexOf("g?msg=MSG") != -1){
       		message = response.getLinks()[i];
       		break;
       	}
       }
       
       //Replace the JavaScript G: link with valid aidbase link
       String nonJavaScriptLink = 
	   		message.getURLString().substring(14, message.getURLString().length() - 2);
       nonJavaScriptLink = URLDecoder.decode(nonJavaScriptLink,"UTF-8");
       
       nonJavaScriptLink = "http://by20fd.bay20.hotmail.msn.com" + nonJavaScriptLink;
       
       response = getThroughAidbase(nonJavaScriptLink);
       response = conversation.getCurrentPage();
       
       assertTrue("Message not reached", response.getText().indexOf(
       		"<title>&gt;0- MSN Hotmail - Message</title>") != -1 );
    	
    }
    
    public void testDeleteMessageShouldWork() throws Exception {

    	logIn();
        //Go to the inbox folder
        WebLink link = response.getLinkWith("Mail"); // find the link
        String url = link.getURLString().replaceAll("hmhome", "HoTMaiL");
        url=url.substring(21);
        url=URLDecoder.decode(url,"UTF-8");
        response = getThroughAidbase(url);
        response = conversation.getCurrentPage();
        
        
        WebForm form = response.getFormWithName("hotmail");
        String[] parameterNames = form.getParameterNames();
        String messageId = "";
        
        for(int i= 0; i< parameterNames.length; i++){
        	if((parameterNames[i].indexOf("MSG") != -1)){
           		form.toggleCheckbox(parameterNames[i]);
        		//form.setCheckbox(parameterNames[i], true);
           		messageId = parameterNames[i];
        		break;
        	}
        }
        
        form.setParameter("_ab_url", "http://by20fd.bay20.hotmail." +
        		"msn.com/cgi-bin/HoTMaiL");
        form.setParameter("_HMaction", "delete");
        form.submit();
        
        response = conversation.getCurrentPage();
        
       
       assertTrue("Message hasn't been deleted", 
       		response.getText().indexOf(messageId) == -1);
       
    	
    }
  
}


