/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

package org.aidworld.aidbase.specialsites;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.DocumentSpec;


/**
 * The DocumentSpecs class encapsulates some
 * specifications for 'standard' test pages, making it
 * easier to see which tests are available, and to add
 * new test pages to potentially a large number of
 * unit tests.
 */

public class OnlineDocumentSpecs extends TestCase {
	public static final String RESOURCE_PREFIX = "/data/"; 





	public final DocumentSpec [] allSpecs = new DocumentSpec [] {
        
		    new DocumentSpec(
		            "Indymedia Mexico",
		            "http://mexico.indymedia.org",
		            new String[] { "Indymedia Mexico" },
		            new String[] {}
	        ),
			
		    new DocumentSpec(
		            "Reliefweb Contacts Page",
		            "http://www.reliefweb.int/w/rwc.nsf/contactDirHome?OpenForm",
		            new String[] { "ReliefWeb", 
		                           "Show Organization",
		                           "Government",
		                           "Home",
		                           "View by Alpha" },
		            new String[] {}
	        ),

		    new DocumentSpec(
		            "BBC News (live)",
		            "http://news.bbc.co.uk",
		            new String[] { "BBC",
		                           "Low Graphics version",
		                           "Education",
		                           "Arabic",
		                           "About the BBC"
								   },
		            new String[] {}
	    	),

		    new DocumentSpec(
		            "BBC News Text Only (live)",
		            "http://news.bbc.co.uk/text_only.stm",
		            new String[] { "BBC",
		                           "BBC NEWS",
		                           "Technology",
		                           "SEARCH BBC NEWS",
		                           "STORIES" },
		            new String[] {}
	        ),


		    new DocumentSpec(
		            "Aidworld Home Page (live)",
		            "http://www.aidworld.org/",
		            new String[] { "aidworld", 
		                           "taking the world wide web world-wide",
		                           "site map" },
		            new String[] {}
	    	),

		    new DocumentSpec(
		            "Google.com Home Page (live)",
		            "http://www.google.com",
		            new String[] { "Web",
		                           "Images",
		                           "Advertising&nbsp;Program",
		                           "Language Tools",
		                           "Advanced Search" },
		            new String[] {}
	    	),

		    new DocumentSpec(
		            "Google Answers (live)",
		            "http://answers.google.com",
		            new String[] { "Ask a question. Set your price. Get your answer.",
		                           "Step 1 - Enter your Question.",
		                           "Google Home" },
		            new String[] {}
	     	),

		    new DocumentSpec(
		            "Google France Home Page (live)",
		            "http://www.google.fr",
		            new String[] { "Web",
		                           "Images",
		                           "Groupes",
		                           "Pages francophones"
								  },  //can you do that?
		                           
		            new String[] {"Pr??f??rences",
		                          "Recherche avanc??e",
		                          "Publicit??"}
	     	),

		    new DocumentSpec(
		            "Google Japan Home Page (live)",
		            "http://www.google.jp",
		            new String[] { "Google.com in English" },
		            new String[] {}
	     	),

		    new DocumentSpec(
		            "Google Germany Home Page (live)",
		            "http://www.google.de",
		            new String[] { "Seiten auf Deutsch",
		                           "Seiten aus Deutschland"
		                          },
		                           
		            new String[] {}
	     	),

		    new DocumentSpec(
		            "Google Korea Home Page (live)",
		            "http://www.google.co.kr",
		            new String[] { "Google.com in English"
		                          },
		                           
		            new String[] {}
	     	),

		    new DocumentSpec(
		            "Google China Home Page (live)",
		            "http://www.yahoo.com.cn",
		            new String[] { "Yahoo! Holdings (Hong Kong) Ltd" },
		            new String[] {}
	     	),

		    new DocumentSpec(
		            "Yahoo Hong Kong Home Page",
		            "http://www.yahoo.com.hk",
		            new String[] { "Yahoo! Holdings (Hong Kong) Limited." },
		            new String[] {}
	     	),

		    new DocumentSpec(
		            "OCHA Register Page (live)",
		            "http://www.reliefweb.int/cr/",
		            new String[] { "frames"
		            				},
		            new String[] { }
	     	),

		    new DocumentSpec(
		            "FAO Home Page (live)",
		            "http://www.fao.org/",
		            new String[] { "FAO",
		                           "FOOD AND AGRICULTURE ORGANIZATION OF THE UNITED NATIONS",
		                           "International Partners",
		                           "Search" },
		            new String[] {}
	        ),

		    new DocumentSpec(
		            "WHO Home Page (live)",
		            "http://www.who.int",
		            new String[] { "WHO",
		                           "Home",
		                           "EMERGENCIES",
		                           "World Health Organization",
		                           "Other UN Sites" },
		            new String[] {}
	        ),

		    new DocumentSpec(
		            "UNDP Home Page (live)",
		            "http://www.undp.org/",
		            new String[] { "Regions",
		                           "Focus Areas",
		                           "Terms of Use" },
		            new String[] {}
	        ),

		    new DocumentSpec(
		            "ICRC Home Page (live)",
		            "http://www.icrc.org",
		            new String[] { "ICRC",
		                           "International Committee of the Red Cross",
		                           "The mission of the ICRC" },
		            new String[] {}
	        ),

		    new DocumentSpec(
		            "IFRC Home Page (live)",
		            "http://www.ifrc.org/",
		            new String[] { "Red Cross Red Crescent",
								  },
		            new String[] {}
	        ),

		    new DocumentSpec(
		            "Red Cross UK Home Page (live)",
		            "http://www.redcross.org.uk",
		            new String[] { "Red Cross",
		                           "British Red Cross",
		                           "EC2Y 9AL",
		                           "Registered charity" },
		            new String[] {}
	        ),

			/* 
			 * If this test fails, check your boot classpath.
			 * On JDK 1.4, xalan.jar, xml-apis.jar and xercesImpl.jar 
			 * need to be added before rt.jar, in order to override
			 * the built-in version of Xalan in Java 1.4.
			 */
		    new DocumentSpec(
		            "UNHCR Home Page (live)",
		            "http://www.unhcr.ch",
		            new String[] { "UNHCR",
		                           "Today's date",
		                           "All Rights Reserved" },
		            new String[] {}
	        ),

		    new DocumentSpec(
		            "UNHCR Home Page 2 (live)",
		            "http://www.unhcr.ch/cgi-bin/texis/vtx/home",
			        new String[] { "UNHCR",
			                       "Today's date",
			                       "All Rights Reserved" },
			            new String[] {}
		    ),

			/*
			 * Site is al javascript
		    new DocumentSpec(
		            "WFP Home Page (live)",
		            "http://www.wfp.org/",
		            new String[] { "WFP",
		                           "Donor" },
		            new String[] {}
	        ),
	        */
		    new DocumentSpec(
		            "UNICEF Home Page (live)",
		            "http://www.unicef.org/",
		            new String[] { "UNICEF",
		                           "About UNICEF",
		                           "UN links",
		                           "Press Centre" //This is in an ALT tag 
		                          },
		            new String[] {}
	        ),

		    new DocumentSpec(
		            "IOM Home Page (live)",
		            "http://www.iom.ch/",
		            new String[] { "IOM",
		                           //"What we do",   //navigation in javascript
		                           "info@iom.int" },
		            new String[] {}
	        ),

		    new DocumentSpec(
		            "UNFPA Home Page",
		            "http://www.unfpa.org/",
		            new String[] { "UNFPA",
		                           "Population Issues"
		                           },
		            new String[] {}
	        )
	};

	public static TestSuite suite() throws Exception {
	    TestSuite suite = new TestSuite();
	    OnlineDocumentSpecs specs = new OnlineDocumentSpecs();
	    DocumentSpec[] all = specs.allSpecs;
	    
	    for (int i = 0; i < all.length; i++) {
	        suite.addTest(all[i].getSuite());
	    }
	    return suite;
	}
}
