/* ------------------------------------------------------------------------
 * Aidbase simplifies HTML pages to reduce bandwidth usage, et cetera.
 * Copyright (C) 2002-2004 Aidworld Humanitarian ICT, Cambridge, UK. 
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * This software is intended to be as accessible as possible. If acceptance 
 * of this license conflicts with your interests, please contact us to 
 * discuss how we might be able to accommodate your requirements. 
 * 
 * Please look at our website, http://www.aidworld.org, to find out more 
 * about our organisation and to read our mission statement. 
 * ------------------------------------------------------------------------
 */

/**
 * This set of end-to-end tests uses httpunit and is intended to run on a
 * machine with a deployed copy of aidbase running. They can also be run
 * against a remote copy of aidbase.
 */

package org.aidworld.aidbase.specialsites;
import java.util.Properties;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.aidworld.aidbase.core.BasedTest;
import org.aidworld.aidbase.util.Utils;

import com.meterware.httpunit.WebForm;
import com.meterware.httpunit.WebLink;

public class YahooTests extends BasedTest {
	/**
	 * Tests requiring an SSL connection should be put at the end.
	 */

	
	
	public void logIn() throws Exception 
	{	    	
		response = getThroughAidbase("http://mail.yahoo.com");
		
		WebForm form = response.getFormWithName("login_form");
		assertNotNull("form not found", form);
		form.setParameter("passwd", "aidworld");
		form.setParameter("login", "aidbase2");
		form.submit();
		
		response = conversation.getCurrentPage();
	}
	
	public static Test suite() {
		return new TestSuite( YahooTests.class );
	}

    public void testBodyJavascriptShouldNotBeFiltered() throws Exception {
    	response = getThroughAidbase(
			"http://www.yahoo.com");
    	assertTrue(response.getText().indexOf(
    			"d=document;") != -1);
    }

    public void testYahooSigninPageShouldLoad() throws Exception 
	{
		response = getThroughAidbase("http://mail.yahoo.com");
		assertTrue(response.getText()
				.indexOf("Yahoo! Mail") != -1);
	}
	
	public void testYahooSigninShouldWork() throws Exception {
	
		logIn();
	    	
		assertTrue(response.getText()
				.indexOf("Mail - aidbase2@yahoo.com") != -1);
	}
	
	public void testYahooSendEmailShouldWork() throws Exception {
		logIn();
		
	    WebLink message = null;
	    for( int i = 0; i < response.getLinks().length; i++){
	    	if(response.getLinks()[i].getURLString().indexOf("Compose") != -1){
	    		message = response.getLinks()[i];
	    		break;
	    	}
	    }
	    
	    message.click();
	    response = conversation.getCurrentPage();
	    
	    WebForm form = response.getFormWithName("Compose");
	    
	    form.setParameter("To","aidbase2@yahoo.com");
	    form.setParameter("Subj","Loband test");
	    form.setParameter("Body","Loband test body");
	    form.setParameter("SEND","1");
	    form.submit();
	    response = conversation.getCurrentPage();
	    assertTrue("Message not send.",response.getText().indexOf("<b>Sent to:</b>")>-1);
	}
	
	public void testYahooReadingEmailShouldWork() throws Exception{
		logIn();
		
	    WebLink inbox = null;
	    for( int i = 0; i < response.getLinks().length; i++){
	    	if(response.getLinks()[i].getURLString().indexOf("Inbox") != -1){
	    		inbox = response.getLinks()[i];
	    		break;
	    	}
	    }
	    
	    inbox.click();
	    response = conversation.getCurrentPage();
	    
	    WebLink message = null;
	    for( int i = 0; i < response.getLinks().length; i++){
	    	if(response.getLinks()[i].getURLString().indexOf("ShowLetter") != -1){
	    		message = response.getLinks()[i];
	    		break;
	    	}
	    }
	    
	    message.click();
	    response = conversation.getCurrentPage();
	
		assertTrue("Didn't make it to the message",
				response.getText().indexOf("Reply To Sender")>-1);
	}
	
	/**
	 * Put the tests requiring SSL after this line
	 */

	/**
	 * 
	 */
	public void testAidbaseShouldFixSSLLinks() throws Exception{
    	response = getThroughAidbase("http://mail.yahoo.com/?.intl=us");
    	//System.out.println(response.getText());
    	Properties props = Utils.getAppProps();
    	String host = props.getProperty("application.host", "dev.aidworld.org");
		assertTrue("Link didn't get mangled",
    			response.getText().indexOf("https://" + host + ":") > -1);
    }
}


